/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "ExitKind.h"

#include <wtf/Assertions.h>
#include <wtf/PrintStream.h>

namespace JSC {

ASCIILiteral exitKindToString(ExitKind kind)
{
    switch (kind) {
    case ExitKindUnset:
        return ASCIILiteral("Unset");
    case BadType:
        return ASCIILiteral("BadType");
    case BadCell:
        return ASCIILiteral("BadCell");
    case BadIdent:
        return ASCIILiteral("BadIdent");
    case BadExecutable:
        return ASCIILiteral("BadExecutable");
    case BadCache:
        return ASCIILiteral("BadCache");
    case BadWeakConstantCache:
        return ASCIILiteral("BadWeakConstantCache");
    case BadIndexingType:
        return ASCIILiteral("BadIndexingType");
    case BadTypeInfoFlags:
        return ASCIILiteral("BadTypeInfoFlags");
    case Overflow:
        return ASCIILiteral("Overflow");
    case NegativeZero:
        return ASCIILiteral("NegativeZero");
    case Int52Overflow:
        return ASCIILiteral("Int52Overflow");
    case StoreToHole:
        return ASCIILiteral("StoreToHole");
    case LoadFromHole:
        return ASCIILiteral("LoadFromHole");
    case OutOfBounds:
        return ASCIILiteral("OutOfBounds");
    case InadequateCoverage:
        return ASCIILiteral("InadequateCoverage");
    case ArgumentsEscaped:
        return ASCIILiteral("ArgumentsEscaped");
    case ExoticObjectMode:
        return ASCIILiteral("ExoticObjectMode");
    case TDZFailure:
        return ASCIILiteral("TDZFailure");
    case Uncountable:
        return ASCIILiteral("Uncountable");
    case UncountableInvalidation:
        return ASCIILiteral("UncountableInvalidation");
    case WatchdogTimerFired:
        return ASCIILiteral("WatchdogTimerFired");
    case DebuggerEvent:
        return ASCIILiteral("DebuggerEvent");
    case ExceptionCheck:
        return ASCIILiteral("ExceptionCheck");
    case GenericUnwind:
        return ASCIILiteral("GenericUnwind");
    }
	ASSERT_NOT_REACHED();
	return ASCIILiteral("Unknown");
}

bool exitKindMayJettison(ExitKind kind)
{
    switch (kind) {
    case ExceptionCheck:
    case GenericUnwind:
        return false;
    default:
        return true;
    }
}

} // namespace JSC

namespace WTF {

void printInternal(PrintStream& out, JSC::ExitKind kind)
{
#ifndef NDEBUG
    out.print(exitKindToString(kind));
#endif
}

} // namespace WTF


