/*
 * Copyright (C) 2008, 2009 Apple Inc. All rights reserved.
 * Copyright (C) 2008 Cameron Zwarich <cwzwarich@uwaterloo.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <algorithm>
#include <string.h>

#include <wtf/Assertions.h>

namespace JSC {

#if ENABLE(LLINT)
#define OP_GET_BY_ID_LENGTH 9
#else
#define OP_GET_BY_ID_LENGTH 5
#endif

#if ENABLE(LLINT)
#define OP_GET_BY_ID_DIRECT_LENGTH 7
#else
#define OP_GET_BY_ID_DIRECT_LENGTH 5
#endif

#if ENABLE(LLINT)
#define OP_PUT_BY_ID_LENGTH 9
#else
#define OP_PUT_BY_ID_LENGTH 5
#endif

#if ENABLE(LLINT)
#define OP_GET_PRIVATE_NAME_LENGTH 8
#else
#define OP_GET_PRIVATE_NAME_LENGTH 5
#endif

#if ENABLE(LLINT)
#define OP_PUT_PRIVATE_NAME_LENGTH 9
#else
#define OP_PUT_PRIVATE_NAME_LENGTH 5
#endif

#define FOR_EACH_OPCODE_ID(macro) \
	macro(op_enter, 1) \
	macro(op_get_scope, 2) \
	macro(op_create_direct_arguments, 2) \
	macro(op_create_scoped_arguments, 3) \
	macro(op_create_cloned_arguments, 2) \
	macro(op_create_this, 5) \
	macro(op_get_argument, 4) \
	macro(op_argument_count, 2) \
	macro(op_to_this, 6) /* has value profiling */ \
	macro(op_check_tdz, 2) \
	\
	macro(op_new_object, 4) \
	macro(op_new_array, 5) \
	macro(op_new_array_with_size, 4) \
	macro(op_new_array_with_spread, 5) \
	macro(op_spread, 3) \
	macro(op_new_array_buffer, 4) \
	macro(op_new_regexp, 3) \
	macro(op_mov, 3) \
	\
	macro(op_not, 3) \
	macro(op_eq, 4) \
	macro(op_eq_null, 3) \
	macro(op_neq, 4) \
	macro(op_neq_null, 3) \
	macro(op_stricteq, 4) \
	macro(op_nstricteq, 4) \
	macro(op_less, 4) \
	macro(op_lesseq, 4) \
	macro(op_greater, 4) \
	macro(op_greatereq, 4) \
	macro(op_below, 4) \
	macro(op_beloweq, 4) \
	\
	macro(op_inc, 3) \
	macro(op_dec, 3) \
	macro(op_to_number, 4) /* has value profiling */ \
	macro(op_to_numeric, 4) /* has value profiling */ \
	macro(op_to_string, 3) \
	macro(op_to_object, 5) \
	macro(op_negate, 4) \
	macro(op_add, 5) \
	macro(op_mul, 5) \
	macro(op_div, 5) \
	macro(op_mod, 4) \
	macro(op_sub, 5) \
	macro(op_pow, 4) \
	\
	macro(op_lshift, 5)  /* has value profiling */ \
	macro(op_rshift, 5) /* has value profiling */ \
	macro(op_urshift, 4) \
	macro(op_unsigned, 3) \
	macro(op_bitand, 5) /* has value profiling */ \
	macro(op_bitxor, 5) /* has value profiling */ \
	macro(op_bitor, 5) /* has value profiling */ \
	macro(op_bitnot, 4) /* has value profiling */ \
	\
	macro(op_overrides_has_instance, 4) \
	macro(op_identity_with_profile, 4) \
	macro(op_instanceof, 4) \
	macro(op_instanceof_custom, 5) \
	macro(op_typeof, 3) \
	macro(op_is_empty, 3) \
	macro(op_typeof_is_undefined, 3) \
	macro(op_typeof_is_object, 3) \
	macro(op_typeof_is_function, 3) \
	macro(op_is_undefined_or_null, 3) \
	macro(op_is_boolean, 3) \
	macro(op_is_number, 3) \
	macro(op_is_big_int, 3) \
	macro(op_is_object, 3) \
	macro(op_is_callable, 3) \
	macro(op_is_constructor, 3) \
	macro(op_is_cell_with_type, 4) \
	macro(op_in, 5) \
	\
	macro(op_get_by_id_direct, OP_GET_BY_ID_DIRECT_LENGTH) \
	macro(op_get_prototype_of, 4) /* has value profiling */ \
	macro(op_try_get_by_id, 5) /* has value profiling */ \
	macro(op_get_by_id, OP_GET_BY_ID_LENGTH) /* has value profiling */ \
	macro(op_get_by_id_with_this, 6) \
	macro(op_get_array_length, OP_GET_BY_ID_LENGTH) /* has value profiling */ \
	macro(op_put_by_id, OP_PUT_BY_ID_LENGTH) \
	macro(op_put_by_id_with_this, 6) \
	macro(op_del_by_id, 5) \
	macro(op_get_by_val, 6) /* has value profiling */ \
	macro(op_get_private_name, OP_GET_PRIVATE_NAME_LENGTH) /* has value profiling */ \
	macro(op_put_private_name, OP_PUT_PRIVATE_NAME_LENGTH) \
	macro(op_set_private_brand, 3) \
	macro(op_check_private_brand, 3) \
	macro(op_has_private_name, 4) \
	macro(op_has_private_brand, 4) \
	macro(op_get_by_val_with_this, 6) \
	macro(op_put_by_val, 6) \
	macro(op_put_by_val_with_this, 6) \
	macro(op_put_by_val_direct, 6) \
	macro(op_del_by_val, 5) \
	macro(op_put_getter_by_id, 5) \
	macro(op_put_setter_by_id, 5) \
	macro(op_put_getter_setter, 6) \
	macro(op_put_getter_by_val, 5) \
	macro(op_put_setter_by_val, 5) \
	macro(op_define_data_property, 5) \
	macro(op_define_accessor_property, 6) \
	\
	macro(op_jmp, 2) \
	macro(op_jtrue, 3) \
	macro(op_jfalse, 3) \
	macro(op_jeq_null, 3) \
	macro(op_jneq_null, 3) \
	macro(op_jundefined_or_null, 3) \
	macro(op_jnundefined_or_null, 3) \
	macro(op_jneq_ptr, 5) \
	macro(op_jeq, 4) \
	macro(op_jstricteq, 4) \
	macro(op_jneq, 4) \
	macro(op_jnstricteq, 4) \
	macro(op_jless, 4) \
	macro(op_jlesseq, 4) \
	macro(op_jgreater, 4) \
	macro(op_jgreatereq, 4) \
	macro(op_jnless, 4) \
	macro(op_jnlesseq, 4) \
	macro(op_jngreater, 4) \
	macro(op_jngreatereq, 4) \
	macro(op_jbelow, 4) \
	macro(op_jbeloweq, 4) \
	\
	macro(op_loop_hint, 1) \
	\
	macro(op_switch_imm, 4) \
	macro(op_switch_char, 4) \
	macro(op_switch_string, 4) \
	\
	macro(op_new_func, 4) \
	macro(op_new_func_exp, 4) \
	macro(op_new_generator_func, 4) \
	macro(op_new_generator_func_exp, 4) \
	macro(op_new_async_func, 4) \
	macro(op_new_async_func_exp, 4) \
	macro(op_new_async_generator_func, 4) \
	macro(op_new_async_generator_func_exp, 4) \
	macro(op_set_function_name, 3) \
	macro(op_call, 9) /* has value profiling */ \
	macro(op_call_eval, 9) /* has value profiling */ \
	macro(op_call_varargs, 9) /* has value profiling */ \
	macro(op_ret, 2) \
	\
	macro(op_construct, 9) \
	macro(op_construct_varargs, 9) \
	macro(op_strcat, 4) \
	macro(op_to_primitive, 3) \
	macro(op_to_property_key, 3) \
	\
	macro(op_resolve_scope, 7) \
	macro(op_get_from_scope, 8) /* has value profiling */ \
	macro(op_put_to_scope, 7) \
	macro(op_get_from_arguments, 5) \
	macro(op_put_to_arguments, 4) \
	\
	macro(op_push_with_scope, 4) \
	macro(op_create_lexical_environment, 5) \
	macro(op_create_generator_frame_environment, 5) \
	macro(op_get_parent_scope, 3) \
	\
	macro(op_catch, 4) \
	macro(op_throw, 2) \
	macro(op_throw_static_error, 3) \
	\
	macro(op_debug, 2) \
	macro(op_profile_will_call, 2) \
	macro(op_profile_did_call, 2) \
	\
	macro(op_get_enumerable_length, 3) \
	macro(op_has_enumerable_indexed_property, 5) \
	macro(op_has_enumerable_structure_property, 5) \
	macro(op_has_enumerable_property, 4) \
	macro(op_get_direct_pname, 7) \
	macro(op_get_property_enumerator, 3) \
	macro(op_enumerator_structure_pname, 4) \
	macro(op_enumerator_generic_pname, 4) \
	macro(op_to_index_string, 3) \
	macro(op_unreachable, 1) \
	macro(op_copy_rest, 4) \
	macro(op_get_rest_length, 3) \
	macro(op_yield, 4) \
	macro(op_resolve_scope_for_hoisting_func_decl_in_eval, 4) \
	\
	macro(op_nop, 1) \
	\
	macro(op_end, 2) // end must be the last opcode in the list

#define OPCODE_ID_ENUM(opcode, length) opcode,
	typedef enum { FOR_EACH_OPCODE_ID(OPCODE_ID_ENUM) } OpcodeID;
#undef OPCODE_ID_ENUM

constexpr int maxOpcodeLength = 9;
constexpr int numOpcodeIDs = op_end + 1;

#define OPCODE_ID_LENGTHS(id, length) constexpr int id##_length = length;
	 FOR_EACH_OPCODE_ID(OPCODE_ID_LENGTHS);
#undef OPCODE_ID_LENGTHS

#define OPCODE_LENGTH(opcode) opcode##_length

#define OPCODE_ID_LENGTH_MAP(opcode, length) length,
	const unsigned opcodeLengths[numOpcodeIDs] = { FOR_EACH_OPCODE_ID(OPCODE_ID_LENGTH_MAP) };
#undef OPCODE_ID_LENGTH_MAP

#define VERIFY_OPCODE_ID(id, size) COMPILE_ASSERT(id <= op_end, ASSERT_THAT_JS_OPCODE_IDS_ARE_VALID);
	FOR_EACH_OPCODE_ID(VERIFY_OPCODE_ID);
#undef VERIFY_OPCODE_ID

#if ENABLE(COMPUTED_GOTO_OPCODES)
typedef void* Opcode;
#else
typedef OpcodeID Opcode;
#endif

extern const char* const opcodeNames[];

#if ENABLE(OPCODE_SAMPLING) || ENABLE(OPCODE_STATS)
static constexpr char PADDING_STRING[] = "                                ";
#define PADDING_STRING_LENGTH static_cast<unsigned>(strlen(PADDING_STRING))

inline const char* padOpcodeName(OpcodeID op, unsigned width)
{
	unsigned pad = width - strlen(opcodeNames[op]);
	pad = std::min(pad, PADDING_STRING_LENGTH);
	return PADDING_STRING + PADDING_STRING_LENGTH - pad;
}

#undef PADDING_STRING_LENGTH
#endif

#if ENABLE(OPCODE_STATS)

struct OpcodeStats {
	OpcodeStats();
	~OpcodeStats();
	static long long opcodeCounts[numOpcodeIDs];
	static long long opcodePairCounts[numOpcodeIDs][numOpcodeIDs];
	static int lastOpcode;

	static void recordInstruction(int opcode);
	static void resetLastInstruction();
};

#endif

inline constexpr bool isBranch(OpcodeID opcodeID)
{
	switch (opcodeID) {
	case op_jmp:
	case op_jtrue:
	case op_jfalse:
	case op_jeq_null:
	case op_jneq_null:
    case op_jundefined_or_null:
    case op_jnundefined_or_null:
	case op_jneq_ptr:
	case op_jless:
	case op_jlesseq:
	case op_jgreater:
	case op_jgreatereq:
	case op_jnless:
	case op_jnlesseq:
	case op_jngreater:
	case op_jngreatereq:
    case op_jeq:
    case op_jneq:
    case op_jstricteq:
    case op_jnstricteq:
    case op_jbelow:
    case op_jbeloweq:
	case op_switch_imm:
	case op_switch_char:
	case op_switch_string:
		return true;
	default:
		return false;
	}
}

inline constexpr bool isUnconditionalBranch(OpcodeID opcodeID)
{
	switch (opcodeID) {
	case op_jmp:
		return true;
	default:
		return false;
	}
}

inline constexpr bool isTerminal(OpcodeID opcodeID)
{
	switch (opcodeID) {
	case op_ret:
	case op_end:
    case op_unreachable:
		return true;
	default:
		return false;
	}
}

inline constexpr bool isThrow(OpcodeID opcodeID)
{
	switch (opcodeID) {
	case op_throw:
	case op_throw_static_error:
		return true;
	default:
		return false;
	}
}

} // namespace JSC
