/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#if ENABLE(JIT)

#include "AdaptiveInferredPropertyValueWatchpointBase.h"
#include "ObjectPropertyCondition.h"
#include "Watchpoint.h"
#include <wtf/Platform.h>
#include <wtf/Bag.h>
#include <wtf/FastAllocBase.h>
#include <wtf/Noncopyable.h>
#include <wtf/OwnPtr.h>
#include <wtf/PassOwnPtr.h>

namespace JSC {

class CodeBlock;
class WatchpointsOnStructureStubInfo;
struct StructureStubInfo;

class StructureTransitionStructureStubClearingWatchpoint FINAL : public Watchpoint {
    WTF_MAKE_NONCOPYABLE(StructureTransitionStructureStubClearingWatchpoint);
    WTF_MAKE_FAST_ALLOCATED;
public:
    StructureTransitionStructureStubClearingWatchpoint(
        const ObjectPropertyCondition& key,
        const WatchpointsOnStructureStubInfo& holder)
		: Watchpoint(Watchpoint::Type::StructureTransitionStructureStubClearing)
		, m_key(key)
        , m_holder(holder)
    {
    }

protected:
    void fireInternal(JSGlobalData&);

private:
    ObjectPropertyCondition m_key;
    const WatchpointsOnStructureStubInfo& m_holder;
};

class AdaptiveValueStructureStubClearingWatchpoint FINAL : public AdaptiveInferredPropertyValueWatchpointBase {
    typedef AdaptiveInferredPropertyValueWatchpointBase Base;
    WTF_MAKE_NONCOPYABLE(AdaptiveValueStructureStubClearingWatchpoint);
    WTF_MAKE_FAST_ALLOCATED;

    virtual void handleFire(VM&) OVERRIDE;

public:
    AdaptiveValueStructureStubClearingWatchpoint(const ObjectPropertyCondition& key, WatchpointsOnStructureStubInfo& holder)
        : Base(key)
        , m_holder(holder)
    {
        RELEASE_ASSERT(key.condition().kind() == PropertyCondition::Equivalence);
    }

private:
    const WatchpointsOnStructureStubInfo& m_holder;
};

class WatchpointsOnStructureStubInfo {
    WTF_MAKE_NONCOPYABLE(WatchpointsOnStructureStubInfo);
    WTF_MAKE_FAST_ALLOCATED;
public:
    WatchpointsOnStructureStubInfo(CodeBlock* codeBlock, StructureStubInfo* stubInfo)
        : m_codeBlock(codeBlock)
        , m_stubInfo(stubInfo)
    {
    }

    static void ensureReferenceAndInstallWatchpoint(
        OwnPtr<WatchpointsOnStructureStubInfo>& holderRef,
        CodeBlock*, StructureStubInfo*, const ObjectPropertyCondition& key);
    static Watchpoint* ensureReferenceAndAddWatchpoint(
		OwnPtr<WatchpointsOnStructureStubInfo>& holderRef,
        CodeBlock*, StructureStubInfo*);
    
    CodeBlock* codeBlock() const { return m_codeBlock; }
    StructureStubInfo* stubInfo() const { return m_stubInfo; }

    bool isValid(VM&) const;
    
private:
    CodeBlock* m_codeBlock;
    StructureStubInfo* m_stubInfo;
    Bag<StructureTransitionStructureStubClearingWatchpoint> m_structureTransitionStructureStubClearingWatchpoints;
    Bag<AdaptiveValueStructureStubClearingWatchpoint> m_adaptiveValueStructureStubClearingWatchpoints;
};

} // namespace JSC

#endif // ENABLE(JIT)
