/*
 *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
 *  Copyright (C) 2003-2017 Apple Inc. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#pragma once

#include "ErrorInstance.h"
#include "InternalFunction.h"
#include <wtf/Optional.h>

namespace JSC {

class ErrorPrototype;
class GetterSetter;

class ErrorConstructor : public InternalFunction {
public:
	typedef InternalFunction Base;
    static const unsigned StructureFlags = Base::StructureFlags | OverridesPut;

	static ErrorConstructor* create(VM& vm, Structure* structure, ErrorPrototype* errorPrototype, GetterSetter*)
	{
		ErrorConstructor* constructor = new (NotNull, allocateCell<ErrorConstructor>(vm.heap)) ErrorConstructor(vm, structure);
		constructor->finishCreation(vm, errorPrototype);
		return constructor;
	}

	DECLARE_INFO;

	static Structure* createStructure(JSGlobalData& globalData, JSGlobalObject* globalObject, JSValue prototype)
	{
		return Structure::create(globalData, globalObject, prototype, TypeInfo(InternalFunctionType, StructureFlags), info());
	}

    Optional<unsigned> stackTraceLimit() const { return m_stackTraceLimit; }

protected:
	void finishCreation(VM&, ErrorPrototype*);

    static bool put(JSCell*, ExecState*, PropertyName, JSValue, PutPropertySlot&);
    static bool deleteProperty(JSCell*, ExecState*, PropertyName);

private:
	ErrorConstructor(VM&, Structure*);

    Optional<unsigned> m_stackTraceLimit;
};

} // namespace JSC
