/*
 *  Copyright (C) 1999-2002 Harri Porten (porten@kde.org)
 *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
 *  Copyright (C) 2004, 2007, 2008, 2009 Apple Inc. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#include "config.h"
#include "GetterSetter.h"

#include "Error.h"
#include "Exception.h"
#include "JSGlobalObject.h"
#include "JSObject.h"
#include "JSScope.h"
#include "Operations.h"
#include <wtf/Assertions.h>

namespace JSC {

ASSERT_HAS_TRIVIAL_DESTRUCTOR(GetterSetter);

const ClassInfo GetterSetter::s_info = { "GetterSetter", 0, 0, CREATE_METHOD_TABLE(GetterSetter) };

void GetterSetter::visitChildren(JSCell* cell, SlotVisitor& visitor)
{
    GetterSetter* thisObject = jsCast<GetterSetter*>(cell);
    ASSERT_GC_OBJECT_INHERITS(thisObject, info());
    Base::visitChildren(thisObject, visitor);

    visitor.append(&thisObject->m_getter);
    visitor.append(&thisObject->m_setter);
}

JSValue GetterSetter::callGetter(ExecState* exec, JSValue thisValue)
{
	VM& vm = exec->vm();
    auto scope = DECLARE_THROW_SCOPE(vm);
    // FIXME: Some callers may invoke get() without checking for an exception first.
    // We work around that by checking here.
	RETURN_IF_EXCEPTION(vm, vm.exception()->value());

    JSObject* getter = this->getter();

    CallData callData = JSC::getCallData(vm, getter);
    return call(vm, exec, getter, callData, thisValue, ArgList());
}

bool GetterSetter::callSetter(ExecState* exec, JSValue thisValue, JSValue value, bool shouldThrow)
{
	VM& vm = exec->vm();
    auto scope = DECLARE_THROW_SCOPE(vm);

    JSObject* setter = this->setter();

    if (this->isSetterNull())
        return typeError(exec, vm, shouldThrow, ReadonlyPropertyWriteError);

    MarkedArgumentBuffer args;
    args.append(value);

    CallData callData = JSC::getCallData(vm, setter);
    scope.release();
    call(vm, exec, setter, callData, thisValue, args);
    return true;
}

} // namespace JSC
