/*
 * Copyright (C) 2015 Andy VanWagoner (thetalecrafter@gmail.com)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(INTL)


#include "JSCJSValueInlines.h"
#include "JSObject.h"

namespace JSC {

class IntlCollatorConstructor;
class IntlCollatorPrototype;
class IntlDateTimeFormatConstructor;
class IntlDateTimeFormatPrototype;
class IntlNumberFormatConstructor;
class IntlNumberFormatPrototype;
class IntlPluralRulesConstructor;
class IntlPluralRulesPrototype;

class IntlObject : public JSNonFinalObject {
public:
    typedef JSNonFinalObject Base;
    static const unsigned StructureFlags = Base::StructureFlags | HasStaticPropertyTable | OverridesGetOwnPropertySlot;

    static IntlObject* create(VM&, JSGlobalObject*, Structure*);
    static Structure* createStructure(VM&, JSGlobalObject*, JSValue);

    DECLARE_INFO;

    Structure* collatorStructure() const { return m_collatorStructure.get(); }
    Structure* dateTimeFormatStructure() const { return m_dateTimeFormatStructure.get(); }
    Structure* numberFormatStructure() const { return m_numberFormatStructure.get(); }
    Structure* localeStructure() const { return m_localeStructure.get(); }
    Structure* pluralRulesStructure() const { return m_pluralRulesStructure.get(); }
    Structure* relativeTimeFormatStructure() const { return m_relativeTimeFormatStructure.get(); }

protected:
    void finishCreation(VM&, JSGlobalObject*);

private:
    static void visitChildren(JSCell*, SlotVisitor&);

    IntlObject(VM&, Structure*);
    WriteBarrier<Structure> m_collatorStructure;
    WriteBarrier<Structure> m_dateTimeFormatStructure;
    WriteBarrier<Structure> m_localeStructure;
    WriteBarrier<Structure> m_numberFormatStructure;
    WriteBarrier<Structure> m_pluralRulesStructure;
    WriteBarrier<Structure> m_relativeTimeFormatStructure;
};

String defaultLocale(ExecState&);
const HashSet<String>& intlAvailableLocales();
const HashSet<String>& intlCollatorAvailableLocales();
inline const HashSet<String>& intlDateTimeFormatAvailableLocales() { return intlAvailableLocales(); }
inline const HashSet<String>& intlNumberFormatAvailableLocales() { return intlAvailableLocales(); }
inline const HashSet<String>& intlPluralRulesAvailableLocales() { return intlAvailableLocales(); }
inline const HashSet<String>& intlRelativeTimeFormatAvailableLocales() { return intlAvailableLocales(); }

bool intlBooleanOption(ExecState*, JSValue options, PropertyName, bool& usesFallback);
String intlStringOption(ExecState*, JSValue options, PropertyName, const HashSet<String>& values, ASCIILiteral notFound, ASCIILiteral fallback);
unsigned intlDefaultNumberOption(ExecState&, JSValue, PropertyName, unsigned minimum, unsigned maximum, unsigned fallback);
unsigned intlNumberOption(ExecState&, JSValue options, PropertyName, unsigned minimum, unsigned maximum, unsigned fallback);
Vector<char, 32> localeIDBufferForLanguageTag(const CString&);
String languageTagForLocaleID(const char*, bool isImmortal = false);
Vector<String> canonicalizeLocaleList(ExecState*, JSValue locales);
HashMap<String, String> resolveLocale(ExecState&, const HashSet<String>& availableLocales, const Vector<String>& requestedLocales, const HashMap<String, String>& options, const char* const relevantExtensionKeys[], size_t relevantExtensionKeyCount, Vector<String> (*localeData)(const String&, size_t));
JSValue supportedLocales(ExecState*, const HashSet<String>& availableLocales, const Vector<String>& requestedLocales, JSValue options);
String removeUnicodeLocaleExtension(const String& locale);
String bestAvailableLocale(const HashSet<String>& availableLocales, const String& requestedLocale);
template<typename Predicate> String bestAvailableLocale(const String& requestedLocale, Predicate);
Vector<String> numberingSystemsForLocale(const String& locale);

bool isUnicodeLocaleIdentifierType(StringView);

bool isUnicodeLanguageSubtag(StringView);
bool isUnicodeScriptSubtag(StringView);
bool isUnicodeRegionSubtag(StringView);
bool isUnicodeVariantSubtag(StringView);
bool isUnicodeLanguageId(StringView);
bool isStructurallyValidLanguageTag(StringView);

bool isWellFormedCurrencyCode(StringView);

} // namespace JSC

#endif // ENABLE(INTL)
