/*
 * Copyright (C) 2018 Andy VanWagoner (thetalecrafter@gmail.com)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(INTL)

#include "JSDestructibleObject.h"
#include <wtf/Optional.h>
#include <unicode/unum.h>
#include <unicode/upluralrules.h>
#include <unicode/uvernum.h>

namespace JSC {

class IntlPluralRulesConstructor;
class JSBoundFunction;

class IntlPluralRules FINAL : public JSDestructibleObject {
public:
    typedef JSDestructibleObject Base;

    static IntlPluralRules* create(VM&, Structure*);
    static Structure* createStructure(VM&, JSGlobalObject*, JSValue);

    DECLARE_INFO;

    void initializePluralRules(ExecState*, JSValue locales, JSValue options);
    JSValue select(ExecState&, double value);
    JSObject* resolvedOptions(ExecState&);

protected:
    IntlPluralRules(VM&, Structure*);
    ~IntlPluralRules();
    void finishCreation(VM&);
    static void destroy(JSCell*);
    static void visitChildren(JSCell*, SlotVisitor&);

private:
    bool m_initializedPluralRules;
    OwnPtr<UPluralRules> m_pluralRules;
    OwnPtr<UNumberFormat> m_numberFormat;

    String m_locale;
    UPluralType m_type;
    unsigned m_minimumIntegerDigits;
    unsigned m_minimumFractionDigits;
    unsigned m_maximumFractionDigits;
    Optional<unsigned> m_minimumSignificantDigits;
    Optional<unsigned> m_maximumSignificantDigits;
};

} // namespace JSC

#endif // ENABLE(INTL)
