
#ifndef JSStringInlines_h
#define JSStringInlines_h

#include "ExceptionHelpers.h"
#include "JSString.h"

namespace JSC {

inline bool JSString::equal(ExecState* exec, JSString* other) const
{
    if (isRope() || other->isRope())
        return equalSlowCase(exec, other);
    return WTF::equal(*m_value.impl(), *other->m_value.impl());
}

template<typename StringType1, typename StringType2>
inline JSValue jsMakeNontrivialString(JSGlobalData& vm, ExecState* exec, StringType1 string1, StringType2 string2)
{
	RefPtr<StringImpl> result = WTF::tryMakeString(string1, string2);
    if (UNLIKELY(!result)) {
        throwTerminatedExecutionException(exec, vm);
        return vm.smallStrings.emptyString();
    }
    return jsNontrivialString(vm, result.release());
}

template<typename StringType1, typename StringType2, typename StringType3>
inline JSValue jsMakeNontrivialString(JSGlobalData& vm, ExecState* exec, StringType1 string1, StringType2 string2, StringType3 string3)
{
	RefPtr<StringImpl> result = WTF::tryMakeString(string1, string2, string3);
    if (UNLIKELY(!result)) {
        throwTerminatedExecutionException(exec, vm);
        return vm.smallStrings.emptyString();
    }
    return jsNontrivialString(vm, result.release());
}

template<typename StringType1, typename StringType2, typename StringType3, typename StringType4>
inline JSValue jsMakeNontrivialString(JSGlobalData& vm, ExecState* exec, StringType1 string1, StringType2 string2, StringType3 string3, StringType4 string4)
{
	RefPtr<StringImpl> result = WTF::tryMakeString(string1, string2, string3, string4);
    if (UNLIKELY(!result)) {
        throwTerminatedExecutionException(exec, vm);
        return vm.smallStrings.emptyString();
    }
    return jsNontrivialString(vm, result.release());
}

template<typename StringType1, typename StringType2, typename StringType3, typename StringType4, typename StringType5>
inline JSValue jsMakeNontrivialString(JSGlobalData& vm, ExecState* exec, StringType1 string1, StringType2 string2, StringType3 string3, StringType4 string4, StringType5 string5)
{
	RefPtr<StringImpl> result = WTF::tryMakeString(string1, string2, string3, string4, string5);
    if (UNLIKELY(!result)) {
        throwTerminatedExecutionException(exec, vm);
        return vm.smallStrings.emptyString();
    }
    return jsNontrivialString(vm, result.release());
}

template<typename StringType1, typename StringType2, typename StringType3, typename StringType4, typename StringType5, typename StringType6>
inline JSValue jsMakeNontrivialString(JSGlobalData& vm, ExecState* exec, StringType1 string1, StringType2 string2, StringType3 string3, StringType4 string4, StringType5 string5, StringType6 string6)
{
	RefPtr<StringImpl> result = WTF::tryMakeString(string1, string2, string3, string4, string5, string6);
    if (UNLIKELY(!result)) {
        throwTerminatedExecutionException(exec, vm);
        return vm.smallStrings.emptyString();
    }
    return jsNontrivialString(vm, result.release());
}

template <typename CharacterType>
inline JSString* repeatCharacter(VM& vm, ExecState* exec, CharacterType character, unsigned repeatCount)
{
    CharacterType* buffer = 0;
    RefPtr<StringImpl> impl = StringImpl::tryCreateUninitialized(repeatCount, buffer);
    if (UNLIKELY(!impl)) {
        throwOutOfMemoryError(vm, exec);
        return vm.smallStrings.emptyString();
    }

    // std::fill_n(buffer, repeatCount, character);
    for(int i = 0; i < repeatCount; ++i)
    	buffer[i] = character;

    return jsString(vm, impl.release());
}

} // namespace JSC

#endif // #ifndef JSStringInlines_h
