
#pragma once

#include <atomic>
#include <wtf/ForbidHeapAllocation.h>
#include <wtf/StdLibExtras.h>

#ifndef FASTMALLOC_FUZZ_FAILURE
#define FASTMALLOC_FUZZ_FAILURE 0
#endif

namespace WTF
{

struct FastMallocInfo
{
    size_t allocatedBytes { 0 };
    size_t freePageBytes { 0 };
    size_t freeListBytes { 0 };
    size_t metaDataBytes { 0 };
    size_t shrunkPages { 0 };
    size_t maxVirtualPages { 0 };
    size_t largestFreeChunk { 0 };
};

FastMallocInfo fastMallocInfo();

typedef bool (*oomFuncPtr)(int askBytes, bool crashOnFailure, int iteration);

typedef int (*FastMallocLog)(const char * format, ... );

struct FastMallocInitInfo
{
	int capSizeKb { 0 };
	oomFuncPtr failedHandler { nullptr };
	oomFuncPtr reportHandler { nullptr };
	bool debugOn { false };
	bool enforce { false };
	bool backgroundScavenge { false };
	FastMallocLog log { nullptr };
	unsigned shrunkPages { 0 };
	unsigned minSystemAllocPages { 0 };
	bool mmapGrowDownward { true };
};
void setFastMallocCap(const FastMallocInitInfo& info);

class MonitorMallocFailedPreventer {
	WTF_FORBID_HEAP_ALLOCATION;
public:
	MonitorMallocFailedPreventer();
    ~MonitorMallocFailedPreventer();

    bool lowMemory() const { return m_lowMemory.load(std::memory_order_relaxed); }

    void setLowMemory() { m_lowMemory = 1; }

    void setPrev(MonitorMallocFailedPreventer* prev) { m_prev = prev; }
    void setNext(MonitorMallocFailedPreventer* next) { m_next = next; }

    MonitorMallocFailedPreventer* prev() const { return m_prev; }
    MonitorMallocFailedPreventer* next() const { return m_next; }

private:
    std::atomic<unsigned> m_lowMemory;

    MonitorMallocFailedPreventer* m_next { nullptr };
    MonitorMallocFailedPreventer* m_prev { nullptr };
};

}

using WTF::MonitorMallocFailedPreventer;
