/*
 * Copyright (C) 2017 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(WEB_RTC)

#include <wtf/Optional.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {

struct RTCRtpFecParameters {
    unsigned long ssrc { 0 };
};

struct RTCRtpRtxParameters {
    unsigned long ssrc { 0 };
};

enum class RTCPriorityType { VeryLow, Low, Medium, High };
enum class RTCDegradationPreference { MaintainFramerate, MaintainResolution, Balanced };
enum class RTCDtxStatus { Disabled, Enabled };

struct RTCRtpCodecParameters {
    unsigned short payloadType { 0 };
    String mimeType;
    unsigned long clockRate { 0 };
    unsigned short channels = 1;
};

struct RTCRtpEncodingParameters {
	using PriorityType = RTCPriorityType;
	using DtxStatus = RTCDtxStatus;

    unsigned long ssrc { 0 };
    RTCRtpRtxParameters rtx;
    RTCRtpFecParameters fec;
    RTCDtxStatus dtx { RTCDtxStatus::Disabled };
    bool active { false};
    RTCPriorityType priority { RTCPriorityType::Medium };
    unsigned long maxBitrate { 0 };
    unsigned long maxFramerate { 0 };
    String rid;
    double scaleResolutionDownBy { 1 };
};

struct RTCRtpHeaderExtensionParameters {
	RTCRtpHeaderExtensionParameters() : id(0) { }
    String uri;
    unsigned short id;
};

struct RTCRtpParameters {
	using PriorityType = RTCPriorityType;
	using DegradationPreference = RTCDegradationPreference;
	using DtxStatus = RTCDtxStatus;

    String transactionId;
    Vector<RTCRtpEncodingParameters> encodings;
    Vector<RTCRtpHeaderExtensionParameters> headerExtensions;
    Vector<RTCRtpCodecParameters> codecs;
    RTCDegradationPreference degradationPreference { RTCDegradationPreference::Balanced };
};


} // namespace WebCore

#endif // ENABLE(WEB_RTC)
