/*
 * Copyright (C) 2017 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


#pragma once

#if ENABLE(STREAMS_API)

#include "ExceptionCode.h"
#include "ReadableStreamChunk.h"
//#include <wtf/Function.h>
#include <wtf/PassOwnPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/RefPtr.h>
#include <wtf/PassRefPtr.h>
#include <wtf/text/WTFString.h>

namespace WebCore {

class BufferSource;
class ReadableStream;
struct ReadableStreamChunk;
class SharedBuffer;

class ReadableStreamSink : public RefCounted<ReadableStreamSink> {
public:
    virtual ~ReadableStreamSink() { }

    virtual void enqueue(const BufferSource&) = 0;
    virtual void close() = 0;
    virtual void error(const String&) = 0;
};

class ReadableStreamToSharedBufferSink FINAL : public ReadableStreamSink {
public:
	class Callback {
	    WTF_MAKE_FAST_ALLOCATED;
	public:
		virtual ~Callback() { }
		virtual void operator()(ReadableStreamChunk*, ExceptionCode ec, const String& message) = 0;
	};
    static PassRefPtr<ReadableStreamToSharedBufferSink> create(PassOwnPtr<Callback> callback) { return adoptRef(new ReadableStreamToSharedBufferSink(callback)); }
    void pipeFrom(ReadableStream&);
    void clearCallback() { m_callback = nullptr; }

private:
    explicit ReadableStreamToSharedBufferSink(PassOwnPtr<Callback>);

    virtual void enqueue(const BufferSource&) FINAL;
    virtual void close() FINAL;
    virtual void error(const String&) FINAL;

    OwnPtr<Callback> m_callback;
};

} // namespace WebCore

#endif // ENABLE(STREAMS_API)
