/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#if ENABLE(SQL_DATABASE)

#include "AbstractDatabaseServer.h"

namespace WebCore {
namespace DBBackend {

class Server final: public AbstractDatabaseServer {
	WTF_MAKE_FAST_ALLOCATED;
public:
    Server() { };
    virtual ~Server() { }

    virtual void initialize(const String& databasePath) override;

    virtual void setClient(DatabaseManagerClient*) override;
    virtual String databaseDirectoryPath() const override;
    virtual void setDatabaseDirectoryPath(const String&) override;

    virtual String fullPathForDatabase(SecurityOrigin*, const String& name, bool createIfDoesNotExist) override;

#if !PLATFORM(CHROMIUM)
    virtual bool hasEntryForOrigin(SecurityOrigin*) override;
    virtual void origins(Vector<RefPtr<SecurityOrigin> >& result) override;
    virtual bool databaseNamesForOrigin(SecurityOrigin*, Vector<String>& result) override;
    virtual DatabaseDetails detailsForNameAndOrigin(const String&, SecurityOrigin*) override;

    virtual unsigned long long usageForOrigin(SecurityOrigin*) override;
    virtual unsigned long long quotaForOrigin(SecurityOrigin*) override;

    virtual void setQuota(SecurityOrigin*, unsigned long long) override;

    virtual void deleteAllDatabases() override;
    virtual bool deleteOrigin(SecurityOrigin*) override;
    virtual bool deleteDatabase(SecurityOrigin*, const String& name) override;

    // From a secondary thread, must be thread safe with its data
    virtual void scheduleNotifyDatabaseChanged(SecurityOrigin*, const String& name) override;
    virtual void databaseChanged(AbstractDatabase*) override;

#else // PLATFORM(CHROMIUM)
    virtual void closeDatabasesImmediately(const String& originIdentifier, const String& name) override;
#endif // PLATFORM(CHROMIUM)

    virtual void interruptAllDatabasesForContext(const ScriptExecutionContext*) override;

    virtual bool canEstablishDatabase(ScriptExecutionContext*, const String& name, const String& displayName, unsigned long estimatedSize, ExceptionCode& ec) override;
    virtual void addOpenDatabase(AbstractDatabase*) override;
    virtual void removeOpenDatabase(AbstractDatabase*) override;

    virtual void setDatabaseDetails(SecurityOrigin*, const String& name, const String& displayName, unsigned long estimatedSize) override;
    virtual unsigned long long getMaxSizeForDatabase(const AbstractDatabase*) override;
};

} // namespace DBBackend
} // namespace WebCore

#endif // ENABLE(SQL_DATABASE)
