/*
 *  Copyright (C) 2000 Harri Porten (porten@kde.org)
 *  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Apple Inc. All rights reseved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#pragma once

#include "CommonVM.h"
#include "JSDOMBinding.h"
#include "JSDOMGlobalObject.h"
#include <wtf/Forward.h>
#include <wtf/HashSet.h>

namespace WebCore {

class DOMWindow;
class Frame;
class DOMWrapperWorld;
class JSDOMWindow;
class JSDOMWindowShell;

class JSDOMWindowBasePrivate;

class JSDOMWindowBase : public JSDOMGlobalObject {
	typedef JSDOMGlobalObject Base;
protected:
	JSDOMWindowBase(JSC::JSGlobalData&, JSC::Structure*, PassRefPtr<DOMWindow>, JSDOMWindowShell*);
	void finishCreation(JSC::JSGlobalData&, JSDOMWindowShell*);

	~JSDOMWindowBase();

	static void destroy(JSCell*);

public:
	void updateDocument();

	DOMWindow& wrapped() const { return *m_impl; }
	ScriptExecutionContext* scriptExecutionContext() const;

	// Called just before removing this window from the JSDOMWindowShell.
	void willRemoveFromWindowShell();

	DECLARE_INFO;

	static JSC::Structure* createStructure(JSC::JSGlobalData& globalData, JSC::JSValue prototype)
	{
		return JSC::Structure::create(globalData, 0, prototype, JSC::TypeInfo(JSC::GlobalObjectType, StructureFlags), info());
	}

	static const JSC::GlobalObjectMethodTable s_globalObjectMethodTable;

	static bool supportsProfiling(const JSC::JSGlobalObject*);
	static bool supportsRichSourceInfo(const JSC::JSGlobalObject*);
	static bool shouldInterruptScript(const JSC::JSGlobalObject*);
	static bool javaScriptExperimentsEnabled(const JSC::JSGlobalObject*);
	static void queueMicrotaskToEventLoop(const JSC::JSGlobalObject*, PassRefPtr<JSC::Microtask>);

	void printErrorMessage(const String&) const;

	JSDOMWindowShell* shell() const;

	static JSC::JSGlobalData& commonVM() { return WebCore::commonVM(); }
	static JSC::JSGlobalData* commonJSGlobalData() { return &commonVM(); }
	static void fireFrameClearedWatchpointsForWindow(DOMWindow*);

    void setCurrentEvent(Event*);
    Event* currentEvent() const;

protected:
	RefPtr<JSC::WatchpointSet> m_windowCloseWatchpoints;

private:
	static JSC::Identifier moduleLoaderResolve(JSC::JSGlobalObject*, JSC::ExecState*, JSC::JSModuleLoader*, JSC::JSValue, JSC::JSValue, JSC::JSValue);
	static JSC::JSInternalPromise* moduleLoaderFetch(JSC::JSGlobalObject*, JSC::ExecState*, JSC::JSModuleLoader*, JSC::JSValue, JSC::JSValue, JSC::JSValue);
	static JSC::JSValue moduleLoaderEvaluate(JSC::JSGlobalObject*, JSC::ExecState*, JSC::JSModuleLoader*, JSC::JSValue, JSC::JSValue, JSC::JSValue);
	static JSC::JSInternalPromise* moduleLoaderImportModule(JSC::JSGlobalObject*, JSC::ExecState*, JSC::JSModuleLoader*, JSC::JSString*, JSC::JSValue, const JSC::SourceOrigin&);

	RefPtr<DOMWindow> m_impl;
    RefPtr<Event> m_currentEvent;
	JSDOMWindowShell* m_shell;
};

inline Event* JSDOMWindowBase::currentEvent() const
{
    return m_currentEvent.get();
}

inline JSDOMWindowShell* JSDOMWindowBase::shell() const
{
    return m_shell;
}

// The following return a JSDOMWindowShell or jsNull()
// JSDOMGlobalObject* is ignored, accessing a window in any context will use that DOMWindow's prototype chain.
JSC::JSValue toJS(JSC::ExecState*, JSDOMGlobalObject*, DOMWindow&);
inline JSC::JSValue toJS(JSC::ExecState* state, JSDOMGlobalObject* globalObject, DOMWindow* window) { return window ? toJS(state, globalObject, *window) : JSC::jsNull(); }
JSC::JSValue toJS(JSC::ExecState*, JSDOMGlobalObject*, Frame&);
inline JSC::JSValue toJS(JSC::ExecState* state, JSDOMGlobalObject* globalObject, Frame* frame) { return frame ? toJS(state, globalObject, *frame) : JSC::jsNull(); }
JSC::JSValue toJS(JSC::ExecState*, DOMWindow&);
inline JSC::JSValue toJS(JSC::ExecState* state, DOMWindow* window) { return window ? toJS(state, *window) : JSC::jsNull(); }

// The following return a JSDOMWindow or nullptr.
JSDOMWindow* toJSDOMWindow(Frame&, DOMWrapperWorld&);
inline JSDOMWindow* toJSDOMWindow(Frame* frame, DOMWrapperWorld& world) { return frame ? toJSDOMWindow(*frame, world) : 0; }
inline JSDOMWindow* toJSDOMWindow(Frame* frame, DOMWrapperWorld* world) { return  toJSDOMWindow(frame, *world); }
JSDOMWindow* toJSDOMWindow(JSC::JSValue);

} // namespace WebCore
