/*
 * Copyright (C) 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef JSDOMWindowShell_h
#define JSDOMWindowShell_h

#include "DOMWindow.h"
#include "JSDOMWindow.h"
#include <runtime/JSProxy.h>

namespace WebCore {

class Frame;

// FIXME: Rename to JSDOMWindowProxy or JSWindowProxy to match HTML spec naming. (https://webkit.org/b/80733)
class JSDOMWindowShell : public JSC::JSProxy {
	typedef JSC::JSProxy Base;
public:
	~JSDOMWindowShell();
	static void destroy(JSCell*);

	ALWAYS_INLINE JSDOMWindow* window() const { return JSC::jsCast<JSDOMWindow*>(target()); }
	void setWindow(JSC::JSGlobalData& globalData, JSDOMWindow* window)
	{
		ASSERT_ARG(window, window);
		setTarget(globalData, window);
		structure()->setGlobalObject(*JSDOMWindow::commonJSGlobalData(), window);
	}
	void setWindow(PassRefPtr<DOMWindow>);

	DECLARE_INFO;

	DOMWindow& wrapped() const;

	static JSDOMWindowShell* create(PassRefPtr<DOMWindow> window, JSC::Structure* structure, DOMWrapperWorld* world)
	{
		JSC::Heap& heap = JSDOMWindow::commonJSGlobalData()->heap;
		JSDOMWindowShell* shell = new (NotNull, JSC::allocateCell<JSDOMWindowShell>(heap)) JSDOMWindowShell(structure, world);
		shell->finishCreation(*world->globalData(), window);
		return shell;
	}

	static JSC::Structure* createStructure(JSC::JSGlobalData& globalData, JSC::JSValue prototype)
	{
		return JSC::Structure::create(globalData, 0, prototype, JSC::TypeInfo(JSC::PureForwardingProxyType, StructureFlags), info());
	}

	ALWAYS_INLINE DOMWrapperWorld* world() { return m_world.get(); }

protected:
	JSDOMWindowShell(JSC::Structure*, DOMWrapperWorld*);
	void finishCreation(JSC::JSGlobalData&, PassRefPtr<DOMWindow>);

	RefPtr<DOMWrapperWorld> m_world;
};

// JSDOMWindowShell is a little odd in that it's not a traditional wrapper and has no back pointer. It
// is however strongly owned by Frame (via its ScriptController), and therefore, to get the JSDOMWindowShell
// a Frame is passed.
JSC::JSValue toJS(JSC::ExecState*, Frame&);
inline JSC::JSValue toJS(JSC::ExecState* state, Frame* frame) { return frame ? toJS(state, *frame) : JSC::jsNull(); }

JSDOMWindowShell* toJSDOMWindowShell(Frame&, DOMWrapperWorld&);
inline JSDOMWindowShell* toJSDOMWindowShell(Frame* frame, DOMWrapperWorld& world) { return frame ? toJSDOMWindowShell(*frame, world) : 0; }
inline JSDOMWindowShell* toJSDOMWindowShell(Frame* frame, DOMWrapperWorld* world) { return toJSDOMWindowShell(frame, world); }

} // namespace WebCore

#endif // JSDOMWindowShell_h
