/*
 * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "JSCSSStyleDeclarationCustom.h"
#include "JSDeprecatedCSSOMPrimitiveValue.h"
#include "JSDeprecatedCSSOMValue.h"
#include "JSDeprecatedCSSOMValueList.h"
#include "JSNode.h"

using namespace JSC;

namespace WebCore {

bool JSDeprecatedCSSOMValueOwner::isReachableFromOpaqueRoots(JSC::Handle<JSC::Unknown> handle, void*, SlotVisitor& visitor)
{
	JSDeprecatedCSSOMValue* jsCSSValue = jsCast<JSDeprecatedCSSOMValue*>(handle.get().asCell());
    if (!jsCSSValue->hasCustomProperties())
        return false;
    return visitor.containsOpaqueRoot(root(&jsCSSValue->wrapped().owner()));
}

JSValue toJS(ExecState*, JSDOMGlobalObject* globalObject, DeprecatedCSSOMValue* value)
{
    if (!value)
        return jsNull();

    if (JSObject* wrapper = getCachedWrapper(globalObject, value))
        return wrapper;

    if (value->isValueList())
        return CREATE_DOM_WRAPPER(globalObject, DeprecatedCSSOMValueList, value);// Expose CSS-wide keywords as plain CSSValues to keep the existing behavior.
    if (value->isPrimitiveValue() && !static_cast<DeprecatedCSSOMPrimitiveValue&>(*value).isCSSWideKeyword())
        return CREATE_DOM_WRAPPER(globalObject, DeprecatedCSSOMPrimitiveValue, value);
    return CREATE_DOM_WRAPPER(globalObject, DeprecatedCSSOMValue, value);
}

} // namespace WebCore
