/*
 * Copyright (C) 2007 Apple Inc. All rights reserved.
 * Copyright (C) 2010 Torch Mobile (Beijing) Co. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "JSHTMLCanvasElement.h"

#include "HTMLCanvasElement.h"
#include "ImageBitmapRenderingContext.h"
#include "JSImageBitmapRenderingContext.h"
#include "JSCanvasRenderingContext2D.h"
#if ENABLE(WEBGL)
#include "JSDictionary.h"
#include "InspectorWebGLInstrumentation.h"
#include "JSWebGLRenderingContext.h"
#include "ScriptObject.h"
#endif
#include <wtf/GetPtr.h>

using namespace JSC;

namespace WebCore {

JSValue JSHTMLCanvasElement::getContext(ExecState* exec)
{
    VM& vm = exec->vm();

    if (UNLIKELY(exec->argumentCount() < 1))
        return vm.throwException(exec, createNotEnoughArgumentsError(exec));

    HTMLCanvasElement& canvas = wrapped();
    const String& contextId = exec->argument(0).toWTFString(exec);

    ExceptionCode ec = 0;
    CanvasRenderingContext* context = canvas.getContext(exec, contextId, ec);
    if (ec) {
    	setDOMException(exec, ec);
        return jsNull();
    }
    if (!context)
        return jsNull();

#if ENABLE(WEBGL)
    if ((*context).is3d())
        return toJS(exec, globalObject(), static_cast<WebGLRenderingContext&>(*context));
#endif

    if ((*context).isBitmapRenderer())
        return toJS(exec, globalObject(), static_cast<ImageBitmapRenderingContext&>(*context));

    return toJS(exec, globalObject(), static_cast<CanvasRenderingContext2D&>(*context));
}

JSValue JSHTMLCanvasElement::toDataURL(ExecState* exec)
{
	VM& vm = exec->vm();

    HTMLCanvasElement& canvas = static_cast<HTMLCanvasElement&>(wrapped());
    ExceptionCode ec = 0;

    String type = valueToStringWithUndefinedOrNullCheck(exec, exec->argument(0));
    RETURN_IF_EXCEPTION(vm, jsUndefined());

    Optional<double> quality;
    JSValue qualityValue = exec->argument(1);
    if (qualityValue.isNumber())
        quality = qualityValue.toNumber(exec);

    String result = canvas.toDataURL(type, quality, ec);
    if (UNLIKELY(ec)) {
    	setDOMException(exec, ec);
        return jsUndefined();
    }
    return jsString(vm, result);
}

} // namespace WebCore
