/*
 * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#if !HAVE(BINDINGS_ALL_IN_ONE)
#include "JSHTMLObjectElementCustom.h"

#include "HTMLObjectElement.h"
#include "JSPluginElementFunctions.h"

namespace WebCore {

using namespace JSC;

bool JSHTMLObjectElement::getOwnPropertySlotDelegate(ExecState* exec, PropertyName propertyName, PropertySlot& slot)
{
	if(static_cast<HTMLObjectElement&>(wrapped()).jsGet(exec, propertyName, slot, this))
		return true;
		
    return pluginElementCustomGetOwnPropertySlot<JSHTMLObjectElement, Base>(exec, propertyName, slot, this);
}

bool JSHTMLObjectElement::put(JSCell* cell, ExecState* state, PropertyName propertyName, JSValue value, PutPropertySlot& putPropertySlot)
{
    auto* thisObject = jsCast<JSHTMLObjectElement*>(cell);
    ASSERT_GC_OBJECT_INHERITS(thisObject, info());

	if(static_cast<HTMLObjectElement&>(thisObject->wrapped()).jsPut(state, propertyName, value, putPropertySlot, thisObject))
		return true;

    bool putResult = false;
    if (runtimeObjectCustomPut(state, propertyName, value, thisObject, putPropertySlot, putResult))
        return putResult;

    return Base::put(thisObject, state, propertyName, value, putPropertySlot);
}

bool JSHTMLObjectElement::putByIndex(JSCell* cell, ExecState* state, unsigned index, JSValue value, bool shouldThrow)
{
    auto* thisObject = jsCast<JSHTMLObjectElement*>(cell);
    ASSERT_GC_OBJECT_INHERITS(thisObject, info());
    VM& vm = thisObject->vm();

    Identifier propertyName = Identifier::from(vm, index);
    PutPropertySlot slot(thisObject, shouldThrow);

	if(static_cast<HTMLObjectElement&>(thisObject->wrapped()).jsPut(state, propertyName, value, slot, thisObject))
		return true;

    bool putResult = false;
    if (runtimeObjectCustomPut(state, propertyName, value, thisObject, slot, putResult))
        return putResult;

    return Base::putByIndex(cell, state, index, value, shouldThrow);
}

CallData JSHTMLObjectElement::getCallData(JSCell* cell)
{
    return runtimeObjectGetCallData(jsCast<JSHTMLObjectElement*>(cell));
}

} // namespace WebCore
#endif
