/*
 * Copyright (C) 2016 Metrological Group B.V.
 * Copyright (C) 2016 Igalia S.L.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "JSWebAnimation.h"

#include "JSAnimationEffect.h"
#include "JSAnimationTimeline.h"
#include "JSCSSAnimation.h"
#include "JSCSSTransition.h"
#include "JSDOMConstructor.h"

namespace WebCore {

using namespace JSC;

JSValue toJSNewlyCreated(ExecState*, JSDOMGlobalObject* globalObject, WebAnimation* value)
{
	if (!value)
		return JSC::jsNull();
    if (value->isCSSAnimation())
        return wrap<JSCSSAnimation>(globalObject, static_cast<CSSAnimation*>(value));
    if (value->isCSSTransition())
        return wrap<JSCSSTransition>(globalObject, static_cast<CSSTransition*>(value));
    return wrap<JSWebAnimation>(globalObject, value);
}

JSValue toJS(ExecState* state, JSDOMGlobalObject* globalObject, WebAnimation* value)
{
    return toJSNewlyCreated(state, globalObject, value);
}

template<> EncodedJSValue JSC_HOST_CALL JSWebAnimationConstructor::construct(ExecState* state)
{
    VM& vm = state->vm();
    //auto throwScope = DECLARE_THROW_SCOPE(vm);
    //UNUSED_PARAM(throwScope);
    auto* jsConstructor = jsCast<JSWebAnimationConstructor*>(state->jsCallee());
    ASSERT(jsConstructor);
    auto* context = jsConstructor->scriptExecutionContext();
    if (UNLIKELY(!context))
        return throwConstructorScriptExecutionContextUnavailableError(*state, "Animation");
    auto* document = static_cast<Document*>(context);
    AnimationEffect* effect = toAnimationEffect(state->argument(0));
    if(!effect)
        throwArgumentTypeError(*state, 0, "effect", "Animation", 0, "AnimationEffect");
    RETURN_IF_EXCEPTION(vm, encodedJSUndefined());

    if (state->argument(1).isUndefined()) {
    	RefPtr<WebAnimation> object = WebAnimation::create(document, effect);
        return JSValue::encode(toJS(state, jsConstructor->globalObject(), object.get()));
    }

    AnimationTimeline* timeline = toAnimationTimeline(state->uncheckedArgument(1));
    if(!timeline)
        throwArgumentTypeError(*state, 1, "timeline", "Animation", 0, "AnimationTimeline");
    RETURN_IF_EXCEPTION(vm, encodedJSUndefined());
    RefPtr<WebAnimation> object = WebAnimation::create(document, effect, timeline);
    return JSValue::encode(toJS(state, jsConstructor->globalObject(), object.get()));
}

} // namespace WebCore

