/*
 * Copyright (C) 2016 Canon Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted, provided that the following conditions
 * are required to be met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Canon Inc. nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CANON INC. AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CANON INC. AND ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "config.h"
#include "ReadableStreamDefaultController.h"

#if ENABLE(STREAMS_API)

#include "JSReadableStream.h"
#include "JSReadableStreamSource.h"
#include "WebCoreJSClientData.h"

namespace WebCore {

JSC::JSValue ReadableStreamDefaultController::invoke(JSC::ExecState& state, JSC::JSObject& object, const ASCIILiteral& propertyName, JSC::JSValue parameter)
{
	JSC::VM& vm = state.vm();
    JSC::JSLockHolder lock(&state);

    auto scope = DECLARE_CATCH_SCOPE(vm);
    auto function = object.get(vm, &state, JSC::Identifier::fromString(vm, propertyName));
    if (vm.exception())
        return JSC::jsUndefined();

    if (!function.isCallable(vm)) {
        if (!function.isUndefined())
            throwTypeError(&state, vm, ASCIILiteral("ReadableStream trying to call a property that is not callable"));
        return JSC::jsUndefined();
    }

    JSC::MarkedArgumentBuffer arguments;
    arguments.append(parameter);

    auto callData = JSC::getCallData(vm, function);
    auto result = call(&state, function, callData, &object, arguments);
    EXCEPTION_ASSERT(!scope.exception() || vm.hasPendingTerminationException());
	return result;
}

} // namespace WebCore

#endif // ENABLE(STREAMS_API)
