/*
 *  Copyright (c) 2015, Canon Inc. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1.  Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *  2.  Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *  3.  Neither the name of Canon Inc. nor the names of
 *      its contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY CANON INC. AND ITS CONTRIBUTORS "AS IS" AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL CANON INC. AND ITS CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <builtins/BuiltinUtils.h>

namespace WebCore {

#define WEBCORE_COMMON_PRIVATE_IDENTIFIERS_EACH_PROPERTY_NAME(macro) \
	macro(AbortSignal) \
	macro(Animation) \
    macro(AnimationEffect) \
    macro(AnimationPlaybackEvent) \
	macro(AnimationTimeline) \
    macro(abortAlgorithm) \
    macro(abortSteps) \
    macro(addTrack) \
    macro(associatedReadableByteStreamController) \
	macro(appendFromJS) \
    macro(autoAllocateChunkSize) \
    macro(backingMap) \
    macro(backpressure) \
    macro(backpressureChangePromise) \
    macro(body) \
    macro(byobRequest) \
    macro(cancel) \
    macro(cancelAlgorithm) \
    macro(cloneArrayBuffer) \
    macro(closeRequested) \
    macro(closedPromiseCapability) \
    macro(closeAlgorithm) \
    macro(closeRequest) \
    macro(closedPromise) \
    macro(consume) \
    macro(consumeChunk) \
    macro(controlledReadableStream) \
    macro(controller) \
    macro(createReadableStream) \
    macro(CSSAnimation) \
    macro(CSSTransition) \
    macro(disturbed) \
    macro(encode) \
    macro(errorSteps) \
    macro(fillFromJS) \
    macro(finishConsumingStream) \
    macro(flush) \
    macro(flushAlgorithm) \
    macro(getUserMediaFromJS) \
    macro(getRemoteStreams) \
    macro(getSenders) \
    macro(getTracks) \
    macro(Headers) \
    macro(highWaterMark) \
    macro(inFlightCloseRequest) \
    macro(inFlightWriteRequest) \
    macro(initializeWith) \
    macro(localStreams) \
    macro(isAbortSignal) \
    macro(isDisturbed) \
    macro(isLoading) \
    macro(KeyframeEffect) \
    macro(makeDOMException) \
    macro(makeThisTypeError) \
    macro(makeGetterTypeError) \
    macro(operations) \
    macro(ownerReadableStream) \
    macro(pendingAbortRequest) \
    macro(pendingPullIntos) \
    macro(privateGetStats) \
    macro(pull) \
    macro(pulling) \
    macro(pullAgain) \
    macro(pullAlgorithm) \
    macro(queue) \
    macro(queuedAddIceCandidate) \
    macro(queuedCreateAnswer) \
    macro(queuedCreateOffer) \
    macro(queuedSetLocalDescription) \
    macro(queuedSetRemoteDescription) \
    macro(readable) \
    macro(reader) \
    macro(readIntoRequests) \
    macro(readRequests) \
    macro(readableStreamController) \
    macro(readyPromise) \
    macro(readyPromiseCapability) \
    macro(removeTrack) \
    macro(setBody) \
    macro(setBodyFromInputRequest) \
    macro(setStatus) \
    macro(size) \
    macro(state) \
    macro(startConsumingStream) \
    macro(start) \
    macro(started) \
    macro(startedPromise) \
    macro(storedError) \
    macro(strategy) \
    macro(strategyHWM) \
    macro(strategySizeAlgorithm) \
    macro(stream) \
    macro(streamClosed) \
    macro(streamClosing) \
    macro(streamErrored) \
    macro(streamReadable) \
    macro(streamWaiting) \
    macro(streamWritable) \
    macro(structuredCloneForStream) \
    macro(textEncoderStreamEncoder) \
    macro(textEncoderStreamTransform) \
    macro(transformAlgorithm) \
    macro(underlyingByteSource) \
    macro(underlyingSink) \
    macro(underlyingSource) \
    macro(view) \
    macro(whenSignalAborted) \
    macro(writeAlgorithm) \
    macro(writer) \
    macro(writeRequests) \
    macro(writing) \
    macro(MediaStream) \
    macro(MediaStreamTrack) \
    macro(ReadableByteStreamController) \
    macro(ReadableStream) \
    macro(ReadableStreamBYOBReader) \
    macro(ReadableStreamBYOBRequest) \
    macro(ReadableStreamDefaultController) \
    macro(ReadableStreamDefaultReader) \
    macro(Request) \
    macro(Response) \
    macro(RTCIceCandidate) \
    macro(RTCSessionDescription) \
    macro(TextEncoderStream) \
    macro(TextEncoderStreamEncoder) \
    macro(TransformStream) \
    macro(TransformStreamDefaultController) \
    macro(writable) \
    macro(WritableStream) \
    macro(WritableStreamDefaultController) \
    macro(WritableStreamDefaultWriter) \

class WebCoreBuiltinNames {
public:
    explicit WebCoreBuiltinNames(JSC::JSGlobalData& vm)
        : m_vm(vm)
        WEBCORE_COMMON_PRIVATE_IDENTIFIERS_EACH_PROPERTY_NAME(INITIALIZE_BUILTIN_NAMES)
    {
#define EXPORT_NAME(name) m_vm.propertyNames->appendExternalName(name##PublicName(), name##PrivateName());
        WEBCORE_COMMON_PRIVATE_IDENTIFIERS_EACH_PROPERTY_NAME(EXPORT_NAME)
#undef EXPORT_NAME
    }

    WEBCORE_COMMON_PRIVATE_IDENTIFIERS_EACH_PROPERTY_NAME(DECLARE_BUILTIN_IDENTIFIER_ACCESSOR)

private:
    JSC::JSGlobalData& m_vm;
    WEBCORE_COMMON_PRIVATE_IDENTIFIERS_EACH_PROPERTY_NAME(DECLARE_BUILTIN_NAMES)
};

} // namespace WebCore
