/*
 * Copyright (C) 2007, 2008, 2011 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "CachedFont.h"
#include "CachedResourceHandle.h"
#include <runtime/ArrayBufferView.h>
#include <wtf/OwnPtr.h>
#include <wtf/WeakPtr.h>
#include <wtf/text/AtomicString.h>

namespace WebCore {

class CSSFontFace;
class CSSFontSelector;
class Document;
class FontCascade;
struct FontCustomPlatformData;
class FontDescription;
class FontFeatureSettings;
struct FontVariantSettings;
class Font;
class SVGFontFaceElement;
class SharedBuffer;

class CSSFontFaceSource FINAL : public CachedFontClient {
public:
    CSSFontFaceSource(CSSFontFace& owner, const String& familyNameOrURI);
	CSSFontFaceSource(CSSFontFace& owner, const String& familyNameOrURI, CSSFontSelector&, CachedFont&);
	CSSFontFaceSource(CSSFontFace& owner, const String& familyNameOrURI, SVGFontFaceElement&);
	CSSFontFaceSource(CSSFontFace& owner, const String& familyNameOrURI, PassRefPtr<JSC::ArrayBufferView>);
	virtual ~CSSFontFaceSource();

    //                      => Success
    //                    //
    // Pending => Loading
    //                    \\.
    //                      => Failure
    enum Status {
        Pending,
        Loading,
        Success,
        Failure
    };
    Status status() const { return m_status; }

    const AtomicString& familyNameOrURI() const { return m_familyNameOrURI; }

    void load();
    PassRefPtr<Font> font(const FontDescription&, bool syntheticBold, bool syntheticItalic, const FontFeatureSettings&, const FontVariantSettings&);

    bool requiresExternalResource() const { return m_font; }

#if ENABLE(SVG_FONTS)
    bool isSVGFontFaceSource() const;
#endif

private:
    bool shouldIgnoreFontLoadCompletions() const;

    virtual void fontLoaded(CachedFont*) OVERRIDE;

	void setStatus(Status);

	AtomicString m_familyNameOrURI; // URI for remote, built-in font name for local.
    CSSFontFace& m_face; // Our owning font face.
    WeakPtr<CSSFontSelector> m_fontSelector; // For remote fonts, to orchestrate loading.
	CachedResourceHandle<CachedFont> m_font; // Also for remote fonts, a pointer to our cached resource.

    RefPtr<SharedBuffer> m_generatedOTFBuffer;
    RefPtr<JSC::ArrayBufferView> m_immediateSource;
    RefPtr<FontCustomPlatformData> m_immediateFontCustomPlatformData;

#if ENABLE(SVG_FONTS)
    WeakPtr<SVGFontFaceElement> m_svgFontFaceElement;
    RefPtr<FontCustomPlatformData> m_inDocumentCustomPlatformData;
#endif

    Status m_status { Status::Pending };
#if ENABLE(SVG_FONTS)
    bool m_hasSVGFontFaceElement { false };
#endif
};

}
