/*
 * (C) 1999-2003 Lars Knoll (knoll@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2008, 2012 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "CSSValue.h"
#include "CachedResourceHandle.h"
#include <wtf/RefPtr.h>

namespace WebCore {

class CachedImage;
class CachedResourceLoader;
class DeprecatedCSSOMValue;
class CSSStyleDeclaration;
class RenderElement;
struct ResourceLoaderOptions;

namespace Style {
class BuilderState;
}

struct ResolvedURL {
    String specifiedURLString;
    KURL resolvedURL;
    bool isLocalURL() const;
};

class CSSImageValue : public CSSValue {
public:
    static Ref<CSSImageValue> create(const ResolvedURL&);
    static Ref<CSSImageValue> create(const KURL&);
    static Ref<CSSImageValue> create(CachedImage&);
    ~CSSImageValue();

    bool isPending() const;
    CachedImage* loadImage(CachedResourceLoader&, const ResourceLoaderOptions&);
    CachedImage* cachedImage() const { return m_cachedImage ? m_cachedImage.value().get() : nullptr; }

    // Take care when using this, and read https://drafts.csswg.org/css-values/#relative-urls
	const KURL& imageURL() const { return m_location.resolvedURL; }

    KURL reresolvedURL(const Document&) const;

    String customCSSText() const;

    Ref<DeprecatedCSSOMValue> createDeprecatedCSSOMWrapper(CSSStyleDeclaration&) const;

    template<class Functor> bool traverseSubresources(const Functor& handler) const;

    bool equals(const CSSImageValue&) const;

    bool knownToBeOpaque(const RenderElement*) const;

    void setInitiator(const AtomicString& name) { m_initiatorName = name; }

    Ref<CSSImageValue> valueWithStylesResolved(Style::BuilderState&);

private:
    explicit CSSImageValue(const ResolvedURL& url);
    explicit CSSImageValue(CachedImage&);

    ResolvedURL m_location;
    Optional<CachedResourceHandle<CachedImage>> m_cachedImage;
    AtomicString m_initiatorName;
    RefPtr<CSSImageValue> m_unresolvedValue;
};

} // namespace WebCore

