/*
 * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "CSSParser.h"
#include "CSSRule.h"
#include "ExceptionCode.h"
#include "StylePropertySet.h"
#include "StyleRule.h"
#include <wtf/PassRefPtr.h>
#include <wtf/RefPtr.h>

namespace WebCore {

class StyleRuleCSSStyleDeclaration;
class CSSKeyframesRule;

// FIXME-NEWPARSER: Rename this to StyleRuleKeyframe
class StyleRuleKeyframe FINAL : public StyleRuleBase {
public:
    // FIXME-NEWPARSER: Remove this create function once we get rid of the old parser.
    static Ref<StyleRuleKeyframe> create(PassRefPtr<StylePropertySet> properties);
    static Ref<StyleRuleKeyframe> create(Vector<double>&& keys, PassRefPtr<StyleProperties> properties);
    ~StyleRuleKeyframe();

    String keyText() const;
    bool setKeyText(const String&);
    void setKey(double key)
	{
		ASSERT(m_keys.isEmpty());
		m_keys.clear();
		m_keys.append(key);
	}

    const Vector<double>& keys() const { return m_keys; };

    const StylePropertySet& properties() const { return *m_properties; }
    MutableStylePropertySet* mutableProperties();
    
    String cssText() const;

private:
    StyleRuleKeyframe(PassRefPtr<StylePropertySet>);
    StyleRuleKeyframe(Vector<double>&&, PassRefPtr<StyleProperties>);

    RefPtr<StylePropertySet> m_properties;
    Vector<double> m_keys;
};

class CSSKeyframeRule final : public CSSRule {
public:
    virtual ~CSSKeyframeRule();

    virtual CSSRule::Type type() const OVERRIDE { return KEYFRAME_RULE; }
    virtual String cssText() const OVERRIDE { return m_keyframe->cssText(); }
    virtual void reattach(StyleRuleBase*) OVERRIDE;

    String keyText() const { return m_keyframe->keyText(); }
    void setKeyText(const String& text) { m_keyframe->setKeyText(text); }

    CSSStyleDeclaration* style() const;

private:
    CSSKeyframeRule(StyleRuleKeyframe*, CSSKeyframesRule* parent);

    RefPtr<StyleRuleKeyframe> m_keyframe;
    mutable RefPtr<StyleRuleCSSStyleDeclaration> m_propertiesCSSOMWrapper;
    
    friend class CSSKeyframesRule;
};

} // namespace WebCore
