/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 * Copyright (C) 2003-2023 Apple Inc. All rights reserved.
 * Copyright (C) 2012-2014 Google Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "CSSPropertyNames.h"
#include <wtf/FastAllocBase.h>
#include <wtf/Noncopyable.h>

namespace WebCore {

class Animation;
class CSSValue;
class FillLayer;
class RenderStyle;
class StyleImage;
class NinePieceImage;

class LengthBox;

namespace Style {
class BuilderState;
}

class CSSToStyleMap {
    WTF_MAKE_NONCOPYABLE(CSSToStyleMap);
    WTF_MAKE_FAST_ALLOCATED;

public:
    CSSToStyleMap(Style::BuilderState&);

    void mapFillAttachment(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillClip(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillComposite(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillBlendMode(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillOrigin(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillImage(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillRepeatX(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillRepeatY(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillSize(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillXPosition(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillYPosition(CSSPropertyID, FillLayer&, CSSValue&);
    void mapFillMaskMode(CSSPropertyID, FillLayer&, CSSValue&);

    void mapAnimationDelay(Animation&, CSSValue&);
    void mapAnimationDirection(Animation&, CSSValue&);
    void mapAnimationDuration(Animation&, CSSValue&);
    void mapAnimationFillMode(Animation&, CSSValue&);
    void mapAnimationIterationCount(Animation&, CSSValue&);
    void mapAnimationName(Animation&, CSSValue&);
    void mapAnimationPlayState(Animation&, CSSValue&);
    void mapAnimationProperty(Animation&, CSSValue&);
    void mapAnimationTimingFunction(Animation&, CSSValue&);

    void mapNinePieceImage(CSSPropertyID, CSSValue*, NinePieceImage&);
    void mapNinePieceImageSlice(CSSValue&, NinePieceImage&);
    LengthBox mapNinePieceImageQuad(CSSValue&);
    void mapNinePieceImageRepeat(CSSValue&, NinePieceImage&);

private:
    RenderStyle* style() const;
    PassRefPtr<StyleImage> styleImage(CSSValue&);

    // FIXME: This type can merge into BuilderState.
    Style::BuilderState& m_builderState;
};

}
