
#pragma once

#include "CachedFont.h"
#include "CSSCrossfadeValue.h"
#include "CSSFilterImageValue.h"
#include "CSSFontFaceSrcValue.h"
#include "CSSImageSetValue.h"
#include "CSSImageValue.h"
#include "CSSValueList.h"
#include "StyleCachedImage.h"
#include "StyleImage.h"

namespace WebCore
{

template<class Functor>
inline bool CSSFontFaceSrcValue::traverseSubresources(const Functor& handler) const
{
    if (!m_cachedFont)
        return false;
    return handler(*m_cachedFont);
}

template<class Functor>
inline bool CSSImageValue::traverseSubresources(const Functor& handler) const
{
    return m_cachedImage.value_or(nullptr) && handler(**m_cachedImage);
}

template<class Functor>
inline bool CSSImageSetValue::traverseSubresources(const Functor& handler) const
{
    return m_selectedImageValue && m_selectedImageValue->traverseSubresources(handler);
}

template<class Functor>
inline bool CSSValue::traverseSubresources(const Functor& handler) const
{
    if (isValueList())
        return static_cast<const CSSValueList*>(this)->traverseSubresources(handler);
    if (classType() == FontFaceSrcClass)
        return static_cast<const CSSFontFaceSrcValue*>(this)->traverseSubresources(handler);
    if (classType() == ImageClass)
        return static_cast<const CSSImageValue*>(this)->traverseSubresources(handler);
    if (classType() == CrossfadeClass)
        return static_cast<const CSSCrossfadeValue*>(this)->traverseSubresources(handler);
    if (this->isFilterImageValue())
        return static_cast<const CSSFilterImageValue&>(*this).traverseSubresources(handler);
    if (classType() == ImageSetClass)
        return static_cast<const CSSImageSetValue*>(this)->traverseSubresources(handler);
    return false;
}

}
