/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Peter Kelly (pmk@post.com)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "ContainerNode.h"
#include "QualifiedName.h"

namespace WebCore {

class Attribute;
class CSSStyleDeclaration;
class MutableStylePropertySet;

class Attr FINAL : public Node {
public:
    static Ref<Attr> create(Element&, const QualifiedName&);
    static Ref<Attr> create(Document&, const QualifiedName&, const AtomicString& value);
    virtual ~Attr();

    String name() const { return qualifiedName().toString(); }
    bool specified() const { return m_specified; }
    Element* ownerElement() const { return m_element.get(); }

    const AtomicString& value() const;
    void setValue(const AtomicString&);

    const QualifiedName& qualifiedName() const { return m_name; }

    CSSStyleDeclaration* style();

    void setSpecified(bool specified) { m_specified = specified; }

    void attachToElement(Element&);
    void detachFromElementWithValue(const AtomicString&);

    virtual const AtomicString& namespaceURI() const FINAL { return m_name.namespaceURI(); }
    virtual const AtomicString& localName() const FINAL { return m_name.localName(); }
    virtual const AtomicString& prefix() const FINAL { return m_name.prefix(); }

    virtual void setPrefix(const AtomicString&, ExceptionCode&) FINAL;

private:
    Attr(Element&, const QualifiedName&);
    Attr(Document&, const QualifiedName&, const AtomicString& value);

    virtual String nodeName() const FINAL { return name(); }
    virtual NodeType nodeType() const FINAL { return ATTRIBUTE_NODE; }

    virtual String nodeValue() const FINAL { return value(); }
    virtual void setNodeValue(const String&) FINAL;
    virtual Ref<Node> cloneNodeInternal(Document&, CloningOperation) FINAL;

    virtual bool isAttributeNode() const FINAL { return true; }

    // Attr wraps either an element/name, or a name/value pair (when it's a standalone Node.)
    // Note that m_name is always set, but m_element/m_standaloneValue may be null.
    WeakPtr<Element> m_element;
    QualifiedName m_name;
    AtomicString m_standaloneValue;

    RefPtr<MutableStylePropertySet> m_style;
    bool m_specified;
};

NODE_TYPE_CASTS(Attr)

} // namespace WebCore
