/*
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "Event.h"
#include "SerializedScriptValue.h"
#include <runtime/WriteBarrier.h>
#include <wtf/Optional.h>

namespace JSC
{
class SlotVisitor;
}

namespace WebCore {

class CustomEvent FINAL : public Event {
public:
    virtual ~CustomEvent();

    static PassRefPtr<CustomEvent> create(IsTrusted isTrusted = IsTrusted::No)
    {
        return adoptRef(new CustomEvent(isTrusted));
    }

	struct Init : public EventInit {
	    JSC::JSValue detail;
	};

    static PassRefPtr<CustomEvent> create(const AtomicString& type, const Init& initializer, IsTrusted isTrusted = IsTrusted::No)
	{
        return adoptRef(new CustomEvent(type, initializer, isTrusted));
    }

    void initCustomEvent(ScriptState*, const AtomicString& type, bool canBubble, bool cancelable, JSC::JSValue detail = JSC::JSValue());

    virtual EventInterface eventInterface() const FINAL;

    JSC::JSValue detail() const;

    RefPtr<SerializedScriptValue> trySerializeDetail(JSC::ExecState*);

    void visitAdditionalChildren(JSC::SlotVisitor&);

private:
    CustomEvent(IsTrusted);
    CustomEvent(const AtomicString& type, const Init& initializer, IsTrusted);

    JSC::WriteBarrier<JSC::Unknown> m_detail;
    RefPtr<SerializedScriptValue> m_serializedDetail;
    bool m_triedToSerialize;
};

} // namespace WebCore
