/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2009 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "Element.h"
#include "ParserContentPolicy.h"

namespace WebCore {

class ScriptExecutionContext;

class DocumentFragment : public ContainerNode {
public:
    static Ref<DocumentFragment> create(Document*);

    void parseHTML(const String&, Element* contextElement, ParserContentPolicy = AllowScriptingContent);
    bool parseXML(const String&, Element* contextElement, ParserContentPolicy = AllowScriptingContent);
    
    virtual bool canContainRangeEndPoint() const FINAL { return true; }
    virtual bool isTemplateContent() const { return false; }

    void setIsDocumentFragmentForInnerOuterHTML() { setNodeFlag(IsDocumentFragmentForInnerOuterHTML); }

    // From the NonElementParentNode interface - https://dom.spec.whatwg.org/#interface-nonelementparentnode
    Element* getElementById(const AtomicString&) const;

protected:
    DocumentFragment(Document* document, ConstructionType constructionType = CreateContainer)
    	: ContainerNode(document, constructionType)
	{
	}

    virtual String nodeName() const override;

private:
    virtual NodeType nodeType() const FINAL;
    virtual Ref<Node> cloneNodeInternal(Document&, CloningOperation) OVERRIDE;
    virtual bool childTypeAllowed(NodeType) const OVERRIDE;
};

NODE_TYPE_CASTS(DocumentFragment)

inline bool isDocumentFragment(const Node& node)
{
	return node.isDocumentFragment();
}

template <typename ArgType>
struct ElementTypeCastTraits<const DocumentFragment, ArgType> {
    static bool is(ArgType& node) { return isDocumentFragment(node); }
};

} //namespace
