/*
 * Copyright (C) 2005, 2007 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Jon Shier (jshier@iastate.edu)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "ThreadGlobalData.h"
#include <wtf/text/AtomicString.h>

namespace WebCore {

#define DOM_EVENT_NAMES_FOR_EACH(macro) \
    \
    macro(abort) \
    macro(beforecopy) \
    macro(beforecut) \
    macro(beforeinput) \
    macro(beforeload) \
    macro(beforepaste) \
    macro(beforeunload) \
    macro(beginEvent) \
    macro(blocked) \
    macro(blur) \
    macro(bufferedamountlow) \
    macro(cancel) \
    macro(cached) \
    macro(change) \
    macro(chargingchange) \
    macro(chargingtimechange) \
    macro(checking) \
    macro(click) \
    macro(close) \
    macro(complete) \
    macro(compositionend) \
    macro(compositionstart) \
    macro(compositionupdate) \
    macro(connect) \
    macro(connectionstatechange) \
    macro(contextmenu) \
    macro(copy) \
    macro(cut) \
    macro(dblclick) \
    macro(devicemotion) \
    macro(deviceorientation) \
    macro(dischargingtimechange) \
    macro(display) \
    macro(downloading) \
    macro(drag) \
    macro(dragend) \
    macro(dragenter) \
    macro(dragleave) \
    macro(dragover) \
    macro(dragstart) \
    macro(drop) \
    macro(end) \
    macro(endEvent) \
    macro(error) \
    macro(fetch) \
    macro(finish) \
    macro(focus) \
    macro(focusin) \
    macro(focusout) \
    macro(hashchange) \
    macro(input) \
    macro(invalid) \
    macro(keydown) \
    macro(keypress) \
    macro(keystatuseschange) \
    macro(keyup) \
    macro(levelchange) \
    macro(message) \
    macro(mousedown) \
    macro(mouseenter) \
    macro(mouseleave) \
    macro(mousemove) \
    macro(mouseout) \
    macro(mouseover) \
    macro(mouseup) \
    macro(mousewheel) \
    macro(noupdate) \
    macro(obsolete) \
    macro(offline) \
    macro(online) \
    macro(open) \
    macro(overflowchanged) \
    macro(pagehide) \
    macro(pageshow) \
    macro(paste) \
    macro(popstate) \
    macro(readystatechange) \
    macro(rejectionhandled) \
    macro(remove) \
    macro(reset) \
    macro(resize) \
    macro(scroll) \
    macro(search) \
    macro(select) \
    macro(selectstart) \
    macro(selectionchange) \
    macro(slotchange) \
    macro(storage) \
    macro(submit) \
    macro(textInput) \
    macro(track) \
    macro(unload) \
    macro(updateready) \
    macro(upgradeneeded) \
    macro(versionchange) \
    macro(visibilitychange) \
	macro(wheel) \
    macro(write) \
    macro(writeend) \
    macro(writestart) \
    macro(zoom) \
    \
    macro(DOMActivate) \
    macro(DOMCharacterDataModified) \
    macro(DOMNodeInserted) \
    macro(DOMNodeInsertedIntoDocument) \
    macro(DOMNodeRemoved) \
    macro(DOMNodeRemovedFromDocument) \
    macro(DOMSubtreeModified) \
    macro(DOMContentLoaded) \
    \
    macro(webkitBeforeTextInserted) \
    macro(webkitEditableContentChanged) \
    \
    macro(canplay) \
    macro(canplaythrough) \
    macro(durationchange) \
    macro(emptied) \
    macro(encrypted) \
    macro(ended) \
    macro(load) \
    macro(loadeddata) \
    macro(loadedmetadata) \
    macro(loadend) \
    macro(loading) \
    macro(loadingdone) \
    macro(loadingerror) \
    macro(loadstart) \
    macro(pause) \
    macro(play) \
    macro(playing) \
    macro(ratechange) \
    macro(seeked) \
    macro(seeking) \
    macro(timeupdate) \
    macro(toggle) \
    macro(volumechange) \
    macro(waiting) \
    macro(waitingforkey) \
    \
    macro(active) \
    macro(inactive) \
    macro(addtrack) \
    macro(cuechange) \
    macro(enter) \
    macro(exit) \
    \
    macro(webkitbeginfullscreen) \
    macro(webkitendfullscreen) \
    \
    macro(addsourcebuffer) \
    macro(removesourcebuffer) \
    macro(sourceopen) \
    macro(sourceended) \
    macro(sourceclose) \
    macro(unhandledrejection) \
    macro(update) \
    macro(updateend) \
    macro(updatestart) \
    macro(webkitaddsourcebuffer) \
    macro(webkitremovesourcebuffer) \
    macro(webkitsourceopen) \
    macro(webkitsourceended) \
    macro(webkitsourceclose) \
    \
    macro(keyadded) \
    macro(keyerror) \
    macro(keymessage) \
    macro(needkey) \
    \
    macro(webkitkeyadded) \
    macro(webkitkeyerror) \
    macro(webkitkeymessage) \
    macro(webkitneedkey) \
    \
    macro(progress) \
    macro(stalled) \
    macro(suspend) \
    \
    macro(webkitAnimationEnd) \
    macro(webkitAnimationStart) \
    macro(webkitAnimationIteration) \
    macro(animationcancel) \
    macro(animationend) \
    macro(animationiteration) \
    macro(animationstart) \
    \
    macro(webkitTransitionEnd) \
    macro(transitioncancel) \
    macro(transitionend) \
    macro(transitionrun) \
    macro(transitionstart) \
    \
    macro(orientationchange) \
    \
    macro(timeout) \
    \
    macro(touchstart) \
    macro(touchmove) \
    macro(touchend) \
    macro(touchcancel) \
    \
    macro(success) \
    \
    macro(webkitfullscreenchange) \
    macro(webkitfullscreenerror) \
    \
    macro(webkitspeechchange) \
    \
    macro(audiostart) \
    macro(soundstart) \
    macro(speechstart) \
    macro(speechend) \
    macro(soundend) \
    macro(audioend) \
    macro(result) \
    macro(nomatch) \
    macro(start) \
    macro(mark) \
    macro(boundary) \
    macro(resume) \
    \
    macro(webglcontextchanged) \
    macro(webglcontextcreationerror) \
    macro(webglcontextlost) \
    macro(webglcontextrestored) \
    \
    macro(audioprocess) \
    \
    macro(connecting) \
    macro(addstream) \
    macro(removestream) \
    macro(signalingstatechange) \
    macro(removetrack) \
    macro(overconstrained) \
    macro(mute) \
    macro(unmute) \
    macro(started) \
    macro(iceconnectionstatechange) \
    macro(icegatheringstatechange) \
    macro(icecandidate) \
    macro(negotiationneeded) \
    macro(datachannel) \
    macro(tonechange) \
    \
    macro(show) \
    \
    macro(pointerlockchange) \
    macro(pointerlockerror) \
    \
    macro(webkitRegionLayoutUpdate) \
    \
    macro(webkitregionoversetchange) \
    \
    macro(webkitnetworkinfochange) \
    \
    macro(webkitresourcetimingbufferfull) \
    \
    macro(securitypolicyviolation) \
    \

// end of DOM_EVENT_NAMES_FOR_EACH

#if ENABLE(RMAI)
#define DOM_EVENT_NAMES_FOR_EACH_GALIO(macro) \
    \
    macro(ApplicationDeactivated) \
    macro(ApplicationNotPrimaryReceiver) \
    macro(ApplicationPrimaryReceiver) \
    macro(ApplicationActivated) \
    macro(ApplicationHidden) \
    macro(ApplicationShown) \
    macro(ApplicationNotTopmost) \
    macro(ApplicationTopmost) \
    macro(ApplicationLoaded) \
    macro(ApplicationUnloaded)
#else
#define DOM_EVENT_NAMES_FOR_EACH_GALIO(macro)
#endif

class EventNames {
	WTF_MAKE_NONCOPYABLE(EventNames); WTF_MAKE_FAST_ALLOCATED;
	int dummy; // Needed to make initialization macro work.
	// Private to prevent accidental call to EventNames() instead of eventNames()
	EventNames();
	friend class ThreadGlobalData;

public:
	#define DOM_EVENT_NAMES_DECLARE(name) AtomicString name##Event;
	DOM_EVENT_NAMES_FOR_EACH(DOM_EVENT_NAMES_DECLARE)
	DOM_EVENT_NAMES_FOR_EACH_GALIO(DOM_EVENT_NAMES_DECLARE)
	#undef DOM_EVENT_NAMES_DECLARE

	inline bool isTouchEventType(const AtomicString& eventType) const
	{
		return eventType == touchstartEvent
			|| eventType == touchmoveEvent
			|| eventType == touchendEvent
			|| eventType == touchcancelEvent;
	}

	Vector<AtomicString, 4> touchEventNames() const
	{
		Vector<AtomicString, 4> names;
		names.append(touchstartEvent);
		names.append(touchmoveEvent);
		names.append(touchendEvent);
		names.append(touchcancelEvent);
		return names;
	}
};

ALWAYS_INLINE EventNames& eventNames()
{
	return threadGlobalData().eventNames();
}

}
