/*
 * Copyright (C) 2012 Victor Carbune (victor@rosedu.org)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include "EventQueue.h"
#include "EventTarget.h"
#include "Timer.h"
#include <wtf/Deque.h>
#include <wtf/PassOwnPtr.h>
#include <wtf/RefPtr.h>
#include <wtf/WeakPtr.h>

namespace WebCore {

class Element;

class GenericEventQueue : public CanMakeWeakPtr<GenericEventQueue> {
    WTF_MAKE_FAST_ALLOCATED;
public:
    explicit GenericEventQueue(EventTarget&);
    virtual ~GenericEventQueue();

    void enqueueEvent(PassRefPtr<Event>);
    void close();

    void cancelAllEvents();
    bool hasPendingEvents() const;

    void suspend();
    void resume();

    static void dispatchAllEvents();

private:
    static StaticTimer& sharedTimer();
    static void sharedTimerFired();
    static Deque<WeakPtr<GenericEventQueue> >& pendingQueues();

    void dispatchOneEvent();

    virtual EventTarget* owner() const { return m_owner; }

    EventTarget* m_owner;
    Deque<RefPtr<Event>> m_pendingEvents;

    bool m_isClosed { false };
    bool m_isSuspended { false };
};

class GenericEventQueueWithElementOwner final : public GenericEventQueue
{
public:
    explicit GenericEventQueueWithElementOwner(Element&);
    virtual ~GenericEventQueueWithElementOwner();
private:
    virtual EventTarget* owner() const final;
    WeakPtr<Element> m_weakOwner;
};

}
