/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "Element.h"
#include "Event.h"
#include "RenderStyle.h"
#include <wtf/Forward.h>
#include <wtf/WeakPtr.h>

namespace WebCore {

class PseudoElement FINAL : public Element {
public:
    static Ref<PseudoElement> create(Element& host, PseudoId);
    virtual ~PseudoElement();

    Element* hostElement() const { return m_hostElement.get(); }
    void clearHostElement();

    virtual bool rendererIsNeeded(const RenderStyle&) OVERRIDE;

    virtual bool canStartSelection() const OVERRIDE { return false; }
    virtual bool canContainRangeEndPoint() const OVERRIDE { return false; }

private:
    PseudoElement(Element&, PseudoId);

    virtual PseudoId customPseudoId() const OVERRIDE { return m_pseudoId; }

    WeakPtr<Element> m_hostElement;
    PseudoId m_pseudoId;
};

const QualifiedName& pseudoElementTagName();

inline bool isPseudoElement(const Node& node)
{
	return node.isPseudoElement();
}

inline PseudoElement& toPseudoElement(Node& node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(node.isPseudoElement());
    return static_cast<PseudoElement&>(node);
}

inline PseudoElement* toPseudoElement(Node* node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!node || node->isPseudoElement());
    return static_cast<PseudoElement*>(node);
}

inline const PseudoElement& toPseudoElement(const Node& node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(node.isPseudoElement());
    return static_cast<const PseudoElement&>(node);
}

inline const PseudoElement* toPseudoElement(const Node* node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!node || node->isPseudoElement());
    return static_cast<const PseudoElement*>(node);
}

} // namespace
