/*
 * Copyright (C) 2011 Google Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma once

#if ENABLE(REQUEST_ANIMATION_FRAME)
#include "ReducedResolutionSeconds.h"
#include "DOMTimeStamp.h"
#include "Timer.h"
#include <wtf/RefCounted.h>
#include <wtf/RefPtr.h>
#include <wtf/Vector.h>
#include <wtf/WeakPtr.h>

namespace WebCore {

class Document;
class RequestAnimationFrameCallback;

class ScriptedAnimationController : public RefCounted<ScriptedAnimationController>
{
public:
    static PassRefPtr<ScriptedAnimationController> create(Document* document)
    {
        return adoptRef(new ScriptedAnimationController(document));
    }
    ~ScriptedAnimationController();
    void clearDocumentPointer() { m_document = nullptr; }

    typedef int CallbackId;

    CallbackId registerCallback(PassRefPtr<RequestAnimationFrameCallback>);
    void cancelCallback(CallbackId);
    void serviceRequestAnimationFrameCallbacks(ReducedResolutionSeconds timestamp);

    void suspend();
    void resume();
    void setThrottled(bool);
    bool isThrottled() const;

private:
    ScriptedAnimationController(Document*);
    
    typedef Vector<RefPtr<RequestAnimationFrameCallback> > CallbackList;
    CallbackList m_callbacks;

    WeakPtr<Document> m_document;
    CallbackId m_nextCallbackId;
    int m_suspendCount;

    void scheduleAnimation();

#if USE(REQUEST_ANIMATION_FRAME_TIMER)
    void animationTimerFired();
    Timer<ScriptedAnimationController> m_animationTimer;
    ReducedResolutionSeconds m_lastAnimationFrameTimestamp;

#if USE(REQUEST_ANIMATION_FRAME_DISPLAY_MONITOR)
    bool m_isUsingTimer;
    bool m_isThrottled;
#endif
#endif
};

}

#endif // ENABLE(REQUEST_ANIMATION_FRAME)
