/*
 * Copyright (C) 2009, 2011 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include "AllDescendantsCollection.h"
#include <runtime/JSValue.h>
#include <wtf/Optional.h>
//#include <wtf/Variant.h>

namespace JSC {
class ExecState;
}

namespace WebCore {

class JSHTMLAllCollection;

class HTMLAllCollection FINAL : public AllDescendantsCollection {
public:
    static PassRefPtr<HTMLAllCollection> create(Document*, CollectionType);

    //std::optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>>
    JSC::JSValue namedOrIndexedItemOrItems(JSC::ExecState*, JSHTMLAllCollection*, const AtomicString& nameOrIndex) const;
    //std::optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>>
    JSC::JSValue namedItemOrItems(JSC::ExecState*, JSHTMLAllCollection*, const AtomicString&) const;

private:
    HTMLAllCollection(Document*, CollectionType);
};

class HTMLAllNamedSubCollection FINAL : public CachedHTMLCollection<HTMLAllNamedSubCollection, CollectionTraversalType::Descendants> {
public:
    static PassRefPtr<HTMLAllNamedSubCollection> create(Document* document, CollectionType type, const AtomicString& name)
    {
        return adoptRef(new HTMLAllNamedSubCollection(document, type, name));
    }
    virtual ~HTMLAllNamedSubCollection();

    bool elementMatches(Element&) const;

private:
    HTMLAllNamedSubCollection(Document* document, CollectionType type, const AtomicString& name)
        : CachedHTMLCollection<HTMLAllNamedSubCollection, CollectionTraversalType::Descendants>(document, type)
        , m_name(name)
    {
        ASSERT(type == DocumentAllNamedItems);
    }

    AtomicString m_name;
};

} // namespace WebCore

//SPECIALIZE_TYPE_TRAITS_HTMLCOLLECTION(HTMLAllCollection, DocAll)
//SPECIALIZE_TYPE_TRAITS_HTMLCOLLECTION(HTMLAllNamedSubCollection, DocumentAllNamedItems)
