/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2004, 2008, 2009, 2011 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLAnchorElement.h"
#include <wtf/OwnArrayPtr.h>

namespace WebCore {

class HitTestResult;
class HTMLImageElement;
class Path;

class HTMLAreaElement FINAL : public HTMLAnchorElement {
public:
    static Ref<HTMLAreaElement> create(const QualifiedName&, Document*);

    bool isDefault() const { return m_shape == Default; }

    bool mapMouseEvent(LayoutPoint location, const LayoutSize&, HitTestResult&);

    LayoutRect computeRect(RenderElement*) const;
    Path computePath(RenderElement*) const;

    // The parent map's image.
    HTMLImageElement* imageElement() const;

    Path getRegion(const LayoutSize&) const;
    
private:
    HTMLAreaElement(const QualifiedName&, Document*);

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual bool supportsFocus() const FINAL;
    virtual String target() const FINAL;
    virtual bool isKeyboardFocusable(KeyboardEvent*) const OVERRIDE;
    virtual bool isMouseFocusable() const FINAL;
    virtual bool isFocusable() const OVERRIDE;
    virtual void updateFocusAppearance(SelectionRestorationMode, SelectionRevealMode) FINAL;
    virtual void setFocus(bool, FocusVisibility = FocusVisibility::Invisible) FINAL;

#if ENABLE(MICRODATA)
    virtual String itemValueText() const OVERRIDE;
    virtual void setItemValueText(const String&, ExceptionCode&) OVERRIDE;
#endif

    enum Shape { Default, Poly, Rect, Circle, Unknown };
    void invalidateCachedRegion();

    OwnPtr<Path> m_region;
    Vector<double> m_coords;
    LayoutSize m_lastSize;
    Shape m_shape;
};

inline bool isHTMLAreaElement(const Node& node)
{
    return node.hasTagName(HTMLNames::areaTag);
}

inline bool isHTMLAreaElement(const Element& element)
{
    return element.hasTagName(HTMLNames::areaTag);
}

NODE_TYPE_CASTS(HTMLAreaElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLAreaElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLAreaElement(node); }
};

} //namespace
