/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2003, 2006, 2009, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {

class HTMLBRElement FINAL : public HTMLElement {
public:
    static Ref<HTMLBRElement> create(Document*);
    static Ref<HTMLBRElement> create(const QualifiedName&, Document*);

    virtual bool canContainRangeEndPoint() const { return false; }

private:
    HTMLBRElement(const QualifiedName&, Document*);

    virtual bool isPresentationAttribute(const QualifiedName&) const OVERRIDE;
    virtual void collectStyleForPresentationAttribute(const QualifiedName&, const AtomicString&, MutableStylePropertySet*) override;

    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&);
};

inline bool isHTMLBRElement(const Node* node)
{
    return node && node->hasTagName(HTMLNames::brTag);
}

inline bool isHTMLBRElement(const Node& node)
{
    return node.hasTagName(HTMLNames::brTag);
}

inline bool isHTMLBRElement(const Element& element)
{
    return element.hasTagName(HTMLNames::brTag);
}

inline bool isHTMLBRElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::brTag);
}

NODE_TYPE_CASTS(HTMLBRElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLBRElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLBRElement(node); }
};

} // namespace
