/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2004, 2006, 2009, 2010, 2015 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {

class HTMLBodyElement FINAL : public HTMLElement {
public:
    static Ref<HTMLBodyElement> create(Document*);
    static Ref<HTMLBodyElement> create(const QualifiedName&, Document*);
    virtual ~HTMLBodyElement();

    static const AtomicString& eventNameForWindowEventHandlerAttribute(const QualifiedName& attributeName);

private:
    HTMLBodyElement(const QualifiedName&, Document*);

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual bool isPresentationAttribute(const QualifiedName&) const OVERRIDE;
    virtual void collectStyleForPresentationAttribute(const QualifiedName&, const AtomicString&, MutableStylePropertySet*) override;

    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual void didFinishInsertingNode() FINAL;
    
    virtual bool isURLAttribute(const Attribute&) const OVERRIDE;
    
    virtual bool supportsFocus() const final;
    
    virtual void addSubresourceAttributeURLs(ListHashSet<KURL>&) const final;

    static EventHandlerNameMap createWindowEventHandlerNameMap();
};

inline bool isHTMLBodyElement(const Node& node)
{
    return node.hasTagName(HTMLNames::bodyTag);
}

inline bool isHTMLBodyElement(const Element& element)
{
    return element.hasTagName(HTMLNames::bodyTag);
}

inline bool isHTMLBodyElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::bodyTag);
}

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLBodyElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLBodyElement(node); }
};

} //namespace
