/*
 * Copyright (C) 2010, 2011 Nokia Corporation and/or its subsidiary(-ies)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {

class HTMLDetailsElement;
class HTMLSlotElement;
class HTMLSummaryElement;

template<typename T> class EventSender;
typedef EventSender<HTMLDetailsElement> DetailEventSender;

class HTMLDetailsElement FINAL : public HTMLElement {
public:
    static PassRefPtr<HTMLDetailsElement> create(const QualifiedName& tagName, Document* document);
    virtual ~HTMLDetailsElement();

    void toggleOpen();

    bool isOpen() const { return m_isOpen; }
    bool isActiveSummary(const HTMLSummaryElement&) const;

    void dispatchPendingEvent(DetailEventSender*, const AtomString& eventType);

private:
    HTMLDetailsElement(const QualifiedName&, Document*);

    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&);
    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;

    virtual void didAddUserAgentShadowRoot(ShadowRoot*) OVERRIDE;
    virtual bool canHaveUserAgentShadowRoot() const OVERRIDE FINAL { return true; }
    virtual bool hasCustomFocusLogic() const FINAL { return true; }
    bool isInteractiveContent() const final { return true; }

    bool m_isOpen { false };
    WeakPtr<HTMLSlotElement> m_summarySlot;
	WeakPtr<HTMLSummaryElement> m_defaultSummary;
    RefPtr<HTMLSlotElement> m_defaultSlot;
};

inline bool isHTMLDetailsElement(const Node& node)
{
    return node.hasTagName(HTMLNames::detailsTag);
}

inline bool isHTMLDetailsElement(const Element& element)
{
    return element.hasTagName(HTMLNames::detailsTag);
}

NODE_TYPE_CASTS(HTMLDetailsElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLDetailsElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLDetailsElement(node); }
};

} // namespace WebCore
