/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {

class HTMLDivElement : public HTMLElement {
public:
    static Ref<HTMLDivElement> create(Document*);
    static Ref<HTMLDivElement> create(Document&);
    static Ref<HTMLDivElement> create(const QualifiedName&, Document*);

protected:
    HTMLDivElement(const QualifiedName&, Document*);

private:
    virtual bool isPresentationAttribute(const QualifiedName&) const OVERRIDE;
    virtual void collectStyleForPresentationAttribute(const QualifiedName&, const AtomicString&, MutableStylePropertySet*) override;
};

inline bool isHTMLDivElement(const Node& node)
{
    return node.hasTagName(HTMLNames::divTag);
}

inline bool isHTMLDivElement(const Element& element)
{
    return element.hasTagName(HTMLNames::divTag);
}

inline bool isHTMLDivElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::divTag);
}

NODE_TYPE_CASTS(HTMLDivElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLDivElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLDivElement(node); }
};

} // namespace WebCore
