/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2004, 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "Document.h"
#include <runtime/JSValue.h>
#include <wtf/HashCountedSet.h>
#include <wtf/text/AtomicStringHash.h>

namespace JSC {
class ExecState;
}

namespace WebCore {

class FrameView;
class HTMLElement;
class JSHTMLDocument;

class HTMLDocument : public Document {
	WTF_MAKE_FAST_ALLOCATED;
public:
    static Ref<HTMLDocument> create(Frame* frame, const KURL& url)
    {
        return adoptRef(*new HTMLDocument(frame, url, HTMLDocumentClass));
    }

    static Ref<HTMLDocument> createSynthesizedDocument(Frame* frame, const URL& url)
    {
        return adoptRef(*new HTMLDocument(frame, url, HTMLDocumentClass, Synthesized));
    }

    virtual ~HTMLDocument();

    int width();
    int height();

    JSC::JSValue /*std::optional<Variant<RefPtr<DOMWindow>, RefPtr<Element>, RefPtr<HTMLCollection>>> */ namedItem(JSC::ExecState*, JSHTMLDocument*, const AtomicString&);
    Vector<AtomicString> supportedPropertyNames() const;

    Element* documentNamedItem(const AtomicStringImpl* name) const { return m_documentNamedItem.getElementByDocumentNamedItem(name, this); }
    bool hasDocumentNamedItem(const AtomicStringImpl* name) const { return m_documentNamedItem.contains(name); }
    bool documentNamedItemContainsMultipleElements(const AtomicStringImpl* name) const { return m_documentNamedItem.containsMultiple(name); }
    void addDocumentNamedItem(const AtomicString&, Element*);
    void removeDocumentNamedItem(const AtomicString&, Element*);

    Element* windowNamedItem(const AtomicStringImpl* name) const { return m_windowNamedItem.getElementByWindowNamedItem(name, this); }
    bool hasWindowNamedItem(const AtomicStringImpl* name) const { return m_windowNamedItem.contains(name); }
    bool windowNamedItemContainsMultipleElements(const AtomicStringImpl* name) const { return m_windowNamedItem.containsMultiple(name); }
    void addWindowNamedItem(const AtomicString&, Element*);
    void removeWindowNamedItem(const AtomicString&, Element*);

    static bool isCaseSensitiveAttribute(const QualifiedName&);

protected:
    HTMLDocument(Frame*, const KURL&, DocumentClassFlags = 0, unsigned constructionFlags = 0);

private:
    virtual bool isFrameSet() const OVERRIDE;
    virtual PassRefPtr<DocumentParser> createParser() OVERRIDE;

    virtual Ref<Document> cloneDocumentWithoutChildren() const FINAL;

    DocumentOrderedMap m_documentNamedItem;
    DocumentOrderedMap m_windowNamedItem;
};

inline bool isHTMLDocument(const Node& node)
{
	return isDocument(node) && toDocument(node).isHTMLDocument();
}

inline bool isHTMLDocument(const Document& document)
{
	return document.isHTMLDocument();
}

NODE_TYPE_CASTS(HTMLDocument)

} // namespace WebCore
