/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2004, 2006, 2008, 2009 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef HTMLEmbedElement_h
#define HTMLEmbedElement_h

#include "HTMLPlugInImageElement.h"

namespace WebCore {

class HTMLEmbedElement FINAL : public HTMLPlugInImageElement {
public:
    static Ref<HTMLEmbedElement> create(const QualifiedName&, Document*);

private:
    HTMLEmbedElement(const QualifiedName&, Document*);

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual bool isPresentationAttribute(const QualifiedName&) const OVERRIDE;
    virtual void collectStyleForPresentationAttribute(const QualifiedName&, const AtomicString&, MutableStylePropertySet*) override;

    virtual bool rendererIsNeeded(const RenderStyle&) OVERRIDE;

    virtual bool isURLAttribute(const Attribute&) const OVERRIDE;
    virtual const AtomicString& imageSourceURL() const OVERRIDE;

    bool isInteractiveContent() const final { return true; }

    virtual RenderWidget* renderWidgetLoadingPlugin() const final;

    virtual void updateWidget(PluginCreationOption) final;

    virtual void addSubresourceAttributeURLs(ListHashSet<KURL>&) const final;

    void parametersForPlugin(Vector<String>& paramNames, Vector<String>& paramValues);

#if ENABLE(MICRODATA)
    virtual String itemValueText() const OVERRIDE;
    virtual void setItemValueText(const String&, ExceptionCode&) OVERRIDE;
#endif
};

inline bool isHTMLEmbedElement(const Node& node)
{
    return node.hasTagName(HTMLNames::embedTag);
}

inline bool isHTMLEmbedElementt(const Element& element)
{
    return element.hasTagName(HTMLNames::embedTag);
}

inline bool isHTMLEmbedElementt(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::embedTag);
}

inline HTMLEmbedElement* toHTMLEmbedElement(Node* node)
{
    ASSERT(!node || isHTMLEmbedElement(node));
    return static_cast<HTMLEmbedElement*>(node);
}

inline HTMLEmbedElement& toHTMLEmbedElementt(Node& node)
{
    ASSERT(isHTMLEmbedElement(node));
    return static_cast<HTMLEmbedElement&>(node);
}

}

#endif
