/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLFormControlElement.h"
#include <wtf/WeakHashSet.h>
#include <wtf/OwnPtr.h>

namespace WebCore {

class FormAssociatedElement;
class HTMLCollection;

class HTMLFieldSetElement FINAL : public HTMLFormControlElement {
public:
    static Ref<HTMLFieldSetElement> create(const QualifiedName&, Document*, HTMLFormElement*);
    HTMLLegendElement* legend() const;

    PassRefPtr<HTMLCollection> elements();

    void addInvalidDescendant(const HTMLFormControlElement&);
    void removeInvalidDescendant(const HTMLFormControlElement&);

private:
    HTMLFieldSetElement(const QualifiedName&, Document*, HTMLFormElement*);
    virtual ~HTMLFieldSetElement();

    virtual bool isDisabledFormControl() const final;
    virtual bool isActuallyDisabled() const final;
    virtual bool isEnumeratable() const override { return true; }
    virtual bool supportsFocus() const override;
    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&) override;
    virtual const AtomicString& formControlType() const override;
    virtual bool computeWillValidate() const override { return false; }
    virtual void disabledAttributeChanged() OVERRIDE;
    virtual void disabledStateChanged() OVERRIDE;
    virtual void childrenChanged(const ChildChange&) OVERRIDE;
    virtual void didMoveToNewDocument(Document& oldDocument, Document& newDocument) OVERRIDE;

    virtual bool matchesValidPseudoClass() const OVERRIDE;
    virtual bool matchesInvalidPseudoClass() const OVERRIDE;

    WeakHashSet<HTMLFormControlElement> m_invalidDescendants;
    bool m_hasDisabledAttribute;
};

inline bool isHTMLFieldSetElement(const Node& node)
{
    return node.hasTagName(HTMLNames::fieldsetTag);
}

inline bool isHTMLFieldSetElement(const Element& element)
{
    return element.hasTagName(HTMLNames::fieldsetTag);
}

inline HTMLFieldSetElement* toHTMLFieldSetElement(Node* node)
{
    ASSERT(!node || isHTMLFieldSetElement(node));
    return static_cast<HTMLFieldSetElement*>(node);
}

inline HTMLFieldSetElement& toHTMLFieldSetElement(Node& node)
{
    ASSERT(isHTMLFieldSetElement(node));
    return static_cast<HTMLFieldSetElement&>(node);
}

} // namespace
