/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2004, 2006, 2008, 2009 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "FrameLoaderTypes.h"
#include "HTMLFrameOwnerElement.h"
#include "ScrollTypes.h"

namespace JSC {
class ExecState;
}

namespace WebCore {

class HTMLFrameElementBase : public HTMLFrameOwnerElement {
public:
    void setLocation(JSC::ExecState*, const String&);

    virtual ScrollbarMode scrollingMode() const final;

protected:
    HTMLFrameElementBase(const QualifiedName&, Document*);

    bool canLoad() const;

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual void didFinishInsertingNode() OVERRIDE;
    virtual void didAttachRenderers() OVERRIDE;

    void setLocation(const String&);

private:
    bool canLoadScriptURL(const URL&) const final;

    bool canLoadURL(const String& relativeURL) const;
    bool canLoadURL(const URL&) const;

    bool canContainRangeEndPoint() const final { return false; }

    virtual bool supportsFocus() const FINAL;
    virtual void setFocus(bool, FocusVisibility = FocusVisibility::Invisible) FINAL;
    
    virtual bool isURLAttribute(const Attribute&) const OVERRIDE;

    void openURL(LockHistory = LockHistory::Yes, LockBackForwardList = LockBackForwardList::Yes);

    AtomicString m_URL;
};

inline bool isHTMLFrameElement(const Node& node)
{
    return node.hasTagName(HTMLNames::frameTag);
}

inline bool isHTMLFrameElement(const Element& element)
{
    return element.hasTagName(HTMLNames::frameTag);
}

inline bool isHTMLFrameElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::frameTag);
}

inline bool isHTMLIFrameElement(const Node& node)
{
    return node.hasTagName(HTMLNames::iframeTag);
}

inline bool isHTMLIFrameElement(const Element& element)
{
    return element.hasTagName(HTMLNames::iframeTag);
}

inline bool isHTMLIFrameElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::iframeTag);
}

inline bool isHTMLFrameElementBase(const Node& node)
{
    return isHTMLFrameElement(node) || isHTMLIFrameElement(node);
}

inline bool isHTMLFrameElementBase(const Element& element)
{
    return isHTMLFrameElement(element) || isHTMLIFrameElement(element);
}

inline bool isHTMLFrameElementBase(const HTMLElement& element)
{
    return isHTMLFrameElement(element) || isHTMLIFrameElement(element);
}

inline bool isHTMLFrameElementBase(const Element* element)
{
    return element && isHTMLFrameElementBase(*element);
}

inline bool isHTMLFrameElementBase(const HTMLElement* element)
{
    return element && isHTMLFrameElementBase(*element);
}

NODE_TYPE_CASTS(HTMLFrameElementBase)

#if 0
SPECIALIZE_TYPE_TRAITS_BEGIN(HTMLFrameElementBase)
    static bool isHTMLFrameElementBase(const HTMLElement& element) { return is<HTMLFrameElement>(element) || is<HTMLIFrameElement>(element); }
    static bool isHTMLFrameElementBase(const Node& node) { return is<HTMLElement>(node) && isHTMLFrameElementBase(downcast<HTMLElement>(node)); }
SPECIALIZE_TYPE_TRAITS_END()
#endif

} // namespace WebCore
