/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2004, 2006, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {

class HTMLHeadElement FINAL : public HTMLElement {
public:
    static Ref<HTMLHeadElement> create(Document*);
    static Ref<HTMLHeadElement> create(const QualifiedName&, Document*);

private:
    HTMLHeadElement(const QualifiedName&, Document*);
};

inline bool isHTMLHeadElement(const Node& node)
{
    return node.hasTagName(HTMLNames::headTag);
}

inline bool isHTMLHeadElement(const Element& element)
{
    return element.hasTagName(HTMLNames::headTag);
}

NODE_TYPE_CASTS(HTMLHeadElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLHeadElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLHeadElement(node); }
};

} // namespace
