/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"
#include "LabelableElement.h"

namespace WebCore {

class HTMLLabelElement FINAL : public HTMLElement {
public:
    static PassRefPtr<HTMLLabelElement> create(const QualifiedName&, Document*);

    LabelableElement* control();
    virtual HTMLFormElement* form() const FINAL;

    virtual bool willRespondToMouseClickEvents() OVERRIDE;

private:
    HTMLLabelElement(const QualifiedName&, Document*);

    bool isEventTargetedAtInteractiveDescendants(Event&) const;

    virtual bool accessKeyAction(bool sendMouseEvents) FINAL;

    // Overridden to update the hover/active state of the corresponding control.
    virtual void setActive(bool, bool pause) final;
    virtual void setHovered(bool) final;

    // Overridden to either click() or focus() the corresponding control.
    virtual void defaultEventHandler(Event*) FINAL;

    virtual void focus(const FocusOptions& = FocusOptions()) FINAL;

    bool isInteractiveContent() const final { return true; }

    bool m_processingClick { false };
};

inline bool isHTMLLabelElement(const Node& node)
{
    return node.hasTagName(HTMLNames::labelTag);
}

inline bool isHTMLLabelElement(const Element& element)
{
    return element.hasTagName(HTMLNames::labelTag);
}

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLLabelElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLLabelElement(node); }
};

inline HTMLLabelElement* toHTMLLabelElement(Node* node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!node || isHTMLLabelElement(node));
    return static_cast<HTMLLabelElement*>(node);
}

inline HTMLLabelElement& toHTMLLabelElement(Node& node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(isHTMLLabelElement(node));
    return static_cast<HTMLLabelElement&>(node);
}

} //namespace
