/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2003, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef HTMLMetaElement_h
#define HTMLMetaElement_h

#include "HTMLElement.h"

namespace WebCore {

class HTMLMetaElement FINAL : public HTMLElement {
public:
    static PassRefPtr<HTMLMetaElement> create(const QualifiedName&, Document*);

    String content() const;
    String httpEquiv() const;
    String name() const;

private:
    HTMLMetaElement(const QualifiedName&, Document*);

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;

    void process();

#if ENABLE(MICRODATA)
    virtual String itemValueText() const OVERRIDE;
    virtual void setItemValueText(const String&, ExceptionCode&) OVERRIDE;
#endif

};

inline bool isHTMLMetaElement(const Node& node)
{
    return node.hasTagName(HTMLNames::metaTag);
}

inline bool isHTMLMetaElement(const Element& element)
{
    return element.hasTagName(HTMLNames::metaTag);
}

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLMetaElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLMetaElement(node); }
};

inline HTMLMetaElement* toHTMLMetaElement(Node* node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!node || isHTMLMetaElement(node));
    return static_cast<HTMLMetaElement*>(node);
}

inline HTMLMetaElement& toHTMLMetaElement(Node& node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(isHTMLMetaElement(node));
    return static_cast<HTMLMetaElement&>(node);
}

} //namespace

#endif
