/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {
    
class HTMLSelectElement;

class HTMLOptGroupElement FINAL : public HTMLElement {
public:
    static Ref<HTMLOptGroupElement> create(const QualifiedName&, Document*);

    virtual bool isDisabledFormControl() const OVERRIDE;
    HTMLSelectElement* ownerSelectElement() const;
    
    String groupLabelText() const;

private:
    HTMLOptGroupElement(const QualifiedName&, Document*);

    const AtomicString& formControlType() const;
    virtual bool isFocusable() const OVERRIDE;
    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual bool rendererIsNeeded(const RenderStyle&) OVERRIDE { return false; }

    virtual void childrenChanged(const ChildChange&) OVERRIDE;

    virtual bool accessKeyAction(bool sendMouseEvents) OVERRIDE;

    void recalcSelectOptions();

    bool m_isDisabled;
};

inline bool isHTMLOptGroupElement(const Node& node)
{
    return node.hasTagName(HTMLNames::optgroupTag);
}

inline bool isHTMLOptGroupElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::optgroupTag);
}

NODE_TYPE_CASTS(HTMLOptGroupElement)

} //namespace
