/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2010, 2011 Apple Inc. All rights reserved.
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"

namespace WebCore {

class HTMLDataListElement;
class HTMLSelectElement;

enum class AllowStyleInvalidation { Yes, No };

class HTMLOptionElement FINAL : public HTMLElement {
public:
    static Ref<HTMLOptionElement> create(Document*);
    static Ref<HTMLOptionElement> create(const QualifiedName&, Document*);
    static PassRefPtr<HTMLOptionElement> createForJSConstructor(Document*, const String& data, const String& value, bool defaultSelected, bool selected, ExceptionCode&);

    virtual String text() const;
    void setText(const String&);

    int index() const;

    String value() const;
    void setValue(const String&);

    bool selected(AllowStyleInvalidation = AllowStyleInvalidation::Yes) const;
    void setSelected(bool);

    HTMLSelectElement* ownerSelectElement() const;

    String label() const;
    void setLabel(const String&);

    bool ownElementDisabled() const { return m_disabled; }

    virtual bool isDisabledFormControl() const;

    String textIndentedToRespectGroupLabel() const;

    void setSelectedState(bool, AllowStyleInvalidation = AllowStyleInvalidation::Yes);
    bool selectedWithoutUpdate() const { return m_isSelected; }

private:
    HTMLOptionElement(const QualifiedName&, Document*);

    virtual bool supportsFocus() const;
    virtual bool isFocusable() const OVERRIDE;
    virtual bool rendererIsNeeded(const RenderStyle&) OVERRIDE { return false; }
    virtual bool matchesDefaultPseudoClass() const FINAL;

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;

    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual bool accessKeyAction(bool) override;

    virtual void childrenChanged(const ChildChange&) OVERRIDE;

    virtual void willResetComputedStyle() OVERRIDE;

    String collectOptionInnerText() const;

    bool m_disabled;
    bool m_isSelected;
    bool m_isDefault;
};

inline bool isHTMLOptionElement(const Node& node)
{
    return node.hasTagName(HTMLNames::optionTag);
}

inline bool isHTMLOptionElement(const Element& element)
{
    return element.hasTagName(HTMLNames::optionTag);
}

inline bool isHTMLOptionElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::optionTag);
}

NODE_TYPE_CASTS(HTMLOptionElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLOptionElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLOptionElement(node); }
};

} // namespace
