/*
 * Copyright (C) 2006, 2011, 2012 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#include "config.h"
#include "HTMLOptionsCollection.h"

#include "ExceptionCode.h"
#include "HTMLOptionElement.h"
#include "HTMLSelectElement.h"

namespace WebCore {

inline HTMLOptionsCollection::HTMLOptionsCollection(ContainerNode* select)
	: CachedHTMLCollection<HTMLOptionsCollection, CollectionTypeTraits<SelectOptions>::traversalType>(select, SelectOptions)
{
    ASSERT(select->hasTagName(HTMLNames::selectTag));
}

PassRefPtr<HTMLOptionsCollection> HTMLOptionsCollection::create(ContainerNode* select, CollectionType)
{
    return adoptRef(new HTMLOptionsCollection(select));
}

HTMLSelectElement* HTMLOptionsCollection::selectElement()
{
	return toHTMLSelectElement(&ownerNode());
}

const HTMLSelectElement* HTMLOptionsCollection::selectElement() const
{
	return toHTMLSelectElement(&ownerNode());
}

void HTMLOptionsCollection::add(HTMLElement* element, HTMLElement* beforeElement, ExceptionCode& ec)
{
    if (!element)
        return;
    selectElement()->add(*element, beforeElement, ec);
}

void HTMLOptionsCollection::add(HTMLElement* element, int beforeIndex, ExceptionCode& ec)
{
    add(element, item(beforeIndex), ec);
}

void HTMLOptionsCollection::remove(int index)
{
    toHTMLSelectElement(ownerNode()).removeByIndex(index);
}

int HTMLOptionsCollection::selectedIndex() const
{
    return toHTMLSelectElement(ownerNode()).selectedIndex();
}

void HTMLOptionsCollection::setSelectedIndex(int index)
{
    toHTMLSelectElement(ownerNode()).setSelectedIndex(index);
}

void HTMLOptionsCollection::setLength(unsigned length, ExceptionCode& ec)
{
    toHTMLSelectElement(ownerNode()).setLength(length, ec);
}

HTMLOptionElement* HTMLOptionsCollection::item(unsigned offset) const
{
    return toHTMLOptionElement(Base::item(offset));
}

HTMLOptionElement* HTMLOptionsCollection::namedItem(const AtomicString& name) const
{
    return toHTMLOptionElement(Base::namedItem(name));
}

void HTMLOptionsCollection::setItem(unsigned index, HTMLOptionElement* optionElement, ExceptionCode& ec)
{
    selectElement()->setItem(index, optionElement, ec);
}

bool HTMLOptionsCollection::elementMatches(Element& element) const
{
    if (!element.hasTagName(HTMLNames::optionTag))
        return false;

    if (element.parentNode() == selectElement())
        return true;

    ASSERT(element.parentNode());
    return element.parentNode()->hasTagName(HTMLNames::optgroupTag) && element.parentNode()->parentNode() == selectElement();
}

} //namespace
