/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
 * Copyright (C) 2008 Nikolas Zimmermann <zimmermann@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "HTMLElement.h"
#include "ScriptElement.h"

namespace WebCore {

class HTMLScriptElement FINAL : public HTMLElement, public ScriptElement {
	WTF_MAKE_FAST_ALLOCATED;
public:
    static Ref<HTMLScriptElement> create(const QualifiedName&, Document*, bool wasInsertedByParser, bool alreadyStarted = false);

    String text() const { return scriptContent(); }
    void setText(const String&);

    KURL src() const;

    void setAsync(bool);
    bool async() const;

    void setCrossOrigin(const AtomicString&);
    String crossOrigin() const;

    void setReferrerPolicyForBindings(const AtomString&);
    String referrerPolicyForBindings() const;
    ReferrerPolicy referrerPolicy() const final;

private:
    HTMLScriptElement(const QualifiedName&, Document*, bool wasInsertedByParser, bool alreadyStarted);

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason = ModifiedDirectly) final;
    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) FINAL;
    virtual void didFinishInsertingNode() FINAL;
    virtual void childrenChanged(const ChildChange&) FINAL;

    virtual bool isURLAttribute(const Attribute&) const FINAL;

    virtual void addSubresourceAttributeURLs(ListHashSet<KURL>&) const FINAL;

    virtual String sourceAttributeValue() const FINAL;
    virtual String charsetAttributeValue() const FINAL;
    virtual String typeAttributeValue() const FINAL;
    virtual String languageAttributeValue() const FINAL;
    virtual String forAttributeValue() const FINAL;
    virtual String eventAttributeValue() const FINAL;
    virtual bool hasAsyncAttribute() const FINAL;
    virtual bool hasDeferAttribute() const FINAL;
    virtual bool hasNoModuleAttribute() const FINAL;
    virtual bool hasSourceAttribute() const FINAL;

    virtual void dispatchLoadEvent() FINAL;

    virtual Ref<Element> cloneElementWithoutAttributesAndChildren(Document&) FINAL;
};

inline bool isHTMLScriptElement(const Node& node)
{
    return node.hasTagName(HTMLNames::scriptTag);
}

inline bool isHTMLScriptElement(const Element& element)
{
    return element.hasTagName(HTMLNames::scriptTag);
}

NODE_TYPE_CASTS(HTMLScriptElement)

} //namespace
