/*
 * Copyright (C) 2007, 2008, 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include "HTMLElement.h"
#include "MediaList.h"
#include "Timer.h"

namespace WebCore {

class HTMLSourceElement FINAL : public HTMLElement, public ActiveDOMObject {
public:
    static PassRefPtr<HTMLSourceElement> create(const QualifiedName&, Document*);
    
    void scheduleErrorEvent();
    void cancelPendingErrorEvent();

    MediaQuerySet* mediaQuerySet() const { return m_mediaQuerySet.get(); }

private:
    HTMLSourceElement(const QualifiedName&, Document*);
    
    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual void removedFromAncestor(const RemovalType&, ContainerNode&) OVERRIDE;
    virtual bool isURLAttribute(const Attribute&) const OVERRIDE;

    virtual void attributeChanged(const QualifiedName&, const AtomicString& oldValue, const AtomicString& newValue, AttributeModificationReason = ModifiedDirectly) final;

    // ActiveDOMObject.
    virtual const char* activeDOMObjectName() const OVERRIDE;
    virtual bool canSuspendForDocumentSuspension() const OVERRIDE;
    virtual void suspend(ReasonForSuspension) OVERRIDE;
    virtual void resume() OVERRIDE;
    virtual void stop() OVERRIDE;

    void errorEventTimerFired();

#if ENABLE(MICRODATA)
    virtual String itemValueText() const OVERRIDE;
    virtual void setItemValueText(const String&, ExceptionCode&) OVERRIDE;
#endif

    Timer<HTMLSourceElement> m_errorEventTimer;
    bool m_shouldRescheduleErrorEventOnResume { false };
    bool m_shouldCallSourcesChanged { false };
    RefPtr<MediaQuerySet> m_mediaQuerySet;
};

inline bool isHTMLSourceElement(const Node& node)
{
    return node.hasTagName(HTMLNames::sourceTag);
}

inline bool isHTMLSourceElement(const Element& element)
{
    return element.hasTagName(HTMLNames::sourceTag);
}

NODE_TYPE_CASTS(HTMLSourceElement)

} //namespace
