/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2003, 2010 Apple Inc. ALl rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "InlineStyleSheetOwner.h"
#include "HTMLElement.h"

namespace WebCore {

class HTMLStyleElement;
class Page;
class StyleSheet;

template<typename T> class EventSender;
typedef EventSender<HTMLStyleElement> StyleEventSender;

class HTMLStyleElement FINAL : public HTMLElement {
public:
    static Ref<HTMLStyleElement> create(const QualifiedName&, Document*, bool createdByParser);
    virtual ~HTMLStyleElement();

    CSSStyleSheet* sheet() const { return m_styleSheetOwner.sheet(); }

    bool disabled() const;
    void setDisabled(bool);

    void dispatchPendingEvent(StyleEventSender*, const AtomString& eventType);
    static void dispatchPendingLoadEvents(Page*);

private:
    HTMLStyleElement(const QualifiedName&, Document*, bool createdByParser);

    // overload from HTMLElement
    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual void removedFromAncestor(const RemovalType&, ContainerNode&) OVERRIDE;
    virtual void childrenChanged(const ChildChange&) OVERRIDE;

    virtual void finishParsingChildren();

    virtual bool isLoading() const { return m_styleSheetOwner.isLoading(); }
    virtual bool sheetLoaded() { return m_styleSheetOwner.sheetLoaded(*this); }
    virtual void notifyLoadedSheetAndAllCriticalSubresources(bool errorOccurred);
    virtual void startLoadingDynamicSheet() { m_styleSheetOwner.startLoadingDynamicSheet(*this); }

    virtual void addSubresourceAttributeURLs(ListHashSet<KURL>&) const;

    InlineStyleSheetOwner m_styleSheetOwner;
    bool m_firedLoad;
    bool m_loadedSheet;
};

inline bool isHTMLStyleElement(const Node& node)
{
    return node.hasTagName(HTMLNames::styleTag);
}

inline bool isHTMLStyleElement(const Element& element)
{
    return element.hasTagName(HTMLNames::styleTag);
}

inline bool isHTMLStyleElement(const HTMLElement& element)
{
    return element.hasTagName(HTMLNames::styleTag);
}

NODE_TYPE_CASTS(HTMLStyleElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLStyleElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLStyleElement(node); }
};

} //namespace
