/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2003, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */
#pragma once

#include "HTMLElement.h"
#include "StringWithDirection.h"

namespace WebCore {

class HTMLTitleElement FINAL : public HTMLElement {
public:
    static Ref<HTMLTitleElement> create(const QualifiedName&, Document*);

    String text() const;
    void setText(const String&);

    const StringWithDirection& textWithDirection() const { return m_title; }

private:
    HTMLTitleElement(const QualifiedName&, Document*);

    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual void removedFromAncestor(const RemovalType&, ContainerNode&) OVERRIDE;
    virtual void childrenChanged(const ChildChange&) OVERRIDE;

    StringWithDirection computedTextWithDirection();

    StringWithDirection m_title;
};

inline bool isHTMLTitleElement(const Node& node)
{
    return node.hasTagName(HTMLNames::titleTag);
}

inline bool isHTMLTitleElement(const Element& element)
{
    return element.hasTagName(HTMLNames::titleTag);
}

NODE_TYPE_CASTS(HTMLTitleElement)

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLTitleElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLTitleElement(node); }
};

} //namespace
