/*
 * Copyright (C) 2017 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "GraphicsTypes.h"
#include "IntSize.h"
#include "ScriptWrappable.h"
#include <wtf/Optional.h>
#include <wtf/OwnPtr.h>
#include <wtf/PassOwnPtr.h>
#include <wtf/RefCounted.h>
//#include <wtf/Variant.h>

namespace WebCore {

class Blob;
class DeferredPromise;
class HTMLCanvasElement;
class HTMLImageElement;
class HTMLVideoElement;
class ImageBitmapImageObserver;
class ImageBuffer;
class ImageData;
class IntRect;
class KURL;
class PendingImageBitmap;
class ScriptExecutionContext;
struct ImageBitmapOptions;

class ImageBitmap : public ScriptWrappable, public RefCounted<ImageBitmap> {
public:
	/*
    using Source = Variant<
        RefPtr<HTMLImageElement>,
#if ENABLE(VIDEO)
        RefPtr<HTMLVideoElement>,
#endif
        RefPtr<HTMLCanvasElement>,
        RefPtr<ImageBitmap>,
        RefPtr<Blob>,
        RefPtr<ImageData>
    >;

    using Promise = DOMPromiseDeferred<IDLInterface<ImageBitmap>>;
    */

    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLImageElement>, const ImageBitmapOptions&, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLCanvasElement>, const ImageBitmapOptions&, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLVideoElement>, const ImageBitmapOptions&, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<ImageBitmap>, const ImageBitmapOptions&, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<Blob>, const ImageBitmapOptions&, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<ImageData>, const ImageBitmapOptions&, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLImageElement>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLCanvasElement>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLVideoElement>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<ImageBitmap>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<Blob>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<ImageData>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);

    static Ref<ImageBitmap> create(IntSize);
    static Ref<ImageBitmap> create(std::pair<OwnPtr<ImageBuffer>, bool>&&);

    ~ImageBitmap();

    unsigned width() const;
    unsigned height() const;
    void close();

    bool isDetached() const { return m_detached; }

    ImageBuffer* buffer() { return m_bitmapData.get(); }

    bool originClean() const { return m_originClean; }

    PassOwnPtr<ImageBuffer> transferOwnershipAndClose();

    static Vector<std::pair<OwnPtr<ImageBuffer>, bool>> detachBitmaps(Vector<RefPtr<ImageBitmap>>&&);

private:
    friend class ImageBitmapImageObserver;
    friend class PendingImageBitmap;

    static Ref<ImageBitmap> create(PassOwnPtr<ImageBuffer>);
    ImageBitmap(PassOwnPtr<ImageBuffer>);

    template<typename T> static void createPromiseInternal(ScriptExecutionContext&, PassRefPtr<T>, const ImageBitmapOptions&, int sx, int sy, int sw, int sh, PassRefPtr<DeferredPromise>);

    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLImageElement>, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLCanvasElement>, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<HTMLVideoElement>, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<ImageBitmap>, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<Blob>, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);
    static void createPromise(ScriptExecutionContext&, PassRefPtr<ImageData>, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);
    static void createFromBuffer(PassRefPtr<ArrayBuffer>, String mimeType, long long expectedContentLength, const KURL&, const ImageBitmapOptions&, Optional<IntRect>, PassRefPtr<DeferredPromise>);

    OwnPtr<ImageBuffer> m_bitmapData;
    bool m_detached { false };
    bool m_originClean { true };

    static RenderingMode bufferRenderingMode;
};

}
