/*
 * Copyright (C) 2008, 2009 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "ImageDataSettings.h"
#include "IntSize.h"
#include "PredefinedColorSpace.h"
#include <runtime/Uint8ClampedArray.h>
#include <wtf/Forward.h>
#include <wtf/Optional.h>
#include <wtf/RefCounted.h>
#include <wtf/RefPtr.h>

namespace WebCore {

typedef int ExceptionCode;

class ImageData : public RefCounted<ImageData> {
public:
    static PassRefPtr<ImageData> create(unsigned sw, unsigned sh, Optional<ImageDataSettings>, ExceptionCode&);
    static PassRefPtr<ImageData> create(const IntSize&);
    static PassRefPtr<ImageData> create(const IntSize&, PassRefPtr<Uint8ClampedArray>, PredefinedColorSpace);
    static PassRefPtr<ImageData> create(PassRefPtr<Uint8ClampedArray>, unsigned sw, Optional<unsigned> sh, Optional<ImageDataSettings>, ExceptionCode&);

    static PredefinedColorSpace computeColorSpace(Optional<ImageDataSettings>, PredefinedColorSpace defaultColorSpace = PredefinedColorSpace::Srgb);

    const IntSize& size() const { return m_size; }
    int width() const { return m_size.width(); }
    int height() const { return m_size.height(); }
    Uint8ClampedArray* data() const { return m_data.get(); }
    PredefinedColorSpace colorSpace() const { return m_colorSpace; }

private:
    ImageData(const IntSize&, PredefinedColorSpace);
    ImageData(const IntSize&, PassRefPtr<Uint8ClampedArray>, PredefinedColorSpace);

    IntSize m_size;
    RefPtr<Uint8ClampedArray> m_data;
    PredefinedColorSpace m_colorSpace;
};

} // namespace WebCore
