/*
 * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#if HAVE(PDFIUM)

#include "HTMLDocument.h"

namespace WebCore {

class HTMLObjectElement;

class PDFDocument FINAL : public HTMLDocument {
public:
    static Ref<PDFDocument> create(Frame& frame, const KURL& url);

    virtual void finishedParsing() OVERRIDE;

private:
    PDFDocument(Frame&, const KURL&);

    virtual PassRefPtr<DocumentParser> createParser() OVERRIDE;

    void createDocumentStructure();

    WeakPtr<HTMLObjectElement> m_objectElement;
};

inline PDFDocument* toPDFDocument(Document* document)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!document || document->isPDFDocument());
    return static_cast<PDFDocument*>(document);
}

inline const PDFDocument* toPDFDocument(const Document* document)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!document || document->isPDFDocument());
    return static_cast<const PDFDocument*>(document);
}

// This will catch anyone doing an unnecessary cast.
void toPDFDocument(const PDFDocument*);

}

#endif // HAVE(PDFIUM)
