/*
 * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "FloatPoint.h"
#include "HTMLDivElement.h"
#include "HTMLNames.h"
#include "RenderBlockFlow.h"
#include "RenderStyleConstants.h"
#include <wtf/Forward.h>

namespace WebCore {

class HTMLElement;
class HTMLInputElement;
class Event;
class FloatPoint;

class SliderThumbElement FINAL : public HTMLDivElement {
public:
    static Ref<SliderThumbElement> create(Document*);

    void setPositionFromValue();

    void dragFrom(const LayoutPoint&);
    virtual void defaultEventHandler(Event*) OVERRIDE;
    virtual bool willRespondToMouseMoveEvents() OVERRIDE;
    virtual bool willRespondToMouseClickEvents() OVERRIDE;
    virtual void willDetachRenderers() OVERRIDE;
    virtual const AtomicString& shadowPseudoId() const OVERRIDE;
    HTMLInputElement* hostInput() const;
    void setPositionFromPoint(const LayoutPoint&);

    void hostDisabledStateChanged();

private:
    SliderThumbElement(Document*);
    virtual bool isSliderThumbElement() const final { return true; }

    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&) OVERRIDE;

    virtual Ref<Element> cloneElementWithoutAttributesAndChildren(Document&) OVERRIDE;
    virtual bool isDisabledFormControl() const OVERRIDE;
    virtual bool matchesReadWritePseudoClass() const OVERRIDE;

    virtual Optional<ElementStyle> resolveCustomStyle(const Style::ResolutionContext&, const RenderStyle*) OVERRIDE;

    void startDragging();
    void stopDragging();

    AtomicString m_shadowPseudoId;
    bool m_inDragMode { false };
};

inline Ref<SliderThumbElement> SliderThumbElement::create(Document* document)
{
    return adoptRef(*new SliderThumbElement(document));
}

inline Ref<Element> SliderThumbElement::cloneElementWithoutAttributesAndChildren(Document& targetDocument)
{
    return create(&targetDocument);
}

NODE_TYPE_CASTS(SliderThumbElement)

// --------------------------------

class RenderSliderThumb FINAL : public RenderBlockFlow {
public:
    RenderSliderThumb(Element&, PassOwnPtr<RenderStyle>);
    void updateAppearance(const RenderStyle& parentStyle);

private:
    virtual bool isSliderThumb() const OVERRIDE;
};

// --------------------------------

class SliderContainerElement FINAL : public HTMLDivElement {
public:
    static Ref<SliderContainerElement> create(Document*);

private:
    SliderContainerElement(Document*);
    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&) OVERRIDE;
    virtual Optional<ElementStyle> resolveCustomStyle(const Style::ResolutionContext&, const RenderStyle*) OVERRIDE;
    virtual const AtomicString& shadowPseudoId() const OVERRIDE;
    virtual bool isSliderContainerElement() const OVERRIDE { return true; }

    AtomicString m_shadowPseudoId;
};

template <typename ArgType>
struct ElementTypeCastTraits<const SliderContainerElement, ArgType> {
    static bool is(const Element& element) { return element.isUploadButton(); }
    static bool is(const Node& node) { return node.isElementNode() && toElement(node).isUploadButton(); }
};

} // namespace WebCore

#if 0
SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::SliderThumbElement)
    static bool isType(const WebCore::Element& element) { return element.isSliderThumbElement(); }
    static bool isType(const WebCore::Node& node) { return is<WebCore::Element>(node) && isType(downcast<WebCore::Element>(node)); }
SPECIALIZE_TYPE_TRAITS_END()

SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::SliderContainerElement)
    static bool isType(const WebCore::Element& element) { return element.isSliderContainerElement(); }
    static bool isType(const WebCore::Node& node) { return is<WebCore::Element>(node) && isType(downcast<WebCore::Element>(node)); }
SPECIALIZE_TYPE_TRAITS_END()
#endif

