/*
 * Copyright (C) 2011 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef TrackEvent_h
#define TrackEvent_h

#if ENABLE(VIDEO_TRACK)

#include "Event.h"
#include "TrackBase.h"

namespace WebCore {

class TrackEvent FINAL : public Event {
public:
	struct Init : public EventInit {
	    RefPtr<TrackBase> track;
	};

    virtual ~TrackEvent();

    static PassRefPtr<TrackEvent> create(const AtomicString& type, bool canBubble, bool cancelable, PassRefPtr<TrackBase> track)
    {
        return adoptRef(new TrackEvent(type, canBubble, cancelable, track));
    }

    static PassRefPtr<TrackEvent> createForBindings(const AtomicString& type, const Init& initializer)
    {
        return adoptRef(new TrackEvent(type, initializer));
    }

    virtual EventInterface eventInterface() const FINAL;

    TrackBase* track() const { return m_track.get(); }

private:
    TrackEvent(const AtomicString& type, bool canBubble, bool cancelable, PassRefPtr<TrackBase> track);
    TrackEvent(const AtomicString& type, const Init& initializer);

    RefPtr<TrackBase> m_track;
};

using TrackEventInit = TrackEvent::Init;

} // namespace WebCore

#endif
#endif
