/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

namespace WebCore {

enum FrameState : uint8_t {
	FrameStateProvisional,
	// This state indicates we are ready to commit to a page,
	// which means the view will transition to use the new data source.
	FrameStateCommittedPage,
	FrameStateComplete
};

enum PolicyAction : uint8_t {
	PolicyUse,
	PolicyDownload,
	PolicyIgnore
};

// NOTE: Keep in sync with WebKit/mac/WebView/WebFramePrivate.h and WebKit/win/Interfaces/IWebFramePrivate.idl
enum FrameLoadType : uint8_t {
	FrameLoadTypeStandard,
	FrameLoadTypeBack,
	FrameLoadTypeForward,
	FrameLoadTypeIndexedBackForward, // a multi-item hop in the backforward list
	FrameLoadTypeReload,
	// Skipped value: 'FrameLoadTypeReloadAllowingStaleData', still present in mac/win public API. Ready to be reused
	FrameLoadTypeSame = FrameLoadTypeReload + 2, // user loads same URL again (but not reload button)
	FrameLoadTypeRedirectWithLockedBackForwardList, // FIXME: Merge "lockBackForwardList", "lockHistory", "quickRedirect" and "clientRedirect" into a single concept of redirect.
	FrameLoadTypeReplace,
	FrameLoadTypeReloadFromOrigin,
};

enum class NewFrameOpenerPolicy : uint8_t {
    Suppress,
    Allow
};

enum NavigationType : uint8_t {
	NavigationTypeLinkClicked,
	NavigationTypeFormSubmitted,
	NavigationTypeBackForward,
	NavigationTypeReload,
	NavigationTypeFormResubmitted,
	NavigationTypeOther
};

enum ClearProvisionalItemPolicy : uint8_t {
	ShouldClearProvisionalItem,
	ShouldNotClearProvisionalItem
};

enum class StopLoadingPolicy : uint8_t {
	PreventDuringUnloadEvents,
	AlwaysStopLoading
};

enum ObjectContentType : uint8_t {
	ObjectContentNone,
	ObjectContentImage,
	ObjectContentFrame,
	ObjectContentNetscapePlugin,
	ObjectContentOtherPlugin
};

enum UnloadEventPolicy : uint8_t {
	UnloadEventPolicyNone,
	UnloadEventPolicyUnloadOnly,
	UnloadEventPolicyUnloadAndPageHide
};

// Passed to FrameLoader::urlSelected() and ScriptController::executeIfJavaScriptURL()
// to control whether, in the case of a JavaScript URL, executeIfJavaScriptURL() should
// replace the document.  It is a FIXME to eliminate this extra parameter from
// executeIfJavaScriptURL(), in which case this enum can go away.
enum ShouldReplaceDocumentIfJavaScriptURL : uint8_t {
	ReplaceDocumentIfJavaScriptURL,
	DoNotReplaceDocumentIfJavaScriptURL
};

enum ReasonForCallingAllowPlugins : uint8_t {
	AboutToInstantiatePlugin,
	NotAboutToInstantiatePlugin
};

enum WebGLLoadPolicy : uint8_t {
	WebGLBlockCreation,
	WebGLAllowCreation,
	WebGLPendingCreation
};

enum class LockHistory : uint8_t {
	Yes,
	No
};

enum class LockBackForwardList : uint8_t {
	Yes,
	No
};

enum class AllowNavigationToInvalidURL : uint8_t {
	Yes,
	No
};

enum class LoadCompletionType : uint8_t {
	Finish,
	Cancel
};

}
