
#include "config.h"

#include "ExportedMalloc.h"
#include "ExportedSymbols.h"
#include "ApplicationCacheStorage.h"
#include "Chrome.h"
#include "CSSPropertyAnimation.h"
#include "CSSSelector.h"
#include "CSSValuePool.h"
#include "CommonVM.h"
#include "Document.h"
#include "DragController.h"
#include "ElementInlines.h"
#include "File.h"
#include "FloatRect.h"
#include "FloatQuad.h"
#include "LayoutRect.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "FrameSelection.h"
#include "FrameView.h"
#include "GCController.h"
#include "GlyphPage.h"
#include "GradientImage.h"
#include "JSDOMWindow.h"
#include "JSElement.h"
#include "JSEventListener.h"
#include "JSLock.h"
#include "HTMLBodyElement.h"
#include "HTMLFrameOwnerElement.h"
#include "HTMLInputElement.h"
#include "HTMLNameCache.h"
#include "HTMLNames.h"
#include "HitTestResult.h"
#include "HTTPParsers.h"
#include "IconController.h"
#include "InputTypeNames.h"
#include "InlineStyleSheetOwner.h"
#include "Node.h"
#include "Page.h"
#include "GroupSettings.h"
#include "PlatformString.h"
#include "QualifiedName.h"
#include "RenderLayer.h"
#include "RenderView.h"
#include "RenderWidget.h"
#include "PageGroup.h"
#include "ResourceLoader.h"
#include "ResourceLoadScheduler.h"
#include "TextEncoding.h"
#include "ScriptState.h"
#include "SecurityOrigin.h"
#include "SimpleJPEGImageDecoder.h"
#include "SimpleRange.h"
#include "StorageAreaSync.h"
#include "SVGPathElement.h"
#include "TextEncoding.h"
#include "VisibleSelection.h"
#include "BitmapImage.h"
#include "DatabaseTracker.h"
#if ENABLE(RMAI)
#include "MOMPlayer.h"
#include "JSMOMPlayer.h"
#include "MOM.h"
#include "PluginViewMOM.h"
#include "JSDOMGlobalObject.h"
#include "Document.h"
#include "GalioWaferCollection.h"
#endif
#include <runtime/IntlObject.h>
#include <sqlite3.h>
#include <wtf/FastMallocMonitor.h>
#include <wtf/PlatformRunState.h>
#include <wtf/text/StringView.h>

namespace WebCore
{
extern int gCSSParserSelectorCount;
extern int gCSSFontSelectorCount;

extern int gCSSFontFaceSetCount;
extern int gCSSSegmentedFontFaceCount;

extern unsigned gElementRareDataCount;

extern unsigned gRuleSetCount;
extern unsigned gStyleRuleCount;

extern unsigned gIntersectionObserverCount;

extern int gStyleUpdateCount;

extern int gNodeCount;
extern int gHTMLSlotElementCount;
extern int gRenderObjectCount;
extern int gFloatingObjectCount;

extern int gSVGImageCount;

extern int gDocumentTimelineCount;
extern int gWebAnimationCount;

extern int gApplicationCacheGroupCount;

extern int gImageLoaderCount;
extern int gHTMLImageElementCount;
extern int gHTMLCanvasElementCount;

extern int gFrameViewCount;

extern std::atomic<int> gEventCount;

extern void clearCachedShadowBlurBuffer();

extern HashSet<FontFace*> gFontFaces;

extern bool gSQLiteUseWALJournalMode;
}

using namespace WebCore;

namespace WebCoreAPIs
{

DLL_PUBLIC const WebCore::HTMLQualifiedName& aTag()			{ return HTMLNames::aTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& appletTag()	{ return HTMLNames::appletTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& areaTag()		{ return HTMLNames::areaTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& bodyTag()		{ return HTMLNames::bodyTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& brTag() 		{ return HTMLNames::brTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& buttonTag()	{ return HTMLNames::buttonTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& canvasTag()	{ return HTMLNames::canvasTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& divTag() 		{ return HTMLNames::divTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& embedTag() 	{ return HTMLNames::embedTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& formTag()		{ return HTMLNames::formTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& frameTag() 	{ return HTMLNames::frameTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& htmlTag()		{ return HTMLNames::htmlTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& iframeTag() 	{ return HTMLNames::iframeTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& imgTag() 	    { return HTMLNames::imgTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& inputTag() 	{ return HTMLNames::inputTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& liTag() 		{ return HTMLNames::liTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& preTag() 		{ return HTMLNames::preTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& spanTag() 		{ return HTMLNames::spanTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& styleTag() 	{ return HTMLNames::styleTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& titleTag() 	{ return HTMLNames::titleTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& objectTag()	{ return HTMLNames::objectTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& pTag() 		{ return HTMLNames::pTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& paramTag() 	{ return HTMLNames::paramTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& scriptTag()	{ return HTMLNames::scriptTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& textareaTag()	{ return HTMLNames::textareaTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& ulTag()		{ return HTMLNames::ulTag; }
DLL_PUBLIC const WebCore::HTMLQualifiedName& videoTag()		{ return HTMLNames::videoTag; }

#if ENABLE(BML)
DLL_PUBLIC const WebCore::HTMLQualifiedName& beitemTag()	{ return HTMLNames::beitemTag; }
#endif

DLL_PUBLIC const WebCore::QualifiedName& dataAttr() 				{ return HTMLNames::dataAttr; }
DLL_PUBLIC const WebCore::QualifiedName& hrefAttr() 				{ return HTMLNames::hrefAttr; }
DLL_PUBLIC const WebCore::QualifiedName& idAttr() 					{ return HTMLNames::idAttr; }
DLL_PUBLIC const WebCore::QualifiedName& inputmodeAttr() 			{ return HTMLNames::inputmodeAttr; }
DLL_PUBLIC const WebCore::QualifiedName& nameAttr() 				{ return HTMLNames::nameAttr; }
DLL_PUBLIC const WebCore::QualifiedName& readonlyAttr() 			{ return HTMLNames::readonlyAttr; }
DLL_PUBLIC const WebCore::QualifiedName& styleAttr()  				{ return HTMLNames::styleAttr; }
DLL_PUBLIC const WebCore::QualifiedName& typeAttr()					{ return HTMLNames::typeAttr; }
DLL_PUBLIC const WebCore::QualifiedName& valueAttr() 				{ return HTMLNames::valueAttr; }

DLL_PUBLIC void jsGarbageCollectNow(bool noStackObjects, bool clearCache, bool shouldDeleteAllCodeBlocks, bool force)
{
	gcController().garbageCollectNow(noStackObjects, clearCache, shouldDeleteAllCodeBlocks, force);

	if (noStackObjects && WTF::platformRunState() != WTF::Platform_State_Running) {
		if (auto* commonVM = WebCore::commonVMOrNull())
			commonVM->purgeWatchpointSetForImpurePropertyIfNecessary();
	}
}

DLL_PUBLIC WebCore::ScriptValue executeScript(WebCore::Frame* frame, const String& script, bool forceUserGesture)
{
	return frame->script()->executeScript(script, forceUserGesture);
}

DLL_PUBLIC WebCore::RenderWidget* find(const WebCore::Widget* widget)
{
	return RenderWidget::find(widget);
}

//copied from RenderLayer.cpp : r57482
DLL_PUBLIC void scrollRectToVisible(RenderLayer* layer, const IntRect &rect)
{
	layer->scrollRectToVisible(SelectionRevealMode::Reveal, rect, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignToEdgeIfNeeded);
}

DLL_PUBLIC Color colorFromPremultipliedARGB(unsigned pixelColor)
{
	return WebCore::colorFromPremultipliedARGB(pixelColor);
}

DLL_PUBLIC unsigned premultipliedARGBFromColor(const Color& color)
{
	return WebCore::premultipliedARGBFromColor(color);
}

DLL_PUBLIC WebCore::FrameView* frameView(const WebCore::Node* node)
{
	return node->document().view();
}

DLL_PUBLIC void setShouldTrackVisitedLinks(bool shouldTrack)
{
	PageGroup::setShouldTrackVisitedLinks(shouldTrack);
}

DLL_PUBLIC void addSubResourceLoad(WebCore::ResourceLoader* resourceLoader)
{
	resourceLoadScheduler()->scheduleLoad(resourceLoader);
}

DLL_PUBLIC WebCore::KURL iconURL(WebCore::Frame* frame)
{
	return frame->loader()->icon()->url();
}

DLL_PUBLIC bool isLiveLink(const WebCore::HitTestResult& result)
{
    Element* urlElement = result.URLElement();
    if (!urlElement || !isDraggableLink(*urlElement))
    	return false;

    return true;
}

DLL_PUBLIC WebCore::KURL absoluteLinkURL(const WebCore::HitTestResult& hit)
{
	return hit.absoluteLinkURL();
}

DLL_PUBLIC bool protocolIsJavaScript(const WebCore::KURL& url)
{
	return WebCore::protocolIsJavaScript(url.string());
}

DLL_PUBLIC String decodeURLEscapeSequences(const String& string)
{
	return WebCore::decodeURLEscapeSequences(string);
}

DLL_PUBLIC String decodeURLEscapeSequences(const String& string, const String& encoding)
{
	return WebCore::decodeURLEscapeSequences(string, TextEncoding(encoding));
}

DLL_PUBLIC WebCore::Color makeColor(int r, int g, int b, int a)
{
	return WebCore::Color(r, g, b, a);
}

DLL_PUBLIC Element* getElementById(WebCore::Document* doc, const String& id)
{
	return doc->getElementById(id);
}

DLL_PUBLIC HTMLElement* getDocumentBodyOrFrameset(const WebCore::Document& doc)
{
	return doc.bodyOrFrameset();
}

DLL_PUBLIC void dispatchSimulatedClick(WebCore::Element* element, WebCore::Event* underlyingEvent, bool sendMouseEvents, bool showPressedLook)
{
	element->dispatchSimulatedClick(underlyingEvent, sendMouseEvents ? SendMouseUpDownEvents : SendNoEvents, showPressedLook ? ShowPressedLook : DoNotShowPressedLook);
}

DLL_PUBLIC String extractMIMETypeFromMediaType(const String& mediaType)
{
	return WebCore::extractMIMETypeFromMediaType(mediaType);
}

DLL_PUBLIC String extractCharsetFromMediaType(const String& mediaType)
{
	return WebCore::extractCharsetFromMediaType(mediaType);
}

DLL_PUBLIC WebCore::IntRect enclosingIntRect(const WebCore::FloatRect& rect)
{
	return WebCore::enclosingIntRect(rect);
}

DLL_PUBLIC WebCore::IntRect enclosingIntRect(const WebCore::LayoutRect& rect)
{
	return WebCore::enclosingIntRect(rect);
}

DLL_PUBLIC WebCore::IntRect enclosingBoundingBox(const WebCore::FloatQuad& quad)
{
	return quad.enclosingBoundingBox();
}

DLL_PUBLIC PassRefPtr<WebCore::SharedBuffer> utf8Buffer(const String& string)
{
	return WebCore::utf8Buffer(string);
}

DLL_PUBLIC bool hasEventListeners(WebCore::EventTarget* target, const AtomicString& eventType)
{
	return target->hasEventListeners(eventType);
}

DLL_PUBLIC PassRefPtr<WebCore::File> createFile(JSC::ExecState* exec, const String& filename)
{
	return WebCore::File::create(WebCore::scriptExecutionContextFromExecState(exec), filename);
}

DLL_PUBLIC PassRefPtr<WebCore::Document> createDocument(WebCore::Frame* frame, const WebCore::KURL& url)
{
	return WebCore::Document::create(frame, url);
}

DLL_PUBLIC PassRefPtr<WebCore::SecurityOrigin> createSecurityOrigin(const WebCore::KURL& kurl)
{
	return WebCore::SecurityOrigin::create(kurl);
}

DLL_PUBLIC void addJSEventListener(WebCore::JSElement* thisJSElement, JSC::ExecState* exec, const String& eventName, JSC::JSValue value)
{
	if (value.isObject())
		thisJSElement->wrapped().addEventListener(eventName, JSEventListener::create(asObject(value), thisJSElement, false, currentWorld(exec)), false);
}

DLL_PUBLIC void removeJSEventListener(WebCore::JSElement* thisJSElement, JSC::ExecState* exec, const String& eventName, JSC::JSValue value)
{
	if (value.isObject())
		thisJSElement->wrapped().removeEventListener(eventName, JSEventListener::create(asObject(value), thisJSElement, false, currentWorld(exec)).get(), false);
}

DLL_PUBLIC void clearJSEventListeners(WebCore::JSElement* thisJSElement, JSC::ExecState* exec, const String& eventName)
{
	Element& element = static_cast<Element&>(thisJSElement->wrapped());

	EventListenerVector listeners = element.getEventListeners(eventName);
	for(int i = 0; i < listeners.size(); ++i) {
		auto& eventListener = listeners[i];
		element.removeEventListener(eventName, &eventListener->listener(), eventListener->useCapture());
	}
}

DLL_PUBLIC void dispatchJSEvent(WebCore::Element* target, const String& eventName, const JSC::MarkedArgumentBuffer& args)
{
	if(!target || !target->document().frame())
		return;

	WebCore::JSDOMWindow* globalObject = target->document().frame()->script()->globalObject(WebCore::mainThreadNormalWorld());
	if(!globalObject)
		return;

	auto& vm = globalObject->vm();
	auto* exec = globalObject->globalExec();

	for (auto& listener : target->getEventListeners(eventName)){
		if(auto* jsEventListener = JSEventListener::cast(&listener->listener())){

			JSC::JSValue jsFunction = jsEventListener->ensureJSFunction(target->document());

			auto callData = JSC::getCallData(vm, jsFunction);

			if (callData.type != JSC::CallData::Type::None)
				JSC::call(exec, jsFunction, callData, JSC::jsUndefined(), args);
		}
	}

	vm.clearException();
}

DLL_PUBLIC Vector<uint8_t> encode(const char* encodingName, StringView characters, WebCore::UnencodableHandling handling)
{
	return TextEncoding(encodingName).encode(characters, handling);
}

DLL_PUBLIC void clearAcceleratedSurfaceCache()
{
	WebCore::clearCachedShadowBlurBuffer();
	WebCore::clearCachedGeneratorGeneratedImageBuffers();

	WebCore::BitmapImage::destroyAcceleratedSurfaces();
}

DLL_PUBLIC String externalRepresentation(WebCore::Frame* frame, WebCore::RenderAsTextBehavior behavior)
{
	return WebCore::externalRepresentation(frame, behavior);
}

DLL_PUBLIC void setIndexedDBDatabasePath(WebCore::Page* page, const String& path)
{
	page->group().groupSettings()->setIndexedDBDatabasePath(path);
}

DLL_PUBLIC void webdatabaseSetQuota(const String& path, const char* domain, unsigned long long quota)
{
#if ENABLE(SQL_DATABASE)
	RefPtr<SecurityOrigin> origin = SecurityOrigin::createFromString(String::fromLatin1(domain));
	DatabaseTracker& tracker = DatabaseTracker::tracker();
	tracker.setDatabaseDirectoryPath(path);
	tracker.setQuota(origin.get(), quota);
#endif
}

DLL_PUBLIC void scheduleRedirect(WebCore::Frame* frame, double delay, const String& url)
{
	if (frame->document())
		frame->navigationScheduler()->scheduleRedirect(*frame->document(), delay, KURL(ParsedURLString, url));
}

DLL_PUBLIC String outerHTML(WebCore::Frame* frame)
{
	if (frame && frame->document()) {
		if (auto* body = frame->document()->documentElement())
			return body->outerHTML();
	}
	return String();
}

DLL_PUBLIC void setFrameSelection(WebCore::Frame* frame, const WebCore::SimpleRange& range)
{
    frame->selection().setSelection(VisibleSelection(range), FrameSelection::defaultSetSelectionOptions(UserTriggered));
}

DLL_PUBLIC void setSQLiteUseWALJournalMode(bool useWAL)
{
	gSQLiteUseWALJournalMode = useWAL;
}

#if ENABLE(RMAI)
DLL_PUBLIC PassRefPtr<WebCore::MOMPlayer> createMOMPlayer(WebCore::ScriptExecutionContext* context)
{
	return WebCore::MOMPlayer::create(context);
}

DLL_PUBLIC JSC::JSValue getJSValueFromMOMPlayer(JSC::ExecState* exec, WebCore::Element* element, WebCore::MOMPlayer* player)
{
    WebCore::JSDOMGlobalObject* globalObject = toJSDOMGlobalObject(&element->document(), exec);
	return toJS(exec, globalObject, player);
}

DLL_PUBLIC void initMOM()
{
	MOM::init();
}

DLL_PUBLIC void loadApplicationTree(WebCore::Frame* frame)
{
    GalioWaferCollection::instance()->createApplicationTree(frame);
}

DLL_PUBLIC PassRefPtr<WebCore::Widget> createPluginViewMOM(WebCore::Frame* parentFrame, const WebCore::IntSize& pluginSize, WebCore::Element* element, const WebCore::KURL& url, const WTF::Vector<WTF::String>& paramNames, const WTF::Vector<WTF::String>& paramValues, const WTF::String& mimeType, bool loadManually)
{
    return PluginViewMOM::create(parentFrame, pluginSize, element, url, paramNames, paramValues, mimeType, loadManually);
}

DLL_PUBLIC void setPluginViewMOMPlayer(PluginViewMOM* pluginViewMOM, JSC::JSValue value)
{
    pluginViewMOM->setPlayer(toMOMPlayer(value));
}
    
DLL_PUBLIC void setPluginViewActive(PluginViewMOM* pluginViewMOM, JSC::JSValue value)
{
    pluginViewMOM->setActive(value.isBoolean() && value.asBoolean());
}

DLL_PUBLIC bool handleKeyEvent(const PlatformKeyboardEvent& kevt, Frame* frame)
{
    return GalioWaferCollection::instance()->handleKeyEvent(kevt, frame);
}
#endif // ENABLE(RMAI)

DLL_PUBLIC void closeAllDatabases()
{
	WebCore::cacheStorageUntrusted().closeDatabase();
	WebCore::cacheStorageTrusted().closeDatabase();

	WebCore::StorageAreaSync::closeAllDatabases();
}

DLL_PUBLIC void handleMemoryPressure()
{
	WebCore::drainCSSValuePool();
	WebCore::RenderStyle::defaultStyle().font().update(nullptr);

	WebCore::Page::jettisonSelectorQueryCacheInAllDocuments();
	WebCore::Page::jettisonStyleResolversInAllDocuments();

	WebCore::InlineStyleSheetOwner::clearCache();
	WebCore::HTMLNameCache::clear();
    WebCore::SVGPathElement::clearCache();

	{
		auto* vm = WebCore::JSDOMWindow::commonJSGlobalData();
		JSC::JSLockHolder lock(vm);
		vm->stringCache.clear();
		vm->heap.deleteCodeBlocks(false);
		vm->clearSourceProviderCaches();
	}
}

DLL_PUBLIC void clearDefaultStyleFonts()
{
	WebCore::RenderStyle::defaultStyle().font().update(nullptr);
}

DLL_PUBLIC void invalidateDOMCookieCacheInAllDocuments()
{
	WebCore::Page::invalidateDOMCookieCacheInAllDocuments();
}

DLL_PUBLIC LeakCounters getLeakCounters()
{
	return LeakCounters {
		WebCore::gCSSParserSelectorCount,
		WebCore::gCSSFontSelectorCount,
		WebCore::gCSSFontFaceSetCount,
		WebCore::gCSSSegmentedFontFaceCount,
		WebCore::CSSSelector::leakCount,
		WebCore::gRuleSetCount,
		WebCore::gStyleRuleCount,
		WebCore::gStyleUpdateCount,
		WebCore::gFontFaces,
		WebCore::gRenderObjectCount,
		WebCore::gFloatingObjectCount,
		WebCore::gNodeCount,
		WebCore::gHTMLSlotElementCount,
		WebCore::gSVGImageCount,
		WebCore::gDocumentTimelineCount,
		WebCore::gWebAnimationCount,
		WebCore::gApplicationCacheGroupCount,
		WebCore::gImageLoaderCount,
		WebCore::gHTMLImageElementCount,
		WebCore::gHTMLCanvasElementCount,
		WebCore::gFrameViewCount,
		WebCore::gElementRareDataCount,
		WebCore::gEventCount,
		WebCore::gIntersectionObserverCount,
		WebCore::GlyphPage::count(),
	};
}

DLL_PUBLIC void initMainThreadStaticObjects()
{
	WebCore::InputTypeNames::button();
	WebCore::InputTypeNames::checkbox();
	WebCore::InputTypeNames::color();
	WebCore::InputTypeNames::date();
	WebCore::InputTypeNames::datetimelocal();
	WebCore::InputTypeNames::email();
	WebCore::InputTypeNames::file();
	WebCore::InputTypeNames::hidden();
	WebCore::InputTypeNames::image();
	WebCore::InputTypeNames::month();
	WebCore::InputTypeNames::number();
	WebCore::InputTypeNames::password();
	WebCore::InputTypeNames::radio();
	WebCore::InputTypeNames::range();
	WebCore::InputTypeNames::reset();
	WebCore::InputTypeNames::search();
	WebCore::InputTypeNames::submit();
	WebCore::InputTypeNames::telephone();
	WebCore::InputTypeNames::text();
	WebCore::InputTypeNames::time();
	WebCore::InputTypeNames::url();
	WebCore::InputTypeNames::week();

	WebCore::CSSPropertyAnimation::ensurePropertyMap();

#if ENABLE(INTL)
	JSC::intlAvailableLocales();
	JSC::intlCollatorAvailableLocales();
#endif
}

static void* sqlite3FastMemMalloc(int n)
{
	void* ptr;
	tryFastMalloc(n).getValue(ptr);
	return ptr;
}

static void sqlite3FastFree(void *p)
{
	fastFree(p);
}

static void* sqlite3FastRealloc(void *oldPtr, int size)
{
	void* newPtr;
	tryFastRealloc(oldPtr, size).getValue(newPtr);
	return newPtr;
}

static int sqlite3FastMemSize(void *pPrior)
{
	return fastMallocSize(pPrior);
}

#define ROUND8(x) (((x)+7)&~7)
static int sqlite3FastMemRoundup(int n)
{
  return ROUND8(n);
}

static int sqlite3FastMemInit(void*)
{
	WTF::logMessage(ASCIILiteral("SQLite3 fastMalloc init."));
	return SQLITE_OK;
}

static void sqlite3FastMemShutdown(void*)
{
	WTF::logMessage(ASCIILiteral("SQLite3 fastMalloc shut down."));
}

DLL_PUBLIC void initSQLiteFastMalloc()
{
	static const sqlite3_mem_methods defaultMethods = {
		sqlite3FastMemMalloc,
		sqlite3FastFree,
		sqlite3FastRealloc,
		sqlite3FastMemSize,
		sqlite3FastMemRoundup,
		sqlite3FastMemInit,
		sqlite3FastMemShutdown,
		nullptr /* pAppData */
	};
	sqlite3_config(SQLITE_CONFIG_MALLOC, &defaultMethods);
}

} // namespace WebCoreAPIs

DLL_PUBLIC void* webkit_malloc(size_t n)
{
	// Cairo/Pixman-1 do not handle out-of-memory in some cases
	return fastMalloc(n);
}

DLL_PUBLIC void* webkit_calloc(size_t n, size_t elem_size)
{
	// Cairo/Pixman-1 do not handle out-of-memory in some cases
	return fastCalloc(n, elem_size);
}

DLL_PUBLIC char* webkit_strdup(const char* src)
{
	return fastStrDup(src);
}

DLL_PUBLIC void* webkit_realloc(void *oldPtr, size_t new_size)
{
	// Cairo/Pixman-1 do not handle out-of-memory in some cases
	return fastRealloc(oldPtr, new_size);
}

DLL_PUBLIC void webkit_free(void *p)
{
	fastFree(p);
}

extern "C"
{
DLL_PUBLIC int webkit_decodeJPEGImage(char* dataSrc, unsigned dataLength, char* decodedDst, unsigned decodedStride, unsigned decodedWidth, unsigned decodedHeight)
{
	WebCore::SimpleJPEGImageDecoder simpleJPEGImageDecoder;
	return simpleJPEGImageDecoder.decode(dataSrc, dataLength, decodedDst, decodedStride, decodedWidth, decodedHeight);
}
}
