/*
 * Copyright (C) 2012 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include <wtf/Forward.h>
#include <wtf/RefCounted.h>
#include <wtf/Vector.h>

#if PLATFORM(MAC) && !PLATFORM(IOS)
OBJC_CLASS NSPasteboard;
#endif

#if PLATFORM(IOS)
OBJC_CLASS UIPasteboard;
#endif

#if PLATFORM(GTK)
class DataObjectGtk;
#endif

namespace WebCore {

class Color;
class SharedBuffer;
class KURL;
struct PasteboardImage;
struct PasteboardWebContent;

class PlatformPasteboard {
public:
    // FIXME: probably we don't need a constructor that takes a pasteboard name for iOS.
    explicit PlatformPasteboard(const String& pasteboardName);
#if PLATFORM(IOS)
    PlatformPasteboard();
#endif
    static String uniqueName();
    
    void getTypes(Vector<String>& types);
    PassRefPtr<SharedBuffer> bufferForType(const String& pasteboardType);
    void getPathnamesForType(Vector<String>& pathnames, const String& pasteboardType);
    String stringForType(const String& pasteboardType);
    int changeCount() const;
    Color color();
    KURL url();
    
    void copy(const String& fromPasteboard);
    void addTypes(const Vector<String>& pasteboardTypes);
    void setTypes(const Vector<String>& pasteboardTypes);
    void setBufferForType(PassRefPtr<SharedBuffer>, const String& pasteboardType);
    void setPathnamesForType(const Vector<String>& pathnames, const String& pasteboardType);
    void setStringForType(const String&, const String& pasteboardType);
    void write(const PasteboardWebContent&);
    void write(const PasteboardImage&);
    void write(const String& pasteboardType, const String&);
    PassRefPtr<SharedBuffer> readBuffer(int index, const String& pasteboardType);
    String readString(int index, const String& pasteboardType);
    KURL readURL(int index, const String& pasteboardType);
    int count();

#if PLATFORM(GTK) && !PLATFORM(ESPIAL)
    WEBCORE_EXPORT void writeToClipboard(const RefPtr<DataObjectGtk>&, std::function<void()>&& primarySelectionCleared);
    WEBCORE_EXPORT RefPtr<DataObjectGtk> readFromClipboard();
#endif

private:
#if PLATFORM(MAC) && !PLATFORM(IOS)
    RetainPtr<NSPasteboard> m_pasteboard;
#endif
#if PLATFORM(IOS)
    RetainPtr<UIPasteboard> m_pasteboard;
#endif
#if PLATFORM(GTK) && !PLATFORM(ESPIAL)
    GtkClipboard* m_clipboard;
#endif
};

}
