/*
 * Copyright (C) 2012 Igalia S.L.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301 USA
 */

#pragma once

#include "GraphicsContext3D.h"
#include "Widget.h"
#include <wtf/Noncopyable.h>
#include <wtf/PassOwnPtr.h>

namespace WebCore {

class GLContext {
    WTF_MAKE_NONCOPYABLE(GLContext); WTF_MAKE_FAST_ALLOCATED;
public:
    static PassOwnPtr<GLContext> createOffscreenContext(bool isDirectGLMode);

    virtual ~GLContext() { }

    virtual bool makeContextCurrent() = 0;
    void swapBuffers() { }
    bool canRenderToDefaultFramebuffer() { return false; }
    const IntSize& defaultFrameBufferSize() const { return m_defaultFrameBufferSize; }
    virtual void markContextChanged() = 0;
    bool isGLES2Compliant() const { return m_isGLES2Compliant; }

    unsigned bufferDataOptimizedSize() const { return m_bufferDataOptimizedSize; }
    unsigned cachedBufferNumber() const { return m_cachedBufferNumber; }

    virtual void didCallUseProgram() = 0;

#if USE(3D_GRAPHICS)
    virtual PlatformGraphicsContext3D platformContext() = 0;
#endif

    virtual bool supportPackedDepthStencilBuffer() const = 0;

protected:
    GLContext()
		: m_isGLES2Compliant(true)
		, m_bufferDataOptimizedSize(0)
		, m_cachedBufferNumber(0)
	{
	}

    bool m_isGLES2Compliant;
    IntSize m_defaultFrameBufferSize;
    unsigned m_bufferDataOptimizedSize;
    unsigned m_cachedBufferNumber;
};

} // namespace WebCore
