/*
 * Copyright (C) 2009 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include "TransformOperation.h"

namespace WebCore {

struct BlendingContext;

class Matrix3DTransformOperation FINAL : public TransformOperation {
public:
    static Ref<Matrix3DTransformOperation> create(const TransformationMatrix& matrix)
    {
        return adoptRef(*new Matrix3DTransformOperation(matrix));
    }

    const TransformationMatrix& matrix() const {return m_matrix; }

private:    
    virtual bool isIdentity() const FINAL { return m_matrix.isIdentity(); }
    virtual bool isAffectedByTransformOrigin() const FINAL { return !isIdentity(); }

    virtual bool isRepresentableIn2D() const final;

    virtual bool operator==(const TransformOperation& o) const FINAL
    {
        if (!isSameType(o))
            return false;
        const Matrix3DTransformOperation* m = static_cast<const Matrix3DTransformOperation*>(&o);
        return m_matrix == m->m_matrix;
    }

    virtual bool apply(TransformationMatrix& transform, const FloatSize&) const FINAL
    {
        transform.multiply(m_matrix);
        return false;
    }

    virtual PassRefPtr<TransformOperation> blend(const TransformOperation* from, const BlendingContext&, bool blendToIdentity = false) FINAL;
    
    Matrix3DTransformOperation(const TransformationMatrix& mat)
    : TransformOperation(MATRIX_3D)
    , m_matrix(mat)
    {
    }

    TransformationMatrix m_matrix;
};

} // namespace WebCore
