/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2008-2009 Torch Mobile, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include <wtf/OwnPtr.h>

namespace WebCore {

class SimpleJPEGImageReader;

// This class decodes the JPEG image format.
class SimpleJPEGImageDecoder final {
public:
	SimpleJPEGImageDecoder();
	~SimpleJPEGImageDecoder();

	bool decode(char* dataSrc, unsigned dataLength, char* decodedDst, unsigned decodedStride, unsigned decodedWidth, unsigned decodedHeight);

	bool setSize(unsigned width, unsigned height);

	bool outputScanlines();
	void jpegComplete();

private:
	ALWAYS_INLINE char* getDecodedAddr(int x, int y) const
	{
		return m_decodedDst + (y * m_decodedStride) + x;
	}

	OwnPtr<SimpleJPEGImageReader> m_reader;

	bool m_frameComplete { false };

	char* m_dataSrc { nullptr };
	unsigned m_dataLength { 0 };

	char* m_decodedDst { nullptr };
	unsigned m_decodedStride { 0 };
	unsigned m_decodedWidth { 0 };
	unsigned m_decodedHeight { 0 };
};

} // namespace WebCore
