/*
 * Copyright (C) 2000 Lars Knoll (knoll@kde.org)
 * Copyright (C) 2013 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "RenderBoxModelObject.h"

namespace WebCore {

class InlineElementBox;
class HTMLElement;
class Position;

// FIXME: Rename this to RenderLineBreak.
class RenderBR final : public RenderBoxModelObject {
public:
    RenderBR(HTMLElement&, PassOwnPtr<RenderStyle>);
    virtual ~RenderBR();

    virtual const char* renderName() const final { return m_isWBR ? "RenderWordBreak" : "RenderBR"; }

    virtual bool isWBR() const final { return m_isWBR; }

    PassOwnPtr<InlineElementBox> createInlineBox();
    InlineElementBox* inlineBoxWrapper() const { return m_inlineBoxWrapper; }
    void setInlineBoxWrapper(InlineElementBox*);
    void deleteInlineBoxWrapper();
    void replaceInlineBoxWrapper(InlineElementBox&);
    void dirtyLineBoxes(bool fullLayout);

    IntRect linesBoundingBox() const;

    virtual void absoluteRects(Vector<IntRect>&, const LayoutPoint& accumulatedOffset) const final;
    virtual void absoluteQuads(Vector<FloatQuad>&, bool* wasFixed = nullptr) const final;
    void ensureLineBoxes();

private:
    void node() const WTF_DELETED_FUNCTION;

    virtual bool canHaveChildren() const final { return false; }

    virtual VisiblePosition positionForPoint(const LayoutPoint&, const RenderFragmentContainer*) final;
    virtual int caretMinOffset() const final;
    virtual int caretMaxOffset() const final;
    virtual bool canBeSelectionLeaf() const final;
    virtual void setSelectionState(SelectionState) final;

    virtual LayoutUnit lineHeight(bool firstLine, LineDirectionMode, LinePositionMode) const final;
    virtual LayoutUnit baselinePosition(FontBaseline, bool firstLine, LineDirectionMode, LinePositionMode) const final;

    virtual LayoutUnit marginTop() const final { return 0; }
    virtual LayoutUnit marginBottom() const final { return 0; }
    virtual LayoutUnit marginLeft() const final { return 0; }
    virtual LayoutUnit marginRight() const final { return 0; }
    virtual LayoutUnit marginBefore(const RenderStyle*) const final { return 0; }
    virtual LayoutUnit marginAfter(const RenderStyle*) const final { return 0; }
    virtual LayoutUnit marginStart(const RenderStyle*) const final { return 0; }
    virtual LayoutUnit marginEnd(const RenderStyle*) const final { return 0; }
    virtual LayoutUnit offsetWidth() const final { return linesBoundingBox().width(); }
    virtual LayoutUnit offsetHeight() const final { return linesBoundingBox().height(); }
    virtual LayoutRect borderBoundingBox() const override { return LayoutRect(LayoutPoint(), linesBoundingBox().size()); }
    virtual LayoutRect frameRectForStickyPositioning() const final { ASSERT_NOT_REACHED(); return LayoutRect(); }
    virtual LayoutRect clippedOverflowRectForRepaint(RenderLayerModelObject*) const final { return LayoutRect(); }

    virtual void updateFromStyle() final;
    virtual bool requiresLayer() const final { return false; }

    InlineElementBox* m_inlineBoxWrapper;
    mutable Optional<LayoutUnit> m_cachedLineHeight;
    bool m_isWBR;
};

typedef RenderBR RenderLineBreak;

template<> inline bool isRendererOfType<const RenderLineBreak>(const RenderObject& renderer) { return renderer.isLineBreak(); }
RENDER_OBJECT_TYPE_CASTS(RenderLineBreak, isLineBreak())
RENDER_OBJECT_TYPE_CASTS(RenderBR, isLineBreak())

} // namespace WebCore
