/*
 * Copyright (C) 2005 Apple Computer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "RenderFlexibleBox.h"
#include "Timer.h"
#include <wtf/OwnPtr.h>

namespace WebCore {

class HTMLFormControlElement;
class RenderTextFragment;

// RenderButtons are just like normal flexboxes except that they will generate an anonymous block child.
// For inputs, they will also generate an anonymous RenderText and keep its style and content up
// to date as the button changes.
class RenderButton FINAL: public RenderFlexibleBox {
public:
    RenderButton(HTMLFormControlElement&, PassOwnPtr<RenderStyle>);
    virtual ~RenderButton();

    HTMLFormControlElement& formControlElement() const;

    virtual bool canBeSelectionLeaf() const OVERRIDE;

    virtual bool createsAnonymousWrapper() const OVERRIDE { return true; }

    virtual void updateFromElement() OVERRIDE;

    virtual bool canHaveGeneratedChildren() const OVERRIDE;
    virtual bool hasControlClip() const OVERRIDE { return true; }
    virtual LayoutRect controlClipRect(const LayoutPoint&) const OVERRIDE;

    virtual void updateAnonymousChildStyle(RenderStyle&) const OVERRIDE;

    void setText(const String&);
    String text() const;

    virtual LayoutUnit baselinePosition(FontBaseline, bool firstLine, LineDirectionMode, LinePositionMode = PositionOnContainingLine) const OVERRIDE;

    RenderBlock* innerRenderer() const { return m_inner.get(); }
    void setInnerRenderer(RenderBlock&);

private:
    void element() const WTF_DELETED_FUNCTION;

    virtual const char* renderName() const OVERRIDE { return "RenderButton"; }
    virtual bool isRenderButton() const OVERRIDE { return true; }

    virtual bool hasLineIfEmpty() const OVERRIDE;

    virtual bool isFlexibleBoxImpl() const OVERRIDE { return true; }

    WeakPtr<RenderTextFragment> m_buttonText;
    WeakPtr<RenderBlock> m_inner;
};

template <> inline bool isRendererOfType<const RenderButton>(const RenderObject& renderer) { return renderer.isRenderButton(); }
RENDER_OBJECT_TYPE_CASTS(RenderButton, isRenderButton())

} // namespace WebCore
