/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2003-2018 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "RenderBlockFlow.h"
#include "RenderListMarker.h"

namespace WebCore {

class HTMLOListElement;

class RenderListItem FINAL : public RenderBlockFlow {
public:
    RenderListItem(Element&, PassOwnPtr<RenderStyle>);
    virtual ~RenderListItem();

    Element& existingElement() const { return *RenderBlockFlow::element(); }

    int value() const;
    void updateValue();

    Optional<int> explicitValue() const { return m_valueWasSetExplicitly ? m_value : Nullopt; }
    void setExplicitValue(Optional<int>);

    void setNotInList(bool notInList) { m_notInList = notInList; }
    bool notInList() const { return m_notInList; }

    const String& markerText() const;
    String markerTextWithSuffix() const;

    void updateListMarkerNumbers();

    static void updateItemValuesForOrderedList(const HTMLOListElement&);
    static unsigned itemCountForOrderedList(const HTMLOListElement&);

    PassOwnPtr<RenderStyle> computeMarkerStyle() const;

    RenderListMarker* markerRenderer() const { return m_marker.get(); }
    void setMarkerRenderer(RenderListMarker& marker) { m_marker = makeWeakPtr(marker); }

    bool isInReversedOrderedList() const;

#if !ASSERT_DISABLED
    bool inLayout() const { return m_inLayout; }
#endif
private:
    void element() const WTF_DELETED_FUNCTION;

    virtual const char* renderName() const FINAL { return "RenderListItem"; }

    virtual bool isListItem() const FINAL { return true; }
    
    virtual void insertedIntoTree(IsInternalMove = IsInternalMove::No) FINAL;
    virtual void willBeRemovedFromTree(IsInternalMove = IsInternalMove::No) FINAL;

    virtual void paint(PaintInfo&, const LayoutPoint&) FINAL;

    virtual void layout() FINAL;

    virtual void addOverflowFromChildren() FINAL;
    virtual void computePreferredLogicalWidths() FINAL;

    void updateValueNow() const;
    void explicitValueChanged();

    WeakPtr<RenderListMarker> m_marker;
    mutable Optional<int> m_value;
#if !ASSERT_DISABLED
    bool m_inLayout { false };
#endif
    bool m_valueWasSetExplicitly;
    bool m_notInList;
};

inline int RenderListItem::value() const
{
    if (!m_value)
        updateValueNow();
    return m_value.value();
}

template<> inline bool isRendererOfType<const RenderListItem>(const RenderObject& renderer) { return renderer.isListItem(); }
RENDER_OBJECT_TYPE_CASTS(RenderListItem, isListItem())

bool isHTMLListElement(const Node&);

} // namespace WebCore
