/*
 * Copyright (C) 2011 Nokia Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef RenderQuote_h
#define RenderQuote_h

#include "RenderInline.h"

namespace WebCore {

class RenderQuote FINAL : public RenderInline {
public:
    RenderQuote(Document&, PassOwnPtr<RenderStyle>, QuoteType);
    virtual ~RenderQuote();

    void updateRenderer(RenderTreeBuilder&, RenderQuote* previousQuote);

private:
    virtual const char* renderName() const OVERRIDE { return "RenderQuote"; }
    virtual bool isQuote() const OVERRIDE { return true; };
    bool isOpen() const;
    virtual void styleDidChange(StyleDifference, const RenderStyle*) OVERRIDE;
    virtual void insertedIntoTree(IsInternalMove = IsInternalMove::No) OVERRIDE;
	virtual void willBeRemovedFromTree(IsInternalMove = IsInternalMove::No) OVERRIDE;

    String computeText() const;
    void updateTextRenderer(RenderTreeBuilder&);

    const QuoteType m_type;
    int m_depth;
    String m_text;

    bool m_needsTextUpdate;
};

template<> inline bool isRendererOfType<const RenderQuote>(const RenderObject& renderer) { return renderer.isQuote(); }

inline RenderQuote* toRenderQuote(RenderObject* object)
{
    ASSERT(!object || object->isQuote());
    return static_cast<RenderQuote*>(object);
}

inline const RenderQuote* toRenderQuote(const RenderObject* object)
{
    ASSERT(!object || object->isQuote());
    return static_cast<const RenderQuote*>(object);
}

inline RenderQuote& toRenderQuote(RenderObject& object)
{
    ASSERT(object.isQuote());
    return static_cast<RenderQuote&>(object);
}

inline const RenderQuote& toRenderQuote(const RenderObject& object)
{
    ASSERT(object.isQuote());
    return static_cast<const RenderQuote&>(object);
}

// This will catch anyone doing an unnecessary cast.
void toRenderQuote(const RenderQuote*);
void toRenderQuote(const RenderQuote&);

} // namespace WebCore

#endif // RenderQuote_h
