/*
 * Copyright (C) 2006, 2007, 2009 Apple Inc. All rights reserved.
 * Copyright (C) 2008 Torch Mobile Inc. All rights reserved. (http://www.torchmobile.com/)
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef RenderTextControlSingleLine_h
#define RenderTextControlSingleLine_h

#include "HTMLInputElement.h"
#include "RenderTextControl.h"

namespace WebCore {

class HTMLInputElement;

class RenderTextControlSingleLine : public RenderTextControl {
public:
    RenderTextControlSingleLine(HTMLInputElement&, PassOwnPtr<RenderStyle>);
    virtual ~RenderTextControlSingleLine();

    void capsLockStateMayHaveChanged();

protected:
    void centerRenderer(RenderBox& renderer) const;
    HTMLElement* containerElement() const;
    HTMLElement* innerBlockElement() const;
    HTMLInputElement& inputElement() const;

private:
    void textFormControlElement() const WTF_DELETED_FUNCTION;

    virtual bool hasControlClip() const OVERRIDE;
    virtual LayoutRect controlClipRect(const LayoutPoint&) const OVERRIDE;
    virtual bool isTextField() const final { return true; }

    virtual void paint(PaintInfo&, const LayoutPoint&) OVERRIDE;
    virtual void layout() OVERRIDE;

    virtual bool nodeAtPoint(const HitTestRequest&, HitTestResult&, const HitTestLocation& locationInContainer, const LayoutPoint& accumulatedOffset, HitTestAction) OVERRIDE;

    virtual void autoscroll() OVERRIDE;

    // Subclassed to forward to our inner div.
    virtual int scrollLeft() const OVERRIDE;
    virtual int scrollTop() const OVERRIDE;
    virtual int scrollWidth() const OVERRIDE;
    virtual int scrollHeight() const OVERRIDE;
    virtual void setScrollLeft(int) OVERRIDE;
    virtual void setScrollTop(int) OVERRIDE;
    virtual bool scroll(ScrollDirection, ScrollGranularity, float multiplier = 1, Node** stopNode = 0, RenderBox* startBox = 0, const IntPoint& wheelEventAbsolutePoint = IntPoint()) OVERRIDE;
    virtual bool logicalScroll(ScrollLogicalDirection, ScrollGranularity, float multiplier = 1, Node** stopNode = 0) OVERRIDE;

    int textBlockWidth() const;
    virtual float getAvgCharWidth(const AtomicString& family) OVERRIDE;
    virtual LayoutUnit preferredContentLogicalWidth(float charWidth) const OVERRIDE;
    virtual LayoutUnit computeControlLogicalHeight(LayoutUnit lineHeight, LayoutUnit nonContentHeight) const OVERRIDE;
    
    virtual void styleDidChange(StyleDifference, const RenderStyle* oldStyle) OVERRIDE;

    HTMLElement* innerSpinButtonElement() const;

    bool m_shouldDrawCapsLockIndicator;
};

inline HTMLElement* RenderTextControlSingleLine::containerElement() const
{
    return inputElement().containerElement();
}

inline HTMLElement* RenderTextControlSingleLine::innerBlockElement() const
{
    return inputElement().innerBlockElement();
}

inline RenderTextControlSingleLine* toRenderTextControlSingleLine(RenderObject* object)
{
    ASSERT(!object || object->isTextField());
    return static_cast<RenderTextControlSingleLine*>(object);
}

// This will catch anyone doing an unnecessary cast.
void toRenderTextControlSingleLine(const RenderTextControlSingleLine*);

// ----------------------------

class RenderTextControlInnerBlock FINAL : public RenderBlockFlow {
public:
    RenderTextControlInnerBlock(Element& element, PassOwnPtr<RenderStyle> style)
	: RenderBlockFlow(element, style)
	{
	}

private:
    virtual bool hasLineIfEmpty() const FINAL { return true; }
    virtual bool isTextControlInnerBlock() const FINAL { return true; }
    virtual bool canBeProgramaticallyScrolled() const FINAL { return true; }
};

inline RenderTextControlInnerBlock* toRenderTextControlInnerBlock(RenderObject* object)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!object || object->isTextControlInnerBlock());
    return static_cast<RenderTextControlInnerBlock*>(object);
}

// This will catch anyone doing an unnecessary cast.
void toRenderTextControlInnerBlock(const RenderTextControlInnerBlock*);

}

#endif
