/*
 * Copyright (C) 2000 Lars Knoll (knoll@kde.org)
 *           (C) 2000 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2003, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Graham Dennis (graham.dennis@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "Color.h"
#include "RenderStyleConstants.h"

namespace WebCore {

class BorderValue {
friend class RenderStyle;
public:
	ALWAYS_INLINE BorderValue()
        : m_style(BNONE)
        , m_isAuto(AUTO_OFF)
    {
    }

    bool nonZero() const
    {
        return width() && style() != EBorderStyle::BNONE;
    }

	ALWAYS_INLINE bool isTransparent() const
    {
        return m_color.isValid() && !alphaChannel(m_color.rgba());
    }

    bool isVisible() const
    {
        return nonZero() && !isTransparent() && style() != EBorderStyle::BHIDDEN;
    }

    ALWAYS_INLINE bool operator==(const BorderValue& o) const
    {
        return m_width == o.m_width && m_style == o.m_style && m_color == o.m_color;
    }

    ALWAYS_INLINE bool operator!=(const BorderValue& o) const
    {
        return !(*this == o);
    }

    ALWAYS_INLINE void setColor(const Color& color)
    {
        m_color = color;
    }

    ALWAYS_INLINE const Color& color() const { return m_color; }

    ALWAYS_INLINE float width() const { return m_width; }
    ALWAYS_INLINE EBorderStyle style() const { return static_cast<EBorderStyle>(m_style); }

protected:
    Color m_color;

    float m_width { 3 };

    unsigned m_style : 4; // EBorderStyle

    // This is only used by OutlineValue but moved here to keep the bits packed.
    unsigned m_isAuto : 1; // OutlineIsAuto
};

} // namespace WebCore
