/*
 * Copyright (C) 2015 Apple Inc. All rights reserved.
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "CSSCustomPropertyValue.h"
#include "CSSVariableReferenceValue.h"
#include <wtf/Forward.h>
#include <wtf/HashMap.h>
#include <wtf/RefCounted.h>
#include <wtf/PassRefPtr.h>
#include <wtf/text/AtomicStringHash.h>

namespace WebCore {

typedef HashMap<AtomicString, RefPtr<CSSCustomPropertyValue>> CustomPropertyValueMap;

class StyleCustomPropertyData : public RefCounted<StyleCustomPropertyData> {
public:
    static Ref<StyleCustomPropertyData> create() { return adoptRef(*new StyleCustomPropertyData); }
    Ref<StyleCustomPropertyData> copy() const { return adoptRef(*new StyleCustomPropertyData(*this)); }
    
    bool operator==(const StyleCustomPropertyData& o) const
	{
		if (values.size() != o.values.size())
			return false;

		for(auto iter = values.begin(); iter != values.end(); ++iter) {
			RefPtr<CSSCustomPropertyValue> other = o.values.get(iter->first);
			if (!other || !iter->second->equals(*other))
				return false;
		}
		return true;
	}

    bool operator!=(const StyleCustomPropertyData &o) const { return !(*this == o); }
    
    void setCustomPropertyValue(const AtomicString& name, PassRefPtr<CSSCustomPropertyValue> value)
    {
        values.set(name, value);
    }

    RefPtr<CSSCustomPropertyValue> getCustomPropertyValue(const AtomicString& name) const { return values.get(name); }

    bool hasCustomProperty(const AtomicString& name) const { return values.contains(name); }

    CustomPropertyValueMap values;
    
private:
    explicit StyleCustomPropertyData()
        : RefCounted<StyleCustomPropertyData>()
    { }
    StyleCustomPropertyData(const StyleCustomPropertyData& other)
        : RefCounted<StyleCustomPropertyData>()
		, values(CustomPropertyValueMap(other.values))
	{ }
};

} // namespace WebCore
