/*
 * Copyright (C) 2000 Lars Knoll (knoll@kde.org)
 *           (C) 2000 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2003, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#include "StyleImage.h"

namespace WebCore {

class CSSValue;
class CSSImageGeneratorValue;

class StyleGeneratedImage FINAL : public StyleImage {
public:
    static Ref<StyleGeneratedImage> create(CSSImageGeneratorValue& value)
    {
        return adoptRef(*new StyleGeneratedImage(value));
    }
    CSSImageGeneratorValue& imageValue() const { return m_imageGeneratorValue.get(); }

private:
    virtual bool operator==(const StyleImage& other) const final;

    virtual WrappedImagePtr data() const FINAL { return m_imageGeneratorValue.ptr(); }

    virtual Ref<CSSValue> cssValue() const FINAL;

    virtual bool isPending() const FINAL;
    virtual void load(CachedResourceLoader&, const ResourceLoaderOptions&) FINAL;
    virtual FloatSize imageSize(const RenderElement*, float multiplier) const FINAL;
    virtual bool imageHasRelativeWidth() const FINAL { return !m_fixedSize; }
    virtual bool imageHasRelativeHeight() const FINAL { return !m_fixedSize; }
    virtual void computeIntrinsicDimensions(const RenderElement*, Length& intrinsicWidth, Length& intrinsicHeight, FloatSize& intrinsicRatio) FINAL;
    virtual bool usesImageContainerSize() const FINAL { return !m_fixedSize; }
    virtual void setContainerContextForRenderer(const RenderElement&, const FloatSize& containerSize, float) FINAL { m_containerSize = containerSize; }
    virtual void addClient(RenderElement&) FINAL;
    virtual void removeClient(RenderElement&) FINAL;
    bool hasClient(RenderElement&) const final;
    virtual PassRefPtr<Image> image(RenderElement*, const FloatSize&) const FINAL;
    virtual bool knownToBeOpaque(const RenderElement*) const FINAL;

    StyleGeneratedImage(CSSImageGeneratorValue& value);
    
    Ref<CSSImageGeneratorValue> m_imageGeneratorValue;
    FloatSize m_containerSize;
    bool m_fixedSize;
};

// SPECIALIZE_TYPE_TRAITS_STYLE_IMAGE(StyleGeneratedImage, isGeneratedImage)
inline bool isStyleGeneratedImage(const StyleImage& image)
{
	return image.isGeneratedImage();
}

inline const StyleGeneratedImage& toStyleGeneratedImage(const StyleImage& image)
{
    ASSERT_WITH_SECURITY_IMPLICATION(image.isGeneratedImage());
    return static_cast<const StyleGeneratedImage&>(image);
}

inline StyleGeneratedImage& toStyleGeneratedImage(StyleImage& image)
{
    ASSERT_WITH_SECURITY_IMPLICATION(image.isGeneratedImage());
    return static_cast<StyleGeneratedImage&>(image);
}

inline StyleGeneratedImage* toStyleGeneratedImage(StyleImage* image)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!image || image->isGeneratedImage());
    return static_cast<StyleGeneratedImage*>(image);
}

}
