/*
 * Copyright (C) 2006 Oliver Hunt <ojh16@student.canterbury.ac.nz>
 * Copyright (C) 2006 Apple Computer Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "RenderInline.h"

#include "SVGGraphicsElement.h"
#include "SVGRenderSupport.h"

namespace WebCore {

class RenderSVGInline : public RenderInline {
public:
    RenderSVGInline(Element&, PassOwnPtr<RenderStyle>);

    SVGGraphicsElement& graphicsElement() const { return toSVGGraphicsElement(nodeForNonAnonymous()); }

    virtual const char* renderName() const OVERRIDE { return "RenderSVGInline"; }
    virtual bool requiresLayer() const FINAL { return false; }
    virtual bool isSVGInline() const FINAL { return true; }

    // Chapter 10.4 of the SVG Specification say that we should use the
    // object bounding box of the parent text element.
    // We search for the root text element and take its bounding box.
    // It is also necessary to take the stroke and repaint rect of
    // this element, since we need it for filters.
    virtual FloatRect objectBoundingBox() const FINAL;
    virtual FloatRect strokeBoundingBox() const FINAL;
    virtual FloatRect repaintRectInLocalCoordinates() const FINAL;

    virtual LayoutRect clippedOverflowRectForRepaint(RenderLayerModelObject* repaintContainer) const FINAL;
    virtual Optional<FloatRect> computeFloatVisibleRectInContainer(const FloatRect&, const RenderLayerModelObject* container, VisibleRectContext) const final;
    virtual void mapLocalToContainer(RenderLayerModelObject* repaintContainer, TransformState&, OptionSet<MapCoordinatesMode>, bool* wasFixed) const FINAL;
	virtual const RenderElement* pushMappingToContainer(const RenderLayerModelObject* ancestorToStopAt, RenderGeometryMap&) const FINAL;
    virtual void absoluteQuads(Vector<FloatQuad>&, bool* wasFixed) const FINAL;

private:
    virtual void updateFromStyle() final;

    virtual PassOwnPtr<InlineFlowBox> createInlineFlowBox() FINAL;

    virtual void willBeDestroyed() FINAL;
    virtual void styleDidChange(StyleDifference, const RenderStyle* oldStyle) FINAL;
};

template<> inline bool isRendererOfType<const RenderSVGInline>(const RenderObject& renderer) { return renderer.isSVGInline(); }
RENDER_OBJECT_TYPE_CASTS(RenderSVGInline, isSVGInline())

}
