/*
 * Copyright (C) Research In Motion Limited 2009-2010. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)
#include "FloatRect.h"
#include "GraphicsContext.h"
#include "ImageBuffer.h"
#include "IntSize.h"
#include "RenderSVGResourceContainer.h"
#include "SVGMaskElement.h"
#include "SVGUnitTypes.h"

#include <wtf/HashMap.h>
#include <wtf/OwnPtr.h>

namespace WebCore {

struct MaskerData {
	WTF_MAKE_FAST_ALLOCATED;
public:
    OwnPtr<ImageBuffer> maskImage;
};

class RenderSVGResourceMasker FINAL : public RenderSVGResourceContainer {
public:
    RenderSVGResourceMasker(SVGMaskElement&, PassOwnPtr<RenderStyle>);
    virtual ~RenderSVGResourceMasker();

    inline SVGMaskElement& maskElement() const;

    virtual const char* renderName() const OVERRIDE { return "RenderSVGResourceMasker"; }

    virtual void removeAllClientsFromCacheIfNeeded(bool markForInvalidation, WeakHashSet<RenderObject>* visitedRenderers) override;
    virtual void removeClientFromCache(RenderElement&, bool markForInvalidation = true) OVERRIDE;
    virtual bool applyResource(RenderElement&, const RenderStyle*, GraphicsContext*&, unsigned short resourceMode) OVERRIDE;
    bool drawContentIntoContext(GraphicsContext&, const FloatRect& objectBoundingBox);
	bool drawContentIntoContext(GraphicsContext&, const FloatRect& destinationRect, const FloatRect& sourceRect, CompositeOperator, BlendMode);
	virtual FloatRect resourceBoundingBox(const RenderObject&) OVERRIDE;

    SVGUnitTypes::SVGUnitType maskUnits() const { return maskElement().maskUnits(); }
    SVGUnitTypes::SVGUnitType maskContentUnits() const { return maskElement().maskContentUnits(); }

    virtual RenderSVGResourceType resourceType() const OVERRIDE { return MaskerResourceType; }
    static RenderSVGResourceType s_resourceType;

private:
    bool drawContentIntoMaskImage(MaskerData*, ColorSpace, RenderObject*);
    void calculateMaskContentRepaintRect();

    FloatRect m_maskContentBoundaries;
    HashMap<RenderObject*, OwnPtr<MaskerData> > m_masker;
};

inline bool isRenderSVGResourceMasker(const RenderSVGResourceContainer* container)
{
	return container && container->resourceType() == RenderSVGResourceType::MaskerResourceType;
}

inline bool isRenderSVGResourceMasker(const RenderSVGResourceContainer& container)
{
	return container.resourceType() == RenderSVGResourceType::MaskerResourceType;
}

inline SVGMaskElement& RenderSVGResourceMasker::maskElement() const
{
    return toSVGMaskElement(RenderSVGResourceContainer::element());
}

RENDER_OBJECT_TYPE_CASTS(RenderSVGResourceMasker, isRenderSVGResourceMasker())

} // namespace WebCore

#endif // ENABLE(SVG)
