/*
 * Copyright (C) 2004, 2005, 2008 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005 Rob Buis <buis@kde.org>
 * Copyright (C) 2007 Eric Seidel <eric@webkit.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)
#include "SVGGraphicsElement.h"
#include "SVGURIReference.h"
#include "LinkHash.h"
#include <wtf/Optional.h>
#include <wtf/Ref.h>

namespace WebCore {

class DOMTokenList;

class SVGAElement final : public SVGGraphicsElement, public SVGURIReference {
public:
    static PassRefPtr<SVGAElement> create(const QualifiedName&, Document*);

    String target() const final { return m_target->currentValue(); }
    SVGAnimatedString& targetAnimated() { return m_target; }

    SharedStringHash visitedLinkHash() const;

    DOMTokenList& relList();

private:
    SVGAElement(const QualifiedName&, Document*);

    using PropertyRegistry = SVGPropertyOwnerRegistry<SVGAElement, SVGGraphicsElement, SVGURIReference>;

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason = ModifiedDirectly) final;
    virtual void svgAttributeChanged(const QualifiedName&) FINAL;

    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&) FINAL;

    bool isValid() const final { return SVGTests::isValid(); }
    String title() const final;
    virtual void defaultEventHandler(Event*) FINAL;
    
    virtual bool supportsFocus() const FINAL;
    virtual bool isMouseFocusable() const FINAL;
    virtual bool isKeyboardFocusable(KeyboardEvent*) const FINAL;
    virtual bool isURLAttribute(const Attribute&) const override;
    virtual bool canStartSelection() const FINAL;
    virtual int defaultTabIndex() const FINAL;

    virtual bool childShouldCreateRenderer(const Node*) const FINAL;

    virtual bool willRespondToMouseClickEvents() OVERRIDE;

    Ref<SVGAnimatedString> m_target { SVGAnimatedString::create(this) };

    // This is computed only once and must not be affected by subsequent URL changes.
    mutable Optional<SharedStringHash> m_storedVisitedLinkHash;

    OwnPtr<DOMTokenList> m_relList;
};

inline bool isSVGAElement(const Node& node)
{
    return node.hasTagName(SVGNames::aTag);
}

inline bool isSVGAElement(const Element& element)
{
    return element.hasTagName(SVGNames::aTag);
}

NODE_TYPE_CASTS(SVGAElement)

} // namespace WebCore

#endif // ENABLE(SVG)
