/*
 * Copyright (C) 2004, 2005, 2007 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
 * Copyright (C) 2008 Apple Inc. All rights reserved.
 * Copyright (C) 2011 Torch Mobile (Beijing) Co. Ltd. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"

#if ENABLE(SVG_FONTS)
#include "SVGAltGlyphElement.h"

#include "ExceptionCode.h"
#include "RenderInline.h"
#include "RenderSVGTSpan.h"
#include "SVGAltGlyphDefElement.h"
#include "SVGGlyphElement.h"
#include "SVGNames.h"
#include "XLinkNames.h"

namespace WebCore {

inline SVGAltGlyphElement::SVGAltGlyphElement(const QualifiedName& tagName, Document* document)
    : SVGTextPositioningElement(tagName, document, adoptPtr(new PropertyRegistry(*this)))
	, SVGURIReference(this)
{
    ASSERT(hasTagName(SVGNames::altGlyphTag));
}

PassRefPtr<SVGAltGlyphElement> SVGAltGlyphElement::create(const QualifiedName& tagName, Document* document)
{
    return adoptRef(new SVGAltGlyphElement(tagName, document));
}

void SVGAltGlyphElement::setGlyphRef(const AtomicString&, ExceptionCode& ec)
{
    ec = NO_MODIFICATION_ALLOWED_ERR;
}

const AtomicString& SVGAltGlyphElement::glyphRef() const
{
    return attributeWithoutSynchronization(SVGNames::glyphRefAttr);
}

void SVGAltGlyphElement::setFormat(const AtomicString&, ExceptionCode& ec)
{
    ec = NO_MODIFICATION_ALLOWED_ERR;
}

const AtomicString& SVGAltGlyphElement::format() const
{
    return attributeWithoutSynchronization(SVGNames::formatAttr);
}

bool SVGAltGlyphElement::childShouldCreateRenderer(const Node* child) const
{
    if (child->isTextNode())
        return true;
    return false;
}

RenderPtr<RenderElement> SVGAltGlyphElement::createElementRenderer(PassOwnPtr<RenderStyle> style, const RenderTreePosition&)
{
    return createRenderer<RenderSVGTSpan>(*this, style);
}

bool SVGAltGlyphElement::hasValidGlyphElements(Vector<String>& glyphNames) const
{
    // No need to support altGlyph referencing another node inside a shadow tree.
	auto target = targetElementFromIRIString(getAttribute(SVGNames::hrefAttr, XLinkNames::hrefAttr), document());
    if (!target.element)
        return false;

    if (target.element->hasTagName(SVGNames::glyphTag)) {
        glyphNames.append(target.identifier);
        return true;
    }

    if (!target.element->hasTagName(SVGNames::altGlyphDefTag))
    	return false;

    return static_cast<SVGAltGlyphDefElement*>(target.element.get())->hasValidGlyphElements(glyphNames);
}

}

#endif // ENABLE(SVG_FONTS)
