/*
 * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005 Rob Buis <buis@kde.org>
 * Copyright (C) 2008 Apple Inc. All rights reserved.
 * Copyright (C) Research In Motion Limited 2011. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)
#include "SVGAnimationElement.h"
#include "SVGNames.h"
#include <wtf/OwnPtr.h>

namespace WebCore {

class SVGAttributeAnimator;

class SVGAnimateElementBase : public SVGAnimationElement {
public:
    bool isDiscreteAnimator() const;
    
protected:
    SVGAnimateElementBase(const QualifiedName&, Document*);

    SVGAttributeAnimator* animator() const;
    SVGAttributeAnimator* animatorIfExists() const { return m_animator.get(); }

    bool hasValidAttributeType() const override;

    void setTargetElement(SVGElement*) override;
    void setAttributeName(const QualifiedName&) override;
    void resetAnimation() override;

    bool calculateFromAndToValues(const String& fromString, const String& toString) override;
    bool calculateFromAndByValues(const String& fromString, const String& byString) override;
    bool calculateToAtEndOfDurationValue(const String& toAtEndOfDurationString) override;

    void resetAnimatedType() override;
    void calculateAnimatedValue(float progress, unsigned repeatCount, SVGSMILElement* resultElement) override;
    void applyResultsToTarget() override;
    void clearAnimatedType(SVGElement* targetElement) override;
    Optional<float> calculateDistance(const String& fromString, const String& toString) override;

    virtual String animateRangeString(const String& string) const { return string; }

private:
    bool hasInvalidCSSAttributeType() const;

    mutable RefPtr<SVGAttributeAnimator> m_animator;
    mutable Optional<bool> m_hasInvalidCSSAttributeType;
};

inline bool isSVGAnimateElementBase(const Element& element)
{
    return element.hasTagName(SVGNames::animateTag) || element.hasTagName(SVGNames::animateColorTag)
        || element.hasTagName(SVGNames::animateTransformTag) || element.hasTagName(SVGNames::setTag);
}

inline SVGAnimateElementBase* toSVGAnimateElementBase(Element* element)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!element || isSVGAnimateElementBase(element));
    return static_cast<SVGAnimateElementBase*>(element);
}

inline SVGAnimateElementBase& toSVGAnimateElementBase(Element& element)
{
    ASSERT_WITH_SECURITY_IMPLICATION(isSVGAnimateElementBase(element));
    return static_cast<SVGAnimateElementBase&>(element);
}

} // namespace WebCore

#endif // ENABLE(SVG)
