/*
 * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)

#include "XMLDocument.h"

namespace WebCore {

class SVGSVGElement;

class SVGDocument FINAL : public XMLDocument {
public:
    static Ref<SVGDocument> create(Frame*, const KURL&);

    static SVGSVGElement* rootElement(const Document&);

    bool zoomAndPanEnabled() const;
    void startPan(const FloatPoint& start);
    void updatePan(const FloatPoint& position) const;

private:
    SVGDocument(Frame*, const KURL&);

    virtual Ref<Document> cloneDocumentWithoutChildren() const OVERRIDE;

    FloatSize m_panningOffset;
};

inline Ref<SVGDocument> SVGDocument::create(Frame* frame, const KURL& url)
{
    return adoptRef(*new SVGDocument(frame, url));
}

inline SVGDocument* toSVGDocument(Document* document)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!document || document->isSVGDocument());
    return static_cast<SVGDocument*>(document);
}

inline SVGDocument& toSVGDocument(Document& document)
{
    ASSERT_WITH_SECURITY_IMPLICATION(document.isSVGDocument());
    return static_cast<SVGDocument&>(document);
}

inline const SVGDocument* toSVGDocument(const Document* document)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!document || document->isSVGDocument());
    return static_cast<const SVGDocument*>(document);
}

inline const SVGDocument& toSVGDocument(const Document& document)
{
    ASSERT_WITH_SECURITY_IMPLICATION(document.isSVGDocument());
    return static_cast<const SVGDocument&>(document);
}

// This will catch anyone doing an unnecessary cast.
void toSVGDocument(const SVGDocument*);
void toSVGDocument(const SVGDocument&);

} // namespace WebCore

#endif // ENABLE(SVG)

// vim:ts=4:noet
