/*
 * Copyright (C) 2004, 2005, 2006, 2008 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
 * Copyright (C) 2018 Adobe Systems Incorporated. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "ExceptionCode.h"
#include "Path.h"
#include "SVGGraphicsElement.h"
#include "SVGNames.h"

namespace WebCore {

struct DOMPointInit;
class SVGPoint;

class SVGGeometryElement : public SVGGraphicsElement {
    //WTF_MAKE_ISO_ALLOCATED(SVGGeometryElement);
public:
    virtual float getTotalLength() const;
    virtual PassRefPtr<SVGPoint> getPointAtLength(float distance, ExceptionCode&) const;

    bool isPointInFill(const DOMPointInit&);
    bool isPointInStroke(const DOMPointInit&);

    using PropertyRegistry = SVGPropertyOwnerRegistry<SVGGeometryElement, SVGGraphicsElement>;

    float pathLength() const { return m_pathLength->currentValue(); }
    SVGAnimatedNumber& pathLengthAnimated() { return m_pathLength; }

protected:
    SVGGeometryElement(const QualifiedName&, Document*, PassOwnPtr<SVGPropertyRegistry>);

    void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason = ModifiedDirectly) override;
    void svgAttributeChanged(const QualifiedName&) OVERRIDE;

private:
    bool isSVGGeometryElement() const override { return true; }

    Ref<SVGAnimatedNumber> m_pathLength { SVGAnimatedNumber::create(this) };
};

NODE_TYPE_CASTS(SVGGeometryElement)

inline bool isSVGGeometryElement(const Node& node)
{
    return node.isSVGElement() && toSVGElement(node).isSVGGeometryElement();
}

inline bool isSVGGeometryElement(const Node* node)
{
    return node && isSVGGeometryElement(*node);
}

inline bool isSVGGeometryElement(const SVGElement& element)
{
    return element.isSVGGeometryElement();
}

inline bool isSVGGeometryElement(const SVGElement* element)
{
    return element && element->isSVGGeometryElement();
}

template <typename ArgType>
struct ElementTypeCastTraits<const SVGGeometryElement, ArgType> {
    static bool is(ArgType& node) { return isSVGGeometryElement(node); }
};

} // namespace WebCore

#if 0
SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::SVGGeometryElement)
    static bool isType(const WebCore::SVGElement& element) { return element.isSVGGeometryElement(); }
    static bool isType(const WebCore::Node& node) { return is<WebCore::SVGElement>(node) && isType(downcast<WebCore::SVGElement>(node)); }
SPECIALIZE_TYPE_TRAITS_END()
#endif
