/*
 * Copyright (C) Research In Motion Limited 2011. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)
#include "FloatRect.h"
#include "SVGLengthValue.h"
#include "SVGUnitTypes.h"
#include <wtf/Optional.h>

namespace WebCore {

class SVGElement;
class SVGLength;

typedef int ExceptionCode;

class SVGLengthContext {
public:
    explicit SVGLengthContext(const SVGElement*);
    explicit SVGLengthContext(const SVGElement&);

    template<typename T>
    static inline FloatRect resolveRectangle(const T& context, SVGUnitTypes::SVGUnitType type, const FloatRect& viewport)
    {
        return SVGLengthContext::resolveRectangle(context, type, viewport, context.x(), context.y(), context.width(), context.height());
    }

    static FloatRect resolveRectangle(const SVGElement&, SVGUnitTypes::SVGUnitType, const FloatRect& viewport, const SVGLengthValue& x, const SVGLengthValue& y, const SVGLengthValue& width, const SVGLengthValue& height);
    static FloatPoint resolvePoint(const SVGElement&, SVGUnitTypes::SVGUnitType, const SVGLengthValue& x, const SVGLengthValue& y);
    static float resolveLength(const SVGElement&, SVGUnitTypes::SVGUnitType, const SVGLengthValue&);

    float valueForLength(const Length&, SVGLengthMode = SVGLengthMode::Other);
    float convertValueToUserUnits(float, SVGLengthMode, SVGLengthType fromUnit, ExceptionCode&) const;
    float convertValueFromUserUnits(float, SVGLengthMode, SVGLengthType toUnit, ExceptionCode&) const;

    Optional<FloatSize> viewportSize() const;

private:
    SVGLengthContext(const SVGElement*, const FloatRect& viewport);

    float convertValueFromUserUnitsToPercentage(float value, SVGLengthMode, ExceptionCode&) const;
    float convertValueFromPercentageToUserUnits(float value, SVGLengthMode, ExceptionCode&) const;

    float convertValueFromUserUnitsToEMS(float value, ExceptionCode&) const;
    float convertValueFromEMSToUserUnits(float value, ExceptionCode&) const;

    float convertValueFromUserUnitsToEXS(float value, ExceptionCode&) const;
    float convertValueFromEXSToUserUnits(float value, ExceptionCode&) const;

    Optional<FloatSize> computeViewportSize() const;

    const SVGElement* m_context;
    FloatRect m_overriddenViewport; // Ideally this would be std::optional<FloatRect>, but some tests depend on the behavior of it being a zero rect.
    mutable Optional<FloatSize> m_viewportSize;
};

inline SVGLengthContext::SVGLengthContext(const SVGElement* context)
    : m_context(context)
{
}

inline SVGLengthContext::SVGLengthContext(const SVGElement& context)
    : m_context(&context)
{
}

inline SVGLengthContext::SVGLengthContext(const SVGElement* context, const FloatRect& viewport)
    : m_context(context)
    , m_overriddenViewport(viewport)
{
}

} // namespace WebCore

#endif // ENABLE(SVG)
