/*
 * Copyright (C) 2007 Eric Seidel <eric@webkit.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)
#include "SVGElement.h"
#include "SVGNames.h"
#include "SVGURIReference.h"

namespace WebCore {
    
class SVGPathElement;

class SVGMPathElement final : public SVGElement, public SVGURIReference {
public:
    static PassRefPtr<SVGMPathElement> create(const QualifiedName&, Document*);

    virtual ~SVGMPathElement();

    SVGPathElement* pathElement();

    void targetPathChanged();

private:
    SVGMPathElement(const QualifiedName&, Document*);

	using PropertyRegistry = SVGPropertyOwnerRegistry<SVGMPathElement, SVGElement, SVGURIReference>;

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual void svgAttributeChanged(const QualifiedName&) OVERRIDE;

    void buildPendingResource() final;
    void clearResourceReferences();
    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    void removedFromAncestor(const RemovalType&, ContainerNode&) final;

    virtual bool rendererIsNeeded(const RenderStyle&) final { return false; }
    virtual void didFinishInsertingNode() OVERRIDE;
};

inline bool isSVGMPathElement(const Node& node)
{
	return node.hasTagName(SVGNames::mpathTag);
}

inline bool isSVGMPathElement(const Element& element)
{
	return element.hasTagName(SVGNames::mpathTag);
}

NODE_TYPE_CASTS(SVGMPathElement)

} // namespace WebCore

#endif // ENABLE(SVG)
