/*
 * Copyright (C) Research In Motion Limited 2010. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "FloatPoint.h"
#include "SVGPathSeg.h"
#include "SVGPathSource.h"
#include <wtf/PassOwnPtr.h>
#include <wtf/RefPtr.h>

namespace WebCore {

class SVGPathSegList;

class SVGPathSegListSource final : public SVGPathSource {
public:
    explicit SVGPathSegListSource(const SVGPathSegList&);

private:
    virtual bool hasMoreData() const FINAL;
    virtual bool moveToNextToken() FINAL { return true; }
    virtual bool parseSVGSegmentType(SVGPathSegType&) FINAL;
    virtual SVGPathSegType nextCommand(SVGPathSegType) FINAL;

    virtual bool parseMoveToSegment(FloatPoint&) FINAL;
    virtual bool parseLineToSegment(FloatPoint&) FINAL;
    virtual bool parseLineToHorizontalSegment(float&) FINAL;
    virtual bool parseLineToVerticalSegment(float&) FINAL;
    virtual bool parseCurveToCubicSegment(FloatPoint&, FloatPoint&, FloatPoint&) FINAL;
    virtual bool parseCurveToCubicSmoothSegment(FloatPoint&, FloatPoint&) FINAL;
    virtual bool parseCurveToQuadraticSegment(FloatPoint&, FloatPoint&) FINAL;
    virtual bool parseCurveToQuadraticSmoothSegment(FloatPoint&) FINAL;
    virtual bool parseArcToSegment(float&, float&, float&, bool&, bool&, FloatPoint&) FINAL;

    const SVGPathSegList& m_pathSegList;
    RefPtr<SVGPathSeg> m_segment;
    size_t m_itemCurrent;
    size_t m_itemEnd;
};

} // namespace WebCore
