/*
 * Copyright (C) 2016-2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "ExceptionCode.h"
#include "SVGPreserveAspectRatioValue.h"
#include "SVGValueProperty.h"

namespace WebCore {

class SVGPreserveAspectRatio : public SVGValueProperty<SVGPreserveAspectRatioValue> {
    using Base = SVGValueProperty<SVGPreserveAspectRatioValue>;
    using Base::Base;
    using Base::m_value;

public:
    static Ref<SVGPreserveAspectRatio> create(SVGPropertyOwner* owner, SVGPropertyAccess access, const SVGPreserveAspectRatioValue& value = SVGPreserveAspectRatioValue())
    {
        return adoptRef(*new SVGPreserveAspectRatio(owner, access, value));
    }

#if 0
    template<typename T>
	static ExceptionOr<Ref<SVGPreserveAspectRatio>> create(ExceptionOr<T>&& value)
	{
		if (value.hasException())
			return value.releaseException();
		return adoptRef(*new SVGPreserveAspectRatio(value.releaseReturnValue()));
	}
#endif

    unsigned short align() const { return m_value.align(); }

    void setAlign(float value, ExceptionCode& ec)
    {
        if (isReadOnly()) {
            ec = NO_MODIFICATION_ALLOWED_ERR;
            return;
        }

        m_value.setAlign(value, ec);
        if (ec)
            return;

        commitChange();
    }

    unsigned short meetOrSlice() const { return m_value.meetOrSlice(); }

    void setMeetOrSlice(float value, ExceptionCode& ec)
    {
        if (isReadOnly()) {
            ec = NO_MODIFICATION_ALLOWED_ERR;
            return;
        }

        m_value.setMeetOrSlice(value, ec);
        if (ec)
            return;

        commitChange();
    }

    String valueAsString() const override
    {
        return m_value.valueAsString();
    }
};

} // namespace WebCore
