/*
 * Copyright (C) Research In Motion Limited 2010. All rights reserved.
 * Copyright (C) 2018 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "ExceptionCode.h"
#include "SVGPropertyTraits.h"
#include "SVGValueProperty.h"

namespace WebCore {

class SVGRect : public SVGValueProperty<FloatRect> {
    using Base = SVGValueProperty<FloatRect>;
    using Base::Base;
    using Base::m_value;

public:
    static Ref<SVGRect> create(const FloatRect& value = FloatRect())
    {
        return adoptRef(*new SVGRect(value));
    }

    static Ref<SVGRect> create(SVGPropertyOwner* owner, SVGPropertyAccess access, const FloatRect& value = FloatRect())
    {
        return adoptRef(*new SVGRect(owner, access, value));
    }

#if 0
    template<typename T>
    static ExceptionOr<Ref<SVGRect>> create(ExceptionOr<T>&& value)
    {
        if (value.hasException())
            return value.releaseException();
        return adoptRef(*new SVGRect(value.releaseReturnValue()));
    }
#endif

    float x() { return m_value.x(); }

    void setX(float xValue, ExceptionCode& ec)
    {
        if (isReadOnly()) {
            ec = NO_MODIFICATION_ALLOWED_ERR;
            return;
        }

        m_value.setX(xValue);
        commitChange();
    }

    float y() { return m_value.y(); }

    void setY(float xValue, ExceptionCode& ec)
    {
        if (isReadOnly()) {
            ec = NO_MODIFICATION_ALLOWED_ERR;
            return;
        }

        m_value.setY(xValue);
        commitChange();
    }

    float width() { return m_value.width(); }

    void setWidth(float widthValue, ExceptionCode& ec)
    {
        if (isReadOnly()) {
            ec = NO_MODIFICATION_ALLOWED_ERR;
            return;
        }

        m_value.setWidth(widthValue);
        commitChange();
    }

    float height() { return m_value.height(); }

    void setHeight(float heightValue, ExceptionCode& ec)
    {
        if (isReadOnly()) {
            ec = NO_MODIFICATION_ALLOWED_ERR;
            return;
        }

        m_value.setHeight(heightValue);
        commitChange();
    }

    String valueAsString() const override
    {
        return SVGPropertyTraits<FloatRect>::toString(m_value);
    }
};

} // namespace WebCore
