/*
 * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"

#if ENABLE(SVG)
#include "SVGSwitchElement.h"

#include "ElementIterator.h"
#include "LegacyRenderSVGTransformableContainer.h"
#include "SVGNames.h"

namespace WebCore {

inline SVGSwitchElement::SVGSwitchElement(const QualifiedName& tagName, Document* document)
    : SVGGraphicsElement(tagName, document, adoptPtr(new PropertyRegistry(*this)))
{
    ASSERT(hasTagName(SVGNames::switchTag));
}

PassRefPtr<SVGSwitchElement> SVGSwitchElement::create(const QualifiedName& tagName, Document* document)
{
    return adoptRef(new SVGSwitchElement(tagName, document));
}

bool SVGSwitchElement::childShouldCreateRenderer(const Node* child) const
{
    // We create a renderer for the first valid SVG element child.
    // FIXME: The renderer must be updated after dynamic change of the requiredFeatures, requiredExtensions and systemLanguage attributes (https://bugs.webkit.org/show_bug.cgi?id=74749).
	auto children = childrenOfType<SVGElement>(this);
    for (auto it = children.begin(); it != children.end(); ++it) {
        const SVGElement& element = *it;
        if (!element.isValid())
            continue;
        return &element == child; // Only allow this child if it's the first valid child
    }

    return false;
}

RenderPtr<RenderElement> SVGSwitchElement::createElementRenderer(PassOwnPtr<RenderStyle> style, const RenderTreePosition&)
{
    return createRenderer<RenderSVGTransformableContainer>(*this, style);
}

}

#endif // ENABLE(SVG)
