/*
 * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#if ENABLE(SVG)
#include "SVGFitToViewBox.h"
#include "SVGGraphicsElement.h"

namespace WebCore {

class SVGSymbolElement FINAL : public SVGGraphicsElement, public SVGFitToViewBox {
public:
    static PassRefPtr<SVGSymbolElement> create(const QualifiedName&, Document*);

private:
    SVGSymbolElement(const QualifiedName&, Document*);

    using PropertyRegistry = SVGPropertyOwnerRegistry<SVGSymbolElement, SVGGraphicsElement, SVGFitToViewBox>;

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    virtual RenderPtr<RenderElement> createElementRenderer(PassOwnPtr<RenderStyle>, const RenderTreePosition&) override;

    virtual bool selfHasRelativeLengths() const override;
    bool supportsFocus() const final { return false; }
};

inline bool isSVGSymbolElement(const Node& node)
{
    return node.hasTagName(SVGNames::symbolTag);
}

inline bool isSVGSymbolElement(const Element& element)
{
    return element.hasTagName(SVGNames::symbolTag);
}

template <typename ArgType>
struct ElementTypeCastTraits<const SVGSymbolElement, ArgType> {
    static bool is(ArgType& node) { return isSVGSymbolElement(node); }
};

} // namespace WebCore

#endif // ENABLE(SVG)
