/*
 * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
 * Copyright (C) 2016-2017 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"
#if ENABLE(SVG)
#include "SVGTitleElement.h"

#include "Document.h"
#include "SVGNames.h"

namespace WebCore {

inline SVGTitleElement::SVGTitleElement(const QualifiedName& tagName, Document* document)
    : SVGElement(tagName, document, adoptPtr(new PropertyRegistry(*this)))
{
    ASSERT(hasTagName(SVGNames::titleTag));
}

Ref<SVGTitleElement> SVGTitleElement::create(const QualifiedName& tagName, Document* document)
{
    return adoptRef(*new SVGTitleElement(tagName, document));
}

Node::InsertedIntoResult SVGTitleElement::insertedIntoAncestor(const InsertionType& insertionType, ContainerNode& parentOfInsertedTree)
{
	auto result = SVGElement::insertedIntoAncestor(insertionType, parentOfInsertedTree);
	document().titleElementAdded(*this);
    return result;
}

void SVGTitleElement::removedFromAncestor(const RemovalType& removalType, ContainerNode& parentOfRemovedTree)
{
    SVGElement::removedFromAncestor(removalType, parentOfRemovedTree);
	document().titleElementRemoved(*this);
}

void SVGTitleElement::childrenChanged(const ChildChange& change)
{
    SVGElement::childrenChanged(change);
    document().titleElementTextChanged(*this);
}

}

#endif // ENABLE(SVG)
