/*
 * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef SVGTitleElement_h
#define SVGTitleElement_h
#if ENABLE(SVG)

#include "SVGElement.h"
#include "SVGNames.h"

namespace WebCore {

class SVGTitleElement FINAL : public SVGElement {
public:
    static Ref<SVGTitleElement> create(const QualifiedName&, Document*);

private:
    SVGTitleElement(const QualifiedName&, Document*);

    virtual InsertedIntoResult insertedIntoAncestor(const InsertionType&, ContainerNode&) OVERRIDE;
    virtual void removedFromAncestor(const RemovalType&, ContainerNode&) OVERRIDE;
    virtual void childrenChanged(const ChildChange&) OVERRIDE;

    virtual bool rendererIsNeeded(const RenderStyle&) { return false; }
    bool supportsFocus() const final { return false; }
};

inline bool isSVGTitleElement(const Node& node)
{
    return node.hasTagName(SVGNames::titleTag);
}

inline bool isSVGTitleElement(const Element& element)
{
    return element.hasTagName(SVGNames::titleTag);
}

inline SVGTitleElement* toSVGTitleElement(Node* node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!node || isSVGTitleElement(node));
    return static_cast<SVGTitleElement*>(node);
}

inline SVGTitleElement& toSVGTitleElement(Node& node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(isSVGTitleElement(node));
    return static_cast<SVGTitleElement&>(node);
}

template <typename ArgType>
struct ElementTypeCastTraits<const SVGTitleElement, ArgType> {
    static bool is(ArgType& node) { return isSVGTitleElement(node); }
};

} // namespace WebCore

#endif // ENABLE(SVG)
#endif

// vim:ts=4:noet
