/*
 * Copyright (C) 2004, 2005, 2008 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2007 Rob Buis <buis@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#pragma once

#include "SVGElement.h"
#include "SVGFitToViewBox.h"
#include "SVGSVGElement.h"
#include "SVGStringList.h"
#include "SVGZoomAndPan.h"

namespace WebCore {

class SVGViewElement FINAL : public SVGElement, public SVGFitToViewBox, public SVGZoomAndPan {
public:
    static PassRefPtr<SVGViewElement> create(const QualifiedName&, Document*);

    using SVGElement::ref;
    using SVGElement::deref;

    Ref<SVGStringList> viewTarget() { return m_viewTarget.copyRef(); }

    const SVGSVGElement* targetElement() const { return m_targetElement.get(); }
    void setTargetElement(const SVGSVGElement& targetElement) { m_targetElement = makeWeakPtr(targetElement); }
    void resetTargetElement() { m_targetElement = nullptr; }

private:
    SVGViewElement(const QualifiedName&, Document*);

    using PropertyRegistry = SVGPropertyOwnerRegistry<SVGViewElement, SVGElement, SVGFitToViewBox>;

    virtual void attributeChanged(const QualifiedName&, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason) override;
    void svgAttributeChanged(const QualifiedName&) override;

    virtual bool rendererIsNeeded(const RenderStyle&) override { return false; }

    Ref<SVGStringList> m_viewTarget { SVGStringList::create(this) };
    WeakPtr<SVGSVGElement> m_targetElement;
};

inline bool isSVGViewElement(const Node& node)
{
    return node.hasTagName(SVGNames::viewTag);
}

inline bool isSVGViewElement(const Element& element)
{
    return element.hasTagName(SVGNames::viewTag);
}

NODE_TYPE_CASTS(SVGViewElement)


} // namespace WebCore
