/*
 * Copyright (C) 2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"
#include "SVGResourceImage.h"

#include "ReferencedSVGResources.h"
#include "RenderSVGResourceMasker.h"

namespace WebCore {

Ref<SVGResourceImage> SVGResourceImage::create(RenderSVGResourceContainer& renderResource, const URL& reresolvedURL)
{
    return adoptRef(*new SVGResourceImage(renderResource, reresolvedURL));
}

SVGResourceImage::SVGResourceImage(RenderSVGResourceContainer& renderResource, const URL& reresolvedURL)
    : m_renderResource(makeWeakPtr(renderResource))
    , m_reresolvedURL(reresolvedURL)
{
}

void SVGResourceImage::draw(GraphicsContext* context, const FloatRect& destinationRect, const FloatRect& sourceRect, CompositeOperator compositeOp, BlendMode blendMode)
{
    if (isRenderSVGResourceMasker(m_renderResource.get())) {
        if (toRenderSVGResourceMasker(*m_renderResource).drawContentIntoContext(*context, destinationRect, sourceRect, compositeOp, blendMode))
            return;
    }
}

void SVGResourceImage::drawPattern(GraphicsContext* context, const FloatRect& srcRect, const AffineTransform& patternTransform, const FloatPoint& phase, const FloatSize& spacing, CompositeOperator compositeOp, const FloatRect& dstRect, BlendMode blendMode)
{
	OwnPtr<ImageBuffer> imageBuffer = context->createImageBuffer(size());
    if (!imageBuffer)
        return;

    // Fill with the SVG resource.
    GraphicsContext& graphicsContext = *imageBuffer->context();
    graphicsContext.drawImage(this, FloatPoint());

    // Tile the image buffer into the context.
    imageBuffer->drawPattern(context, srcRect, patternTransform, phase, spacing, compositeOp, dstRect, blendMode);
}

#if 0
void SVGResourceImage::dump(WTF::TextStream& ts) const
{
    GeneratedImage::dump(ts);
    ts << m_reresolvedURL;
}
#endif

} // namespace WebCore
