/*
 * This file is part of the XSL implementation.
 *
 * Copyright (C) 2004, 2006, 2008 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#pragma once

#if ENABLE(XSLT)

#include "ResourceError.h"
#include "ResourceResponse.h"
#include "ThreadableLoaderClient.h"
#include "XSLStyleSheet.h"
#include <wtf/PassOwnPtr.h>
#include <wtf/PassRefPtr.h>
#include <wtf/RefPtr.h>
#include <wtf/WeakPtr.h>
#include <wtf/text/StringBuilder.h>

namespace WebCore {

class CachedXSLStyleSheet;
class TextResourceDecoder;

class XSLImportRule final : private ThreadableLoaderClient {
    WTF_MAKE_FAST_ALLOCATED;
public:
    static PassOwnPtr<XSLImportRule> create(XSLStyleSheet& parentSheet, const String& href)
    {
        return adoptPtr(new XSLImportRule(parentSheet, href));
    }

    virtual ~XSLImportRule();
    
    const String& href() const { return m_strHref; }
    XSLStyleSheet* styleSheet() const { return m_styleSheet.get(); }

    XSLStyleSheet* parentStyleSheet() const { return m_parentStyleSheet.get(); }
    void setParentStyleSheet(XSLStyleSheet* styleSheet) { m_parentStyleSheet = makeWeakPtr(styleSheet); }

    bool isLoading();
    void loadSheet();
    
private:
    XSLImportRule(XSLStyleSheet& parentSheet, const String& href);

    void setXSLStyleSheet(const String& href, const KURL& baseURL, const String& sheet);

    virtual void didReceiveResponse(unsigned long identifier, const ResourceResponse&) FINAL;
    virtual void didReceiveData(const char* data, int dataLength) FINAL;
    virtual void didFinishLoading(unsigned long identifier, double finishTime) FINAL;
    virtual void didFail(const ResourceError&) FINAL;

    WeakPtr<XSLStyleSheet> m_parentStyleSheet;
    String m_strHref;
    RefPtr<XSLStyleSheet> m_styleSheet;
    ResourceError m_resourceError;
    ResourceResponse m_resourceResponse;
    StringBuilder m_responseBuilder;
    RefPtr<TextResourceDecoder> m_decoder;
    bool m_loading { false };
};

} // namespace WebCore

#endif // ENABLE(XSLT)
